@@ -1,6 +1,6 @@
 // BreakControl.cc
 //
-// Copyright (C) 2001 - 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2012 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -140,7 +140,6 @@ BreakControl::heartbeat()
 
         TRACE_MSG("prelude time = " << prelude_time);
 
-
         update_prelude_window();
         application->refresh_break_window();
 
@@ -216,14 +215,14 @@ BreakControl::goto_stage(BreakStage stage)
         IActivityMonitor *monitor = core->get_activity_monitor();
         monitor->set_listener(this);
 
-        break_timer->set_insensitive_mode(INSENSITIVE_MODE_IDLE_ON_LIMIT_REACHED);
+        core->set_insensitive_mode_all_breaks(INSENSITIVE_MODE_IDLE_ON_LIMIT_REACHED);
       }
       break;
 
     case STAGE_NONE:
       {
         // Teminate the break.
-        break_timer->set_insensitive_mode(INSENSITIVE_MODE_IDLE_ON_LIMIT_REACHED);
+        core->set_insensitive_mode_all_breaks(INSENSITIVE_MODE_IDLE_ON_LIMIT_REACHED);
         application->hide_break_window();
         core->defrost();
 
@@ -236,7 +235,7 @@ BreakControl::goto_stage(BreakStage stage)
                 timer->force_active();
               }
           }
-        
+
         if (break_stage == STAGE_TAKING && !fake_break)
           {
             // Update statistics and play sound if the break end
@@ -282,7 +281,7 @@ BreakControl::goto_stage(BreakStage stage)
 
     case STAGE_PRELUDE:
       {
-        break_timer->set_insensitive_mode(INSENSITIVE_MODE_FOLLOW_IDLE);
+        core->set_insensitive_mode_all_breaks(INSENSITIVE_MODE_FOLLOW_IDLE);
         prelude_count++;
         postponable_count++;
         prelude_time = 0;
@@ -298,14 +297,15 @@ BreakControl::goto_stage(BreakStage stage)
       {
         // Break timer should always idle.
         // Previous revisions set MODE_IDLE_ON_LIMIT_REACHED
-        break_timer->set_insensitive_mode(INSENSITIVE_MODE_IDLE_ALWAYS);
+        // core->set_insensitive_mode_all_breaks(INSENSITIVE_MODE_IDLE_ALWAYS);
+        core->set_insensitive_mode_all_breaks(INSENSITIVE_MODE_FOLLOW_IDLE);
 
         // Remove the prelude window, if necessary.
         application->hide_break_window();
 
         // "Innocent until proven guilty".
         TRACE_MSG("Force idle");
-        core->force_idle();
+        core->force_idle(break_id);
         break_timer->stop_timer();
 
         // Start the break.
@@ -408,7 +408,7 @@ BreakControl::start_break()
 
       // Idle until proven guilty.
       TRACE_MSG("Force idle");
-      core->force_idle();
+      core->force_idle(break_id);
       break_timer->stop_timer();
 
       // Update statistics.
@@ -457,7 +457,7 @@ BreakControl::force_start_break(BreakHint hint)
   if (!break_timer->get_activity_sensitive())
     {
       TRACE_MSG("Forcing idle");
-      break_timer->force_idle();
+      core->force_idle(break_id);
     }
 
   goto_stage(STAGE_TAKING);
@@ -500,7 +500,7 @@ BreakControl::suspend_break()
   TRACE_ENTER_MSG("BreakControl::suspend_break", break_id);
 
   break_hint = BREAK_HINT_NONE;
-  
+
   goto_stage(STAGE_NONE);
 
   TRACE_EXIT();
@@ -634,7 +634,7 @@ BreakControl::break_window_start()
   application->create_break_window(break_id, break_hint);
   update_break_window();
   application->show_break_window();
-  
+
   TRACE_EXIT();
 }
 
@@ -660,7 +660,7 @@ BreakControl::prelude_window_start()
   update_prelude_window();
 
   application->show_break_window();
-  
+
   TRACE_EXIT();
 }
 
@@ -679,6 +679,7 @@ BreakControl::action_notify()
 void
 BreakControl::set_state_data(bool active, const BreakStateData &data)
 {
+  (void) active;
   TRACE_ENTER_MSG("BreakStateData::set_state_data", active);
 
   TRACE_MSG("forced = " << data.forced_break <<
@@ -765,20 +766,17 @@ BreakControl::send_signal(BreakStage stage)
           switch (break_id)
             {
             case BREAK_ID_MICRO_BREAK:
-              iface->MicrobreakChanged("/org/workrave/Workrave/Core",
-                                       progress);
+              iface->MicrobreakChanged("/org/workrave/Workrave/Core", progress);
               break;
-          
+
             case BREAK_ID_REST_BREAK:
-              iface->MicrobreakChanged("/org/workrave/Workrave/Core",
-                                       progress);
+              iface->RestbreakChanged("/org/workrave/Workrave/Core", progress);
               break;
-          
+
             case BREAK_ID_DAILY_LIMIT:
-              iface->MicrobreakChanged("/org/workrave/Workrave/Core",
-                                       progress);
+              iface->DailylimitChanged("/org/workrave/Workrave/Core", progress);
               break;
-          
+
             default:
               break;
             }
