@@ -1,6 +1,6 @@
 // Configurator.cc --- Configuration Access
 //
-// Copyright (C) 2002, 2003, 2006, 2007, 2008 Rob Caelers <robc@krandor.nl>
+// Copyright (C) 2002, 2003, 2006, 2007, 2008, 2012 Rob Caelers <robc@krandor.nl>
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -65,14 +65,20 @@ Configurator::load(std::string filename)
 bool
 Configurator::save(std::string filename)
 {
-  return backend->save(filename);
+  TRACE_ENTER_MSG("Configurator::save", filename);
+  bool ret = backend->save(filename);
+  TRACE_RETURN(ret);
+  return ret;
 }
 
 
 bool
 Configurator::save()
 {
-  return backend->save();
+  TRACE_ENTER("Configurator::save");
+  bool ret = backend->save();
+  TRACE_RETURN(ret);
+  return ret;
 }
 
 
@@ -91,16 +97,22 @@ Configurator::heartbeat()
 
       if (now >= delayed.until)
         {
+          Variant old_value;
+          bool old_value_valid = backend->get_value(delayed.key, delayed.value.type, old_value);
+
           bool b = backend->set_value(delayed.key, delayed.value);
 
           if (b && dynamic_cast<IConfigBackendMonitoring *>(backend) == NULL)
             {
-              fire_configurator_event(delayed.key);
-
-              if (auto_save_time == 0)
+              if (!old_value_valid || old_value != delayed.value)
                 {
-                  ICore *core = CoreFactory::get_core();
-                  auto_save_time = core->get_time() + 30;
+                  fire_configurator_event(delayed.key);
+
+                  if (auto_save_time == 0)
+                    {
+                      ICore *core = CoreFactory::get_core();
+                      auto_save_time = core->get_time() + 30;
+                    }
                 }
             }
 
@@ -110,7 +122,7 @@ Configurator::heartbeat()
       it = next;
     }
 
-  if (now >= auto_save_time)
+  if (auto_save_time != 0 && now >= auto_save_time)
     {
       save();
       auto_save_time = 0;
@@ -149,7 +161,7 @@ Configurator::rename_key(const std::string &key, const std::string &new_key)
 {
   bool ok = false;
   Variant value;
-  
+
   bool exists = get_value(new_key, VARIANT_TYPE_NONE, value);
   if (!exists)
     {
@@ -211,19 +223,25 @@ Configurator::set_value(const std::string &key, Variant &value, ConfigFlags flag
             }
         }
     }
-  
+
   if (!skip)
     {
+      Variant old_value;
+      bool old_value_valid = backend->get_value(newkey, value.type, old_value);
+
       ret = backend->set_value(newkey, value);
-      
+
       if (ret && dynamic_cast<IConfigBackendMonitoring *>(backend) == NULL)
         {
-          fire_configurator_event(newkey);
-          
-          if (auto_save_time == 0)
+          if (!old_value_valid || old_value != value)
             {
-              ICore *core = CoreFactory::get_core();
-              auto_save_time = core->get_time() + 30;
+              fire_configurator_event(newkey);
+
+              if (auto_save_time == 0)
+                {
+                  ICore *core = CoreFactory::get_core();
+                  auto_save_time = core->get_time() + 30;
+                }
             }
         }
     }
@@ -240,7 +258,7 @@ Configurator::get_value(const std::string &key, VariantType type, Variant &out)
   Setting setting;
 
   TRACE_ENTER_MSG("Configurator::get_value", key);
-  
+
   string newkey = key;
   strip_trailing_slash(newkey);
   strip_leading_slash(newkey);
@@ -335,11 +353,11 @@ Configurator::set_value(const std::string &key, const std::string &v, ConfigFlag
 {
   Variant value;
   bool ret = false;
-  
+
   value.type = VARIANT_TYPE_STRING;
   value.string_value = v;
   ret = set_value(key, value, flags);
-  
+
   return ret;
 }
 
@@ -349,7 +367,7 @@ Configurator::set_value(const std::string &key, const char *v, ConfigFlags flags
 {
   Variant value;
   bool ret = false;
-  
+
   value.type = VARIANT_TYPE_STRING;
   value.string_value = v;
   ret = set_value(key, value, flags);
@@ -362,7 +380,7 @@ Configurator::set_value(const std::string &key, int v, ConfigFlags flags)
 {
   Variant value;
   bool ret = false;
-  
+
   value.type = VARIANT_TYPE_INT;
   value.int_value = v;
   ret = set_value(key, value, flags);
@@ -376,7 +394,7 @@ Configurator::set_value(const std::string &key, bool v, ConfigFlags flags)
 {
   Variant value;
   bool ret = false;
-  
+
   value.type = VARIANT_TYPE_BOOL;
   value.bool_value = v;
   ret = set_value(key, value, flags);
@@ -390,7 +408,7 @@ Configurator::set_value(const std::string &key, double v, ConfigFlags flags)
 {
   Variant value;
   bool ret = false;
-  
+
   value.type = VARIANT_TYPE_DOUBLE;
   value.double_value = v;
   ret = set_value(key, value, flags);
