@@ -1,6 +1,6 @@
 // Core.cc --- The main controller
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -89,11 +89,11 @@ Core::Core() :
   application(NULL),
   statistics(NULL),
   operation_mode(OPERATION_MODE_NORMAL),
+  operation_mode_regular(OPERATION_MODE_NORMAL),
   usage_mode(USAGE_MODE_NORMAL),
   core_event_listener(NULL),
   powersave(false),
   powersave_resume_time(0),
-  powersave_operation_mode(OPERATION_MODE_NORMAL),
   insist_policy(ICore::INSIST_POLICY_HALT),
   active_insist_policy(ICore::INSIST_POLICY_INVALID),
   resume_break(BREAK_ID_NONE),
@@ -195,40 +195,42 @@ Core::init_configurator()
     }
   else
     {
-#if defined(PLATFORM_OS_WIN32) || defined(PLATFORM_OS_OSX)
-      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatNative);
-
-#elif defined(HAVE_GCONF)
+#if defined(HAVE_GCONF)
       gconf_init(argc, argv, NULL);
       g_type_init();
+#endif
+      
       configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatNative);
-
-#elif defined(HAVE_GDOME)
-      string configFile = Util::complete_directory("config.xml", Util::SEARCH_PATH_CONFIG);
-      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatXml);
+#if defined(HAVE_GDOME)
+      if (configurator == NULL)
+        {
+          string configFile = Util::complete_directory("config.xml", Util::SEARCH_PATH_CONFIG);
+          configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatXml);
 
 #  if defined(PLATFORM_OS_UNIX)
-      if (configFile == "" || configFile == "config.xml")
-        {
-          configFile = Util::get_home_directory() + "config.xml";
-        }
+          if (configFile == "" || configFile == "config.xml")
+            {
+              configFile = Util::get_home_directory() + "config.xml";
+            }
 #  endif
-      if (configFile != "")
-        {
-          configurator->load(configFile);
+          if (configFile != "")
+            {
+              configurator->load(configFile);
+            }
         }
-#elif defined(HAVE_QT)
-      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatNative);
-#else
-      ini_file = Util::get_home_directory() + "workrave.ini";
-      configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatIni);
-      configurator->load(ini_file);
-      configurator->save(ini_file);
 #endif
+      if (configurator == NULL)
+        {
+          ini_file = Util::get_home_directory() + "workrave.ini";
+          configurator = ConfiguratorFactory::create(ConfiguratorFactory::FormatIni);
+          configurator->load(ini_file);
+          configurator->save(ini_file);
+        }
     }
-
+  
   string home;
-  if (configurator->get_value(CoreConfig::CFG_KEY_GENERAL_DATADIR, home))
+  if (configurator->get_value(CoreConfig::CFG_KEY_GENERAL_DATADIR, home) &&
+      home != "")
     {
       Util::set_home_directory(home);
     }
@@ -242,32 +244,18 @@ Core::init_bus()
   try
     {
       dbus = new DBus();
-
       dbus->init();
 
-      string name = DBUS_SERVICE_WORKRAVE;
-      char *env = getenv("WORKRAVE_DBUS_NAME");
-      if (env != NULL)
-        {
-          name = env;
-        }      
-      dbus->register_service(name);
-      dbus->register_object_path(DBUS_PATH_WORKRAVE);
-
       extern void init_DBusWorkrave(DBus *dbus);
       init_DBusWorkrave(dbus);
-  
-      dbus->connect(DBUS_PATH_WORKRAVE, "org.workrave.CoreInterface", this);
-      
-      dbus->connect(DBUS_PATH_WORKRAVE,
-                    "org.workrave.ConfigInterface",
-                    configurator);
 
+      dbus->connect(DBUS_PATH_WORKRAVE, "org.workrave.CoreInterface", this);
+      dbus->connect(DBUS_PATH_WORKRAVE, "org.workrave.ConfigInterface", configurator);
+      dbus->register_object_path(DBUS_PATH_WORKRAVE);
+      
 #ifdef HAVE_TESTS
+      dbus->connect("/org/workrave/Workrave/Debug", "org.workrave.DebugInterface", Test::get_instance());
       dbus->register_object_path("/org/workrave/Workrave/Debug");
-      dbus->connect("/org/workrave/Workrave/Debug",
-                    "org.workrave.DebugInterface",
-                    Test::get_instance());
 #endif
     }
   catch (DBusException &)
@@ -293,7 +281,7 @@ Core::init_monitor(const string &display_name)
 #endif
 
   InputMonitorFactory::init(display_name);
-  
+
   monitor = new ActivityMonitor();
   load_monitor_config();
 
@@ -416,8 +404,8 @@ Core::config_changed_notify(const string &key)
         {
           mode = OPERATION_MODE_NORMAL;
         }
-      TRACE_MSG("Setting mode");
-      set_operation_mode(OperationMode(mode), false);
+      TRACE_MSG("Setting operation mode");
+      set_operation_mode_internal(OperationMode(mode), false);
     }
 
   if (key == CoreConfig::CFG_KEY_USAGE_MODE)
@@ -427,7 +415,8 @@ Core::config_changed_notify(const string &key)
         {
           mode = USAGE_MODE_NORMAL;
         }
-      set_usage_mode(UsageMode(mode), false);
+      TRACE_MSG("Setting usage mode");
+      set_usage_mode_internal(UsageMode(mode), false);
     }
   TRACE_EXIT();
 }
@@ -511,6 +500,7 @@ Core::get_break(BreakId id)
   return &breaks[id];
 }
 
+//! Returns the specified break controller.
 Break *
 Core::get_break(std::string name)
 {
@@ -538,85 +528,256 @@ Core::get_distribution_manager() const
 OperationMode
 Core::get_operation_mode()
 {
-  return operation_mode;
+    return operation_mode;
 }
 
-
-//! Sets the operation mode
+//! Retrieves the regular operation mode.
 OperationMode
-Core::set_operation_mode(OperationMode mode, bool persistent)
+Core::get_operation_mode_regular()
 {
-  TRACE_ENTER_MSG("Core::set_operation_mode",
-                  (mode == OPERATION_MODE_NORMAL ? "normal" :
-                   mode == OPERATION_MODE_SUSPENDED ? "suspended" :
-                   mode == OPERATION_MODE_QUIET ? "quiet" : "???") << " " << persistent);
-  
-  OperationMode previous_mode = operation_mode;
+    /* operation_mode_regular is the same as operation_mode unless there's an 
+    override in place, in which case operation_mode is the current override mode and 
+    operation_mode_regular is the mode that will be restored once all overrides are removed
+    */
+    return operation_mode_regular;
+}
 
-  TRACE_MSG("Previous " << (mode == OPERATION_MODE_NORMAL ? "normal" :
-                            mode == OPERATION_MODE_SUSPENDED ? "suspended" :
-                            mode == OPERATION_MODE_QUIET ? "quiet" : "???"));
-  
-  if (operation_mode != mode)
+//! Checks if operation_mode is an override.
+bool
+Core::is_operation_mode_an_override()
+{
+    return !!operation_mode_overrides.size();
+}
+
+//! Sets the operation mode.
+void
+Core::set_operation_mode(OperationMode mode)
+{
+    set_operation_mode_internal( mode, true );
+}
+
+//! Temporarily overrides the operation mode.
+void
+Core::set_operation_mode_override( OperationMode mode, const std::string &id )
+{
+    if( !id.size() )
+        return;
+
+    set_operation_mode_internal( mode, false, id );
+}
+
+//! Removes the overriden operation mode.
+void
+Core::remove_operation_mode_override( const std::string &id )
+{
+    TRACE_ENTER( "Core::remove_operation_mode_override" );
+
+    if( !id.size() || !operation_mode_overrides.count( id ) )
+        return;
+
+    operation_mode_overrides.erase( id );
+
+    /* If there are other overrides still in the queue then pass in the first 
+    override in the map. set_operation_mode_internal() will then search the 
+    map for the most important override and set it as the active operation mode.
+    */
+    if( operation_mode_overrides.size() )
     {
-      operation_mode = mode;
-      
-      if (operation_mode == OPERATION_MODE_SUSPENDED)
+        set_operation_mode_internal( 
+            operation_mode_overrides.begin()->second, 
+            false, 
+            operation_mode_overrides.begin()->first
+            );
+    }
+    else
+    {
+        /* if operation_mode_regular is the same as the active operation mode then just 
+        signal the mode has changed. During overrides the signal is not sent so it needs to 
+        be sent now. Because the modes are the same it would not be called by 
+        set_operation_mode_internal().
+        */
+        if( operation_mode_regular == operation_mode )
         {
+            TRACE_MSG( "Only calling core_event_operation_mode_changed()." );
+            if( core_event_listener )
+                core_event_listener->core_event_operation_mode_changed( operation_mode_regular );
+        }
+        else
+            set_operation_mode_internal( operation_mode_regular, false );
+    }
+
+    TRACE_EXIT();
+}
+
+//! Set the operation mode.
+void
+Core::set_operation_mode_internal(
+    OperationMode mode, 
+    bool persistent, 
+    const std::string &override_id /* default param: empty string */
+    )
+{
+  TRACE_ENTER_MSG("Core::set_operation_mode", ( persistent ? "persistent" : "" ) );
+
+  if( override_id.size() )
+  {
+      TRACE_MSG( "override_id: " << override_id );
+  }
+
+  TRACE_MSG( "Incoming/requested mode is "
+      << ( mode == OPERATION_MODE_NORMAL ? "OPERATION_MODE_NORMAL" :
+            mode == OPERATION_MODE_SUSPENDED ? "OPERATION_MODE_SUSPENDED" :
+                mode == OPERATION_MODE_QUIET ? "OPERATION_MODE_QUIET" : "???" )
+      << ( override_id.size() ? " (override)" : " (regular)" )
+      );
+
+  TRACE_MSG( "Current mode is "
+      << ( mode == OPERATION_MODE_NORMAL ? "OPERATION_MODE_NORMAL" :
+            mode == OPERATION_MODE_SUSPENDED ? "OPERATION_MODE_SUSPENDED" :
+                mode == OPERATION_MODE_QUIET ? "OPERATION_MODE_QUIET" : "???" )
+      << ( operation_mode_overrides.size() ? " (override)" : " (regular)" )
+      );
+  
+  if( ( mode != OPERATION_MODE_NORMAL )
+      && ( mode != OPERATION_MODE_QUIET )
+      && ( mode != OPERATION_MODE_SUSPENDED )
+      )
+  {
+      TRACE_RETURN( "No change: incoming invalid" );
+      return;
+  }
+
+  /* If the incoming operation mode is regular and the current operation mode is an 
+  override then save the incoming operation mode and return.
+  */
+  if( !override_id.size() && operation_mode_overrides.size() )
+  {
+      operation_mode_regular = mode;
+
+      int cm;
+      if( persistent 
+          && ( !get_configurator()->get_value( CoreConfig::CFG_KEY_OPERATION_MODE, cm ) 
+              || ( cm != mode ) )
+          )
+          get_configurator()->set_value( CoreConfig::CFG_KEY_OPERATION_MODE, mode );
+
+      TRACE_RETURN( "No change: current is an override type but incoming is regular" );
+      return;
+  }
+  
+  // If the incoming operation mode is tagged as an override
+  if( override_id.size() )
+  {
+      // Add this override to the map
+      operation_mode_overrides[ override_id ] = mode;
+
+      /* Find the most important override. Override modes in order of importance:
+      OPERATION_MODE_SUSPENDED, OPERATION_MODE_QUIET, OPERATION_MODE_NORMAL
+      */
+      for( map<std::string, OperationMode>::iterator i = operation_mode_overrides.begin();
+          ( i != operation_mode_overrides.end() );
+          ++i
+          )
+      {
+          if( i->second == OPERATION_MODE_SUSPENDED )
+          {
+              mode = OPERATION_MODE_SUSPENDED;
+              break;
+          }
+
+          if( ( i->second == OPERATION_MODE_QUIET )
+              && ( mode == OPERATION_MODE_NORMAL )
+              )
+          {
+              mode = OPERATION_MODE_QUIET;
+          }
+      }
+  }
+
+
+  if (operation_mode != mode)
+  {
+      TRACE_MSG( "Changing active operation mode to "
+          << ( mode == OPERATION_MODE_NORMAL ? "OPERATION_MODE_NORMAL" :
+                mode == OPERATION_MODE_SUSPENDED ? "OPERATION_MODE_SUSPENDED" :
+                    mode == OPERATION_MODE_QUIET ? "OPERATION_MODE_QUIET" : "???" )
+          );
+
+      OperationMode previous_mode = operation_mode;
+
+      operation_mode = mode;
+
+      if( !operation_mode_overrides.size() )
+          operation_mode_regular = operation_mode;
+
+      if (operation_mode == OPERATION_MODE_SUSPENDED)
+      {
           TRACE_MSG("Force idle");
           force_idle();
           monitor->suspend();
           stop_all_breaks();
-          
+
           for (int i = 0; i < BREAK_ID_SIZEOF; ++i)
-            {
+          {
               if (breaks[i].is_enabled())
-                {
+              {
                   breaks[i].get_timer()->set_insensitive_mode(INSENSITIVE_MODE_IDLE_ALWAYS);
-                }
-            }
-        }
+              }
+          }
+      }
       else if (previous_mode == OPERATION_MODE_SUSPENDED)
-        {
+      {
           // stop_all_breaks again will reset insensitive mode (that is good)
           stop_all_breaks();
           monitor->resume();
-        }
-      
+      }
+
       if (operation_mode == OPERATION_MODE_QUIET)
-        {
+      {
           stop_all_breaks();
-        }
+      }
 
-      if (persistent)
-        {
-          get_configurator()->set_value(CoreConfig::CFG_KEY_OPERATION_MODE, mode);
-        }
-      
-      if (core_event_listener != NULL)
-        {
-          core_event_listener->core_event_operation_mode_changed(mode);
-        }
-    }
+      if( !operation_mode_overrides.size() )
+      {
+          /* The two functions in this block will trigger signals that can call back into this function.
+          Only if there are no overrides in place will that reentrancy be ok from here.
+          Otherwise the regular/user mode to restore would be overwritten.
+          */
 
-  TRACE_RETURN(previous_mode);
-  return previous_mode;
+          if( persistent )
+              get_configurator()->set_value( CoreConfig::CFG_KEY_OPERATION_MODE, operation_mode );
+
+          if( core_event_listener )
+              core_event_listener->core_event_operation_mode_changed( operation_mode );
+      }
+  }
+
+  TRACE_EXIT();
 }
 
 
+//! Retrieves the usage mode.
 UsageMode
 Core::get_usage_mode()
 {
   return usage_mode;
 }
 
+//! Sets the usage mode.
 void
-Core::set_usage_mode(UsageMode mode, bool persistent)
+Core::set_usage_mode(UsageMode mode)
+{
+  set_usage_mode_internal(mode, true);
+}
+
+//! Sets the usage mode.
+void
+Core::set_usage_mode_internal(UsageMode mode, bool persistent)
 {
   if (usage_mode != mode)
     {
       usage_mode = mode;
-  
+
       for (int i = 0; i < BREAK_ID_SIZEOF; i++)
         {
           breaks[i].set_usage_mode(mode);
@@ -626,7 +787,12 @@ Core::set_usage_mode(UsageMode mode, bool persistent)
         {
           get_configurator()->set_value(CoreConfig::CFG_KEY_USAGE_MODE, mode);
         }
-    }      
+
+      if (core_event_listener != NULL)
+        {
+          core_event_listener->core_event_usage_mode_changed(mode);
+        }
+    }
 }
 
 //! Sets the listener for core events.
@@ -685,7 +851,7 @@ Core::time_changed()
     {
       breaks[i].get_timer()->shift_time(0);
     }
-  
+
   TRACE_EXIT();
 }
 
@@ -695,14 +861,18 @@ void
 Core::set_powersave(bool down)
 {
   TRACE_ENTER_MSG("Core::set_powersave", down);
+  TRACE_MSG(powersave << " " << powersave_resume_time << " " << operation_mode);
+
   if (down)
     {
-      // Computer is going down
-      powersave_operation_mode = set_operation_mode(OPERATION_MODE_SUSPENDED, false);
-      TRACE_MSG("prev mode " << powersave_operation_mode);
-      powersave_resume_time = 0;
-      powersave = true;
-
+      if (!powersave)
+        {
+          // Computer is going down
+          set_operation_mode_override( OPERATION_MODE_SUSPENDED, "powersave" );
+          powersave_resume_time = 0;
+          powersave = true;
+        }
+      
       save_state();
       statistics->update();
     }
@@ -718,7 +888,7 @@ Core::set_powersave(bool down)
         }
 
       TRACE_MSG("resume time " << powersave_resume_time);
-      set_operation_mode(powersave_operation_mode, false);
+      remove_operation_mode_override( "powersave" );
     }
   TRACE_EXIT();
 }
@@ -764,16 +934,29 @@ void
 Core::force_idle()
 {
   TRACE_ENTER("Core::force_idle");
-  monitor->force_idle();
+  force_idle(BREAK_ID_NONE);
+  TRACE_EXIT();
+}
 
+
+void
+Core::force_idle(BreakId break_id)
+{
+  TRACE_ENTER("Core::force_idle_for_break");
+  
+  monitor->force_idle();
+  
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
-      IActivityMonitor *am = breaks[i].get_timer()->get_activity_monitor();
-      if (am != NULL)
+      if (break_id == BREAK_ID_NONE || i == break_id)
         {
-          am->force_idle();
+          IActivityMonitor *am = breaks[i].get_timer()->get_activity_monitor();
+          if (am != NULL)
+            {
+              am->force_idle();
+            }
         }
-
+      
       breaks[i].get_timer()->force_idle();
     }
   TRACE_EXIT();
@@ -781,7 +964,7 @@ Core::force_idle()
 
 
 /********************************************************************************/
-/**** Break Response                                                        ******/
+/**** Break Response                                                       ******/
 /********************************************************************************/
 
 //! User postpones the specified break.
@@ -889,7 +1072,7 @@ Core::heartbeat()
       // Perform state computation.
       process_state();
     }
-  
+
   // Perform timer processing.
   process_timers();
 
@@ -1078,21 +1261,34 @@ Core::process_timers()
 
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
+      Timer *timer = breaks[i].get_timer();
+      
       infos[i].enabled = breaks[i].is_enabled();
       if (infos[i].enabled)
         {
-          breaks[i].get_timer()->enable();
+          timer->enable();
+          if (i == BREAK_ID_DAILY_LIMIT)
+            {
+              timer->set_limit_enabled(timer->get_limit() > 0);
+            }
         }
       else
         {
-          breaks[i].get_timer()->disable();
+          if (i != BREAK_ID_DAILY_LIMIT)
+            {
+              timer->disable();
+            }
+          else
+            {
+              timer->set_limit_enabled(false);
+            }
         }
 
       // First process only timer that do not have their
       // own activity monitor.
-      if (!(breaks[i].get_timer()->has_activity_monitor()))
+      if (!(timer->has_activity_monitor()))
         {
-          breaks[i].get_timer()->process(monitor_state, infos[i]);
+          timer->process(monitor_state, infos[i]);
         }
     }
 
@@ -1105,7 +1301,7 @@ Core::process_timers()
         }
     }
 
-  
+
   // Process all timer events.
   for (int i = BREAK_ID_SIZEOF - 1; i >= 0;  i--)
     {
@@ -1136,13 +1332,16 @@ bool
 Core::process_timewarp()
 {
   bool ret = false;
-  
+
   TRACE_ENTER("Core::process_timewarp");
   if (last_process_time != 0)
     {
       time_t gap = current_time - 1 - last_process_time;
+  
       if (abs((int)gap) > 5)
         {
+          TRACE_MSG("gap " << gap << " " << powersave << " " << operation_mode << " " << powersave_resume_time << " " << current_time);
+
           if (!powersave)
             {
               TRACE_MSG("Time warp of " << gap << " seconds. Correcting");
@@ -1164,9 +1363,10 @@ Core::process_timewarp()
 
               // In case the windows message was lost. some people reported that
               // workrave never restarted the timers...
-              set_operation_mode(powersave_operation_mode);
+              remove_operation_mode_override( "powersave" );
             }
         }
+      
       if (powersave && powersave_resume_time != 0 && current_time > powersave_resume_time + 30)
         {
           TRACE_MSG("End of time warp after powersave");
@@ -1186,7 +1386,7 @@ bool
 Core::process_timewarp()
 {
   bool ret = false;
-  
+
   TRACE_ENTER("Core::process_timewarp");
   if (last_process_time != 0)
     {
@@ -1195,21 +1395,21 @@ Core::process_timewarp()
       if (gap >= 30)
         {
           TRACE_MSG("Time warp of " << gap << " seconds. Powersafe");
-          
+
           force_idle();
 
           int save_current_time = current_time;
 
           current_time = last_process_time + 1;
           monitor_state = ACTIVITY_IDLE;
-          
+
           process_timers();
 
           current_time = save_current_time;
           ret = true;
         }
     }
-  
+
   TRACE_EXIT();
   return ret;
 }
@@ -1248,7 +1448,7 @@ Core::timer_action(BreakId id, TimerInfo info)
     case TIMER_EVENT_NATURAL_RESET:
       statistics->increment_break_counter(id, Statistics::STATS_BREAKVALUE_NATURAL_TAKEN);
       // FALLTHROUGH
-      
+
     case TIMER_EVENT_RESET:
       if (breaker->get_break_state() == BreakControl::BREAK_ACTIVE)
         {
@@ -1279,14 +1479,11 @@ Core::start_break(BreakId break_id, BreakId resume_this_break)
     }
 
   // Advance restbreak if it follows within 30s after the end of a microbreak
-  BreakControl *restbreak_control;
-  restbreak_control = breaks[BREAK_ID_REST_BREAK].get_break_control();
-
   if (break_id == BREAK_ID_REST_BREAK && resume_this_break == BREAK_ID_NONE)
     {
       breaks[BREAK_ID_REST_BREAK].override(BREAK_ID_REST_BREAK);
     }
-  
+
   if (break_id == BREAK_ID_MICRO_BREAK && breaks[BREAK_ID_REST_BREAK].is_enabled())
     {
       Timer *rb_timer = breaks[BREAK_ID_REST_BREAK].get_timer();
@@ -1308,7 +1505,7 @@ Core::start_break(BreakId break_id, BreakId resume_this_break)
           if (now + duration + 30 >= rb_timer->get_next_limit_time())
             {
               breaks[BREAK_ID_REST_BREAK].override(BREAK_ID_MICRO_BREAK);
-                
+
               start_break(BREAK_ID_REST_BREAK, BREAK_ID_MICRO_BREAK);
 
               // Snooze timer before the limit was reached. Just to make sure
@@ -1353,6 +1550,17 @@ Core::set_freeze_all_breaks(bool freeze)
     }
 }
 
+void
+Core::set_insensitive_mode_all_breaks(InsensitiveMode mode)
+{
+  for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+    {
+      Timer *t = breaks[i].get_timer();
+      assert(t != NULL);
+      t->set_insensitive_mode(mode);
+    }
+}
+
 
 //! Stops all breaks.
 void
@@ -1389,11 +1597,13 @@ Core::daily_reset()
       t->daily_reset_timer();
     }
 
-  
+
 #ifdef HAVE_DISTRIBUTION
   idlelog_manager->reset();
 #endif
 
+  save_state();
+
   TRACE_EXIT();
 }
 
@@ -1426,21 +1636,22 @@ Core::save_state() const
 void
 Core::load_misc()
 {
+  configurator->rename_key("gui/operation-mode", CoreConfig::CFG_KEY_OPERATION_MODE);
   configurator->add_listener(CoreConfig::CFG_KEY_OPERATION_MODE, this);
   configurator->add_listener(CoreConfig::CFG_KEY_USAGE_MODE, this);
-  
+
   int mode;
   if (! get_configurator()->get_value(CoreConfig::CFG_KEY_OPERATION_MODE, mode))
     {
       mode = OPERATION_MODE_NORMAL;
     }
-  set_operation_mode(OperationMode(mode), false);
+  set_operation_mode(OperationMode(mode));
 
   if (! get_configurator()->get_value(CoreConfig::CFG_KEY_USAGE_MODE, mode))
     {
       mode = USAGE_MODE_NORMAL;
     }
-  set_usage_mode(UsageMode(mode), false);
+  set_usage_mode(UsageMode(mode));
 }
 
 
@@ -1638,7 +1849,7 @@ Core::client_message(DistributionClientMessageID id, bool master, const char *cl
   bool ret = false;
 
   (void) client_id;
-  
+
   switch (id)
     {
     case DCM_BREAKS:
@@ -1838,6 +2049,7 @@ Core::set_timer_state(PacketBuffer &buffer)
 bool
 Core::set_monitor_state(bool master, PacketBuffer &buffer)
 {
+  (void) master;
   TRACE_ENTER_MSG("Core::set_monitor_state", master << " " << master_node);
 
   if (!master_node)
@@ -2010,17 +2222,17 @@ namespace workrave
   std::string operator%(const string &key, BreakId id)
   {
     IBreak *b = Core::get_instance()->get_break(id);
-  
+
     string str = key;
     string::size_type pos = 0;
     string name = b->get_name();
-  
+
     while ((pos = str.find("%b", pos)) != string::npos)
       {
         str.replace(pos, 2, name);
         pos++;
       }
-  
+
     return str;
   }
 }
