@@ -93,7 +93,7 @@ Statistics::init(Core *control)
     {
       input_monitor->subscribe_statistics(this);
     }
-  
+
 #ifdef HAVE_DISTRIBUTION
   init_distribution_manager();
 #endif
@@ -135,6 +135,43 @@ Statistics::update()
 }
 
 
+bool 
+Statistics::delete_all_history()
+{
+    update();
+
+    string histfile = Util::get_home_directory() + "historystats";
+    if( Util::file_exists( histfile.c_str() ) && std::remove( histfile.c_str() ) )
+    {
+        return false;
+    }
+    else
+    {
+        for( vector<DailyStatsImpl *>::iterator i = history.begin(); ( i != history.end() ); delete *i++ )
+            ;
+
+        history.clear();
+    }
+
+    string todayfile = Util::get_home_directory() + "todaystats";
+    if( Util::file_exists( todayfile.c_str() ) && std::remove( todayfile.c_str() ) )
+    {
+        return false;
+    }
+    else
+    {
+        if( current_day )
+        {
+            delete current_day;
+            current_day = NULL;
+        }
+        start_new_day();
+   }
+
+    return true;
+}
+
+
 //! Starts a new day and archive current any (if exists)
 void
 Statistics::start_new_day()
@@ -498,7 +535,7 @@ Statistics::load(ifstream &infile, bool history)
     {
       add_history(stats);
     }
-  
+
   TRACE_EXIT();
 }
 
@@ -970,7 +1007,7 @@ Statistics::mouse_notify(int x, int y, int wheel_delta)
           delta_x = abs(x - prev_x);
           delta_y = abs(y - prev_y);
         }
-      
+
       prev_x = x;
       prev_y = y;
 
@@ -1003,7 +1040,7 @@ Statistics::mouse_notify(int x, int y, int wheel_delta)
           last_mouse_time = now;
         }
     }
-  
+
   lock.unlock();
 }
 
