@@ -1,6 +1,6 @@
 // Harpoon.cc --- ActivityMonitor for W32
 //
-// Copyright (C) 2007, 2010 Ray Satiro <raysatiro@yahoo.com>
+// Copyright (C) 2007, 2010, 2013 Ray Satiro <raysatiro@yahoo.com>
 // Copyright (C) 2007, 2008 Rob Caelers <robc@krandor.org>
 // Copyright (C) 2010 Rob Caelers <robc@krandor.nl>
 // All rights reserved.
@@ -66,28 +66,30 @@ Harpoon::~Harpoon()
 bool
 Harpoon::init(HarpoonHookFunc func)
 {
+  TRACE_ENTER("Harpoon::init");
   assert( HARPOON_MAX_UNBLOCKED_APPS );
   init_critical_filename_list();
 
   bool debug, mouse_lowlevel, keyboard_lowlevel;
 
   CoreFactory::get_configurator()->
-      get_value_with_default( "advanced/harpoon/debug", debug, false );
+    get_value_with_default( "advanced/harpoon/debug", debug, false );
 
   bool default_mouse_lowlevel = false;
   if ( LOBYTE( LOWORD( GetVersion() ) ) >= 6)
     {
       default_mouse_lowlevel = true;
     }
-  
-  CoreFactory::get_configurator()->
-      get_value_with_default( "advanced/harpoon/mouse_lowlevel", mouse_lowlevel, default_mouse_lowlevel );
 
   CoreFactory::get_configurator()->
-      get_value_with_default( "advanced/harpoon/keyboard_lowlevel", keyboard_lowlevel, true );
+    get_value_with_default( "advanced/harpoon/mouse_lowlevel", mouse_lowlevel, default_mouse_lowlevel );
+
+  CoreFactory::get_configurator()->
+    get_value_with_default( "advanced/harpoon/keyboard_lowlevel", keyboard_lowlevel, true );
 
   if (!harpoon_init(critical_filename_list, (BOOL)debug))
     {
+      TRACE_RETURN("Cannot init");
       return false;
     }
 
@@ -95,17 +97,18 @@ Harpoon::init(HarpoonHookFunc func)
     {
       if (!harpoon_hook(func, (BOOL)keyboard_lowlevel, (BOOL)mouse_lowlevel))
         {
+          TRACE_RETURN("Cannot hook");
           return false;
         }
     }
 
-#if 1
   if (is_64bit_windows())
     {
-       start_harpoon_helper();
+      TRACE_MSG("start helper");
+      start_harpoon_helper();
     }
-#endif
 
+  TRACE_RETURN(true);
   return true;
 }
 
@@ -119,7 +122,7 @@ Harpoon::terminate()
 }
 
 
-void 
+void
 Harpoon::block_input()
 {
   harpoon_block_input();
@@ -138,7 +141,7 @@ Harpoon::block_input()
     }
 }
 
-void 
+void
 Harpoon::unblock_input()
 {
   harpoon_unblock_input();
@@ -278,11 +281,13 @@ Harpoon::check_for_taskmgr_debugger( char *out )
 }
 
 
-bool 
+bool
 Harpoon::is_64bit_windows()
 {
+  TRACE_ENTER("Harpoon::is_64bit_windows");
 #if defined(_WIN64)
-  return TRUE;  // 64-bit programs run only on Win64
+  TRACE_RETURN("Yes. win64 build");
+  return true;  // 64-bit programs run only on Win64
 #elif defined(_WIN32)
   BOOL f64 = FALSE;
 
@@ -292,14 +297,18 @@ Harpoon::is_64bit_windows()
   fnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress(GetModuleHandleA("kernel32.dll"), "IsWow64Process");
   if (fnIsWow64Process != NULL)
     {
-      return fnIsWow64Process(GetCurrentProcess(), &f64) && f64;
+      bool ret = fnIsWow64Process(GetCurrentProcess(), &f64) && f64;
+      TRACE_RETURN(ret);
+      return ret;
     }
   else
     {
-      return FALSE;
+      TRACE_RETURN("No. IsWow64Process missing.");
+      return false;
     }
 #else
-  return FALSE; // Win64 does not support Win16
+  TRACE_RETURN("No. win16 build");
+  return false; // Win64 does not support Win16
 #endif
 }
 
@@ -360,14 +369,14 @@ Harpoon::start_harpoon_helper()
       ZeroMemory(&pi, sizeof(pi));
 
       string install_dir = g_win32_get_package_installation_directory_of_module(NULL);
-      string helper = install_dir + G_DIR_SEPARATOR_S + "lib" + G_DIR_SEPARATOR_S + "HarpoonHelper.exe";
+      string helper = install_dir + G_DIR_SEPARATOR_S + "lib" + G_DIR_SEPARATOR_S + "WorkraveHelper.exe";
       string args = helper + " " + g_get_prgname();
 
       TRACE_MSG(install_dir.c_str());
       TRACE_MSG(helper.c_str());
       TRACE_MSG(args.c_str());
-      
-      if (CreateProcessA(helper.c_str(), (LPSTR) args.c_str(), NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) 
+
+      if (CreateProcessA(helper.c_str(), (LPSTR) args.c_str(), NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
         {
           helper_started = true;
           helper_window = recursive_find_window(NULL, HARPOON_HELPER_WINDOW_CLASS);
@@ -387,6 +396,10 @@ void
 Harpoon::stop_harpoon_helper()
 {
   TRACE_ENTER("Harpoon::stop_harpoon_helper" );
+  if (helper_window == NULL)
+    {
+      helper_window = recursive_find_window(NULL, HARPOON_HELPER_WINDOW_CLASS);
+    }
   if (helper_window != NULL)
     {
       PostMessage(helper_window, WM_USER + HARPOON_HELPER_EXIT, 0, 0);
