@@ -1,6 +1,6 @@
 // SoundPlayer.cc --- Sound player
 //
-// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -45,10 +45,6 @@
 
 #if defined HAVE_GSTREAMER
 #include "GstSoundPlayer.hh"
-#elif defined HAVE_DEPRECATED_GNOME
-#include "GnomeSoundPlayer.hh"
-#elif defined HAVE_KDE
-#include "KdeSoundPlayer.hh"
 #elif defined PLATFORM_OS_UNIX
 #include <X11/Xlib.h>
 #elif defined PLATFORM_OS_WIN32
@@ -64,9 +60,7 @@
 #include "PulseMixer.hh"
 #endif
 
-#if defined HAVE_GNOME
 #include <gdk/gdk.h>
-#endif
 
 const char *SoundPlayer::CFG_KEY_SOUND_ENABLED = "sound/enabled";
 const char *SoundPlayer::CFG_KEY_SOUND_DEVICE = "sound/device";
@@ -252,13 +246,12 @@ public:
   SpeakerPlayer(short (*beeps)[2]);
   void run();
 private:
-  SoundPlayer *player;
   short (*beeps)[2];
 };
 
 
 SpeakerPlayer::SpeakerPlayer(short (*b)[2])
-  : Thread(true)
+  : Thread()
 {
   beeps = b;
 }
@@ -284,10 +277,7 @@ SpeakerPlayer::run()
   short (*b)[2];
   b = beeps;
 #ifdef PLATFORM_OS_UNIX
-  Display *display = NULL;
-#  ifdef HAVE_GNOME
-  display = XOpenDisplay(gdk_get_display());
-#  endif
+  Display *display = XOpenDisplay(gdk_get_display());
   if (display) {
 #endif
   while (b[0][0])
@@ -317,10 +307,6 @@ SoundPlayer::SoundPlayer()
   driver =
 #if defined HAVE_GSTREAMER
      new GstSoundPlayer()
-#elif defined HAVE_DEPRECATED_GNOME
-     new GnomeSoundPlayer()
-#elif defined HAVE_KDE
-     new KdeSoundPlayer()
 #elif defined PLATFORM_OS_WIN32
      new W32DirectSoundPlayer()
 #elif defined PLATFORM_OS_OSX
@@ -363,7 +349,7 @@ SoundPlayer::init()
     {
       mixer->init();
     }
-  
+
   int volume = 0;
   if( !CoreFactory::get_configurator()->get_value(CFG_KEY_SOUND_VOLUME, volume) )
     {
@@ -397,7 +383,7 @@ SoundPlayer::register_sound_events(string theme)
     {
       string file = Util::complete_directory(path, Util::SEARCH_PATH_SOUNDS);
       TRACE_MSG(file);
-      
+
       Theme theme;
       load_sound_theme(file, theme);
 
@@ -475,7 +461,7 @@ SoundPlayer::sync_settings()
 
           string wav_file;
           valid = driver->get_sound_wav_file((SoundEvent)i, wav_file);
-          if (valid) 
+          if (valid)
            {
               CoreFactory::get_configurator()->set_value(string(SoundPlayer::CFG_KEY_SOUND_EVENTS) +
                                                          snd->id,
@@ -498,12 +484,12 @@ SoundPlayer::load_sound_theme(const string &themefilename, Theme &theme)
 
   r = g_key_file_load_from_file(config, themefilename.c_str(), G_KEY_FILE_KEEP_COMMENTS, NULL);
   TRACE_MSG("load " << r);
-  
+
   if (r)
     {
       gchar *themedir = g_path_get_dirname(themefilename.c_str());
       TRACE_MSG(themedir);
-      
+
       char *desc = g_key_file_get_string(config, "general", "description", NULL);
       if (desc != NULL)
         {
@@ -516,7 +502,7 @@ SoundPlayer::load_sound_theme(const string &themefilename, Theme &theme)
         {
           SoundRegistry *snd = &sound_registry[i];
           char *sound_pathname = NULL;
-          
+
           char *filename = g_key_file_get_string(config, snd->id, "file", NULL);
           if (filename != NULL)
             {
@@ -584,7 +570,7 @@ SoundPlayer::get_sound_themes(std::vector<Theme> &themes)
 		      while ((file = g_dir_read_name(dir)) != NULL)
             {
               gchar *test_path = g_build_filename(it->c_str(), file, NULL);
-              
+
               if (test_path != NULL && g_file_test(test_path, G_FILE_TEST_IS_DIR))
                 {
                   char *path = g_build_filename(it->c_str(), file, "soundtheme", NULL);
@@ -628,7 +614,7 @@ SoundPlayer::get_sound_themes(std::vector<Theme> &themes)
           bool valid = CoreFactory::get_configurator()->get_value(string(CFG_KEY_SOUND_EVENTS) +
                                                                   snd->id,
                                                                   file);
-          if (valid)
+          if (valid && file != "")
             {
               active_theme.files.push_back(file);
             }
@@ -786,7 +772,7 @@ SoundPlayer::get_sound_enabled(SoundEvent snd, bool &enabled)
   if (snd >= SOUND_MIN && snd < SOUND_MAX)
     {
       enabled = true;
-      
+
       ret = CoreFactory::get_configurator()->get_value(string(CFG_KEY_SOUND_EVENTS) +
                                                        sound_registry[snd].id +
                                                        CFG_KEY_SOUND_EVENTS_ENABLED,
@@ -863,7 +849,7 @@ SoundPlayer::capability(SoundCapability cap)
     {
       ret = driver->capability(cap);
     }
-  
+
   return ret;
 }
 
@@ -871,7 +857,7 @@ void
 SoundPlayer::restore_mute()
 {
   TRACE_ENTER("SoundPlayer::restore_mute");
-  
+
   if (mixer != NULL && must_unmute)
     {
       mixer->set_mute(false);
