@@ -1,6 +1,6 @@
 // AppletControl.cc --- Applet info Control
 //
-// Copyright (C) 2006, 2007, 2008, 2009 Rob Caelers & Raymond Penners
+// Copyright (C) 2006, 2007, 2008, 2009, 2011, 2012, 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -23,16 +23,11 @@
 #include "config.h"
 #endif
 
-#include "nls.h"
 #include "debug.hh"
 
 #include "AppletControl.hh"
 
-#ifdef HAVE_KDE
-#include "KdeAppletWindow.hh"
-#endif
-
-#ifdef HAVE_GNOMEAPPLET
+#if defined(HAVE_PANELAPPLET2) || defined(HAVE_PANELAPPLET4)
 #include "GnomeAppletWindow.hh"
 #endif
 
@@ -44,13 +39,17 @@
 #include "W32AppletWindow.hh"
 #endif
 
-// #ifdef PLATFORM_OS_OSX
-// #include "OSXAppletWindow.hh"
-// #endif
+#ifdef HAVE_DBUS_GIO
+#include "GenericDBusApplet.hh"
+#endif
+
+#ifdef PLATFORM_OS_OSX
+#include "OSXAppletWindow.hh"
+#endif
 
 #include "GUI.hh"
-#include "MainWindow.hh"
 #include "TimerBoxControl.hh"
+#include "Menus.hh" // REFACTOR
 
 #include "CoreFactory.hh"
 #include "IConfigurator.hh"
@@ -60,13 +59,14 @@ using namespace std;
 
 //! Constructor.
 AppletControl::AppletControl()
-  : enabled(false),
+  : visible(false),
+    enabled(false),
     delayed_show(-1)
 {
   for (int i = 0; i < APPLET_SIZE; i++)
     {
       applets[i] = NULL;
-      visible[i] = false;
+      applet_state[i] = AppletWindow::APPLET_STATE_DISABLED;
     }
 }
 
@@ -90,26 +90,36 @@ AppletControl::~AppletControl()
 void
 AppletControl::init()
 {
-#ifdef HAVE_KDE
-  applets[APPLET_KDE] = new KdeAppletWindow(this);
+#if defined(HAVE_PANELAPPLET2) || defined(HAVE_PANELAPPLET4)
+  applets[APPLET_GNOME] = new GnomeAppletWindow();
 #endif
 
-#ifdef HAVE_GNOMEAPPLET
-  applets[APPLET_GNOME] = new GnomeAppletWindow(this);
+#ifdef HAVE_DBUS_GIO
+  applets[APPLET_GENERIC_DBUS] = new GenericDBusApplet();
 #endif
-
+  
 #ifdef PLATFORM_OS_UNIX
-  applets[APPLET_TRAY] = new X11SystrayAppletWindow(this);
+  applets[APPLET_TRAY] = new X11SystrayAppletWindow();
 #endif
 
 #ifdef PLATFORM_OS_WIN32
   applets[APPLET_W32] = new W32AppletWindow();
 #endif
 
-// #ifdef PLATFORM_OS_OSX
-//   applets[APPLET_OSX] = new OSXAppletWindow();
-// #endif
-
+#ifdef PLATFORM_OS_OSX
+  applets[APPLET_OSX] = new OSXAppletWindow();
+#endif
+  
+  for (int i = 0; i < APPLET_SIZE; i++)
+    {
+      if (applets[i] != NULL)
+        {
+          applets[i]->init_applet();
+          applets[i]->signal_state_changed().connect(sigc::bind<0>(sigc::mem_fun(*this, &AppletControl::on_applet_state_changed), (AppletType)i));
+          applets[i]->signal_request_activate().connect(sigc::bind<0>(sigc::mem_fun(*this, &AppletControl::on_applet_request_activate), (AppletType)i));
+        }
+    }
+  
   // Read configuration and start monitoring it.
   IConfigurator *config = CoreFactory::get_configurator();
   config->add_listener(TimerBoxControl::CFG_KEY_TIMERBOX + "applet", this);
@@ -123,10 +133,13 @@ void
 AppletControl::show()
 {
   TRACE_ENTER("AppletControl::show");
-  
+
   bool specific = false;
   AppletState rc;
 
+  rc = activate_applet(APPLET_GENERIC_DBUS);
+  TRACE_MSG("Generic" << rc);
+
   rc = activate_applet(APPLET_GNOME);
   TRACE_MSG("Gnome " << rc);
   if (rc != AppletWindow::APPLET_STATE_DISABLED)
@@ -136,13 +149,6 @@ AppletControl::show()
       specific = true;
     }
 
-  rc = activate_applet(APPLET_KDE);
-  TRACE_MSG("kde " << rc);
-  if (rc != AppletWindow::APPLET_STATE_DISABLED)
-    {
-      specific = true;
-    }
-
   rc = activate_applet(APPLET_W32);
   TRACE_MSG("Win32 " << rc);
   if (rc != AppletWindow::APPLET_STATE_DISABLED)
@@ -150,12 +156,12 @@ AppletControl::show()
       specific = true;
     }
 
-//   rc = activate_applet(APPLET_OSX);
-//   TRACE_MSG("OSX " << rc);
-//   if (rc != AppletWindow::APPLET_STATE_DISABLED)
-//     {
-//       specific = true;
-//     }
+  rc = activate_applet(APPLET_OSX);
+  TRACE_MSG("OSX " << rc);
+  if (rc != AppletWindow::APPLET_STATE_DISABLED)
+    {
+      specific = true;
+    }
 
 #ifdef PLATFORM_OS_UNIX
   if (specific)
@@ -168,9 +174,11 @@ AppletControl::show()
       rc = activate_applet(APPLET_TRAY);
       TRACE_MSG("X11 act " << rc);
     }
+#else
+  (void) specific;
 #endif
   check_visible();
-  
+
   TRACE_EXIT();
 }
 
@@ -192,9 +200,7 @@ AppletControl::show(AppletType type)
 #ifdef PLATFORM_OS_UNIX
   if (applets[APPLET_TRAY] != NULL)
     {
-      if ((type == APPLET_KDE || type == APPLET_GNOME)
-          && specific)
-
+      if (type == APPLET_GNOME && specific)
         {
           deactivate_applet(APPLET_TRAY);
         }
@@ -203,6 +209,8 @@ AppletControl::show(AppletType type)
           activate_applet(APPLET_TRAY);
         }
     }
+#else
+  (void) specific;
 #endif
 
   check_visible();
@@ -220,57 +228,51 @@ AppletControl::hide()
 }
 
 
-//! Hide a specific applet.
-void
-AppletControl::hide(AppletType type)
-{
-  deactivate_applet(type);
-}
-
-
 //! The specified applet is not active.
 void
-AppletControl::set_applet_state(AppletType type, AppletWindow::AppletState state)
+AppletControl::on_applet_state_changed(AppletType type, AppletWindow::AppletState state)
 {
   TRACE_ENTER_MSG("AppletControl::set_applet_state", type << " " << state);
-  switch (state)
-    {
-    case AppletWindow::APPLET_STATE_DISABLED:
-      visible[type] = false;
-      break;
-
-    case AppletWindow::APPLET_STATE_VISIBLE:
-      visible[type] = true;
-      break;
-
-    case AppletWindow::APPLET_STATE_PENDING:
-      visible[type] = false;
-      break;
-    }
+  applet_state[type] = state;
 
 #ifdef PLATFORM_OS_UNIX
-  if (visible[type] &&
-      (type == APPLET_KDE || type == APPLET_GNOME))
+  if (is_visible(type) && (type == APPLET_GNOME || type == APPLET_GENERIC_DBUS))
     {
+      TRACE_MSG("Deactivate tray");
       deactivate_applet(APPLET_TRAY);
     }
 #endif
 
   if (enabled && !is_visible())
     {
+      TRACE_MSG("none visible, show in 5s");
       delayed_show = 5;
     }
 
+  // REFACTOR
+  if (state == AppletWindow::APPLET_STATE_VISIBLE)
+    {
+      IGUI *gui = GUI::get_instance();
+      Menus *menus = gui->get_menus();
+      menus->resync();
+    }
+
   check_visible();
   TRACE_EXIT();
 }
 
 
+void
+AppletControl::on_applet_request_activate(AppletType type)
+{
+  show(type);
+}
+
 //! Is at least a single applet visible.
 bool
 AppletControl::is_visible(AppletType type)
 {
-  return visible[type];
+  return applet_state[type] == AppletWindow::APPLET_STATE_VISIBLE;
 }
 
 
@@ -280,9 +282,9 @@ AppletControl::is_visible()
 {
   bool ret = false;
 
-    for (int i = 0; i < APPLET_SIZE; i++)
+  for (int i = APPLET_FIRST; i < APPLET_SIZE; i++)
     {
-      if (visible[i])
+      if (is_visible((AppletType)i))
         {
           ret = true;
         }
@@ -298,7 +300,7 @@ AppletControl::heartbeat()
   TRACE_ENTER("AppletControl::heartbeat");
   if (delayed_show < 0 && enabled && !is_visible())
     {
-      delayed_show = 60;
+      delayed_show = 30;
     }
   else if (delayed_show > 0)
     {
@@ -310,9 +312,9 @@ AppletControl::heartbeat()
       show();
     }
 
-  for (int i = 0; i < APPLET_SIZE; i++)
+  for (int i = APPLET_FIRST; i < APPLET_SIZE; i++)
     {
-      if (applets[i] != NULL && visible[i])
+      if (applets[i] != NULL && is_visible((AppletType)i))
         {
           applets[i]->update_applet();
         }
@@ -323,13 +325,13 @@ AppletControl::heartbeat()
 
 //! Sets the tooltip of all visible applets.
 void
-AppletControl::set_timers_tooltip(std::string& tip)
+AppletControl::set_tooltip(std::string& tip)
 {
-  for (int i = 0; i < APPLET_SIZE; i++)
+  for (int i = APPLET_FIRST; i < APPLET_SIZE; i++)
     {
-      if (applets[i] != NULL && visible[i])
+      if (applets[i] != NULL && is_visible((AppletType)i))
         {
-          applets[i]->set_timers_tooltip(tip);
+          applets[i]->set_applet_tooltip(tip);
         }
     }
 }
@@ -346,7 +348,7 @@ AppletControl::read_configuration()
     {
       delayed_show = -1;
     }
-  
+
   if (!previous_enabled && enabled)
     {
       show();
@@ -376,43 +378,41 @@ AppletControl::check_visible()
   TRACE_ENTER("AppletControl::check_visible");
   int count = 0;
 
-  for (int i = 0; i < APPLET_SIZE; i++)
+  for (int i = APPLET_FIRST; i < APPLET_SIZE; i++)
     {
-      if (visible[i])
+      if (is_visible((AppletType)i))
         {
+          TRACE_MSG(i << " is visible"); 
           count++;
         }
     }
 
-// #ifdef PLATFORM_OS_OSX
-//   count++;
-// #endif
-    
-  GUI *gui = GUI::get_instance();
-  MainWindow *main = gui->get_main_window();
-  if (main != NULL)
-    {
-      main->set_applet_active( count > 0 );
-    }
+#ifdef PLATFORM_OS_OSX
+  count++;
+#endif
 
+  bool new_visible = count > 0;
+  if (new_visible != visible)
+    {
+      visible = new_visible;
+      visibility_changed_signal.emit();
+    }
+  
   TRACE_EXIT();
 }
 
 AppletWindow::AppletState
 AppletControl::activate_applet(AppletType type)
 {
-  AppletState r = AppletWindow::APPLET_STATE_DISABLED;
+  AppletState state = AppletWindow::APPLET_STATE_DISABLED;
 
   if (applets[type] != NULL)
     {
-      r = applets[type]->activate_applet();
-      if (r == AppletWindow::APPLET_STATE_VISIBLE)
-        {
-          visible[type] = true;
-        }
+      state = applets[type]->activate_applet();
+      applet_state[type] = state;
     }
 
-  return r;
+  return state;
 }
 
 void
@@ -421,6 +421,12 @@ AppletControl::deactivate_applet(AppletType type)
  if (applets[type] != NULL)
     {
       applets[type]->deactivate_applet();
-      visible[type] = false;
+      applet_state[type] = AppletWindow::APPLET_STATE_DISABLED;
     }
 }
+
+sigc::signal<void> &
+AppletControl::signal_visibility_changed()
+{
+  return visibility_changed_signal;
+}
