@@ -1,6 +1,6 @@
 // BreakWindow.cc --- base class for the break windows
 //
-// Copyright (C) 2001 - 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -25,15 +25,12 @@
 
 #ifdef PLATFORM_OS_WIN32
 #include "W32Compat.hh"
+#include "W32ForceFocus.hh"
 #include <gdk/gdkwin32.h>
 #endif
 
 #include <gdk/gdkkeysyms.h>
 
-#include <gdk/gdkkeysyms.h>
-
-#include <gdk/gdkkeysyms.h>
-
 #include "preinclude.h"
 #include "debug.hh"
 #include "nls.h"
@@ -42,17 +39,12 @@
 #undef max
 #endif
 
-#include <gtkmm/window.h>
-#include <gtkmm/stock.h>
-#include <gtkmm/buttonbox.h>
-#include <gtkmm/button.h>
-#include <gtkmm/enums.h>
-#include <gdkmm/types.h>
-
+#include <gtkmm.h>
 #include <math.h>
 
-#include "GUI.hh"
 #include "BreakWindow.hh"
+
+#include "GUI.hh"
 #include "IBreak.hh"
 #include "IBreakResponse.hh"
 #include "GtkUtil.hh"
@@ -102,22 +94,18 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
   postpone_button(NULL),
   skip_button(NULL),
   lock_button(NULL),
-  shutdown_button(NULL)
+  shutdown_button(NULL),
+  accel_group(NULL)
+#ifdef PLATFORM_OS_WIN32
+  ,
+  desktop_window( NULL ),
+  force_focus_on_break_start( false ),
+  parent( 0 )
+#endif
 {
   TRACE_ENTER("BreakWindow::BreakWindow");
   this->break_id = break_id;
 
-#ifdef PLATFORM_OS_WIN32
-  desktop_window = NULL;
-
-  // Here's the secret: IMMEDIATELY after your window creation, set focus to it
-  // THEN position it. So:
-
-  HWND hwnd = (HWND) GDK_WINDOW_HWND(Gtk::Widget::gobj()->window);
-  ::SetFocus(hwnd);
-  ::SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
-#endif
-
   if (mode != GUIConfig::BLOCK_MODE_NONE)
     {
       // Disable titlebar to appear like a popup
@@ -132,6 +120,15 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
   // Need to realize window before it is shown
   // Otherwise, there is not gobj()...
   realize();
+
+#ifdef PLATFORM_OS_WIN32
+  // Here's the secret: IMMEDIATELY after your window creation, set focus to it
+  // THEN position it. So:
+	HWND hwnd = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(Gtk::Widget::gobj()));
+	SetFocus(hwnd);
+	SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
+#endif
+
   if (mode == GUIConfig::BLOCK_MODE_NONE)
     {
       Glib::RefPtr<Gdk::Window> window = get_window();
@@ -140,30 +137,30 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
 
   // trace window handles:
   // FIXME: debug, remove later
-#ifdef PLATFORM_OS_WIN32
-  HWND _hwnd = (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
-  HWND _scope = (HWND) GDK_WINDOW_HWND( GTK_WIDGET( this->gobj() )->window );
+#if defined( PLATFORM_OS_WIN32 ) && defined( TRACING )
+  HWND _hwnd = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(Gtk::Widget::gobj()));
+  HWND _scope = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET( this->gobj())));
   HWND _hRoot = GetAncestor( _hwnd, GA_ROOT );
   HWND _hParent = GetAncestor( _hwnd, GA_PARENT );
   HWND _hDesktop = GetDesktopWindow();
-  
+
   TRACE_MSG("BreakWindow created" <<  hex << _hwnd << dec);
   if (_hwnd != _scope)
     {
       TRACE_MSG("!!! Scope issue: " << hex << _scope << dec);
     }
-  
+
   if (_hwnd != _hRoot)
     {
       TRACE_MSG("GetDesktopWindow()" <<  hex << _hDesktop << dec);
       TRACE_MSG("!!! BreakWindow GA_ROOT: " << hex << _hRoot << dec);
     }
-  
+
   if (_hParent != _hDesktop)
     {
       TRACE_MSG("GetDesktopWindow()" <<  hex << _hDesktop << dec);
       TRACE_MSG("!!! PreludeWindow GA_PARENT: " << hex << _hParent << dec);
-      
+
       HWND _hTemp;
       while( IsWindow( _hParent ) && _hParent != _hDesktop )
         {
@@ -176,7 +173,7 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
           TRACE_MSG("!!!" <<  hex << _hTemp << " GWL_HWNDPARENT: " << hex << _hParent2  << dec);
         }
     }
-    
+
 #endif
 
   this->head = head;
@@ -188,11 +185,14 @@ BreakWindow::BreakWindow(BreakId break_id, HeadInfo &head,
   bool initial_ignore_activity = false;
 
 #ifdef PLATFORM_OS_WIN32
-  if( W32Compat::get_force_focus_value() )
-    {
+  if( W32ForceFocus::GetForceFocusValue() )
       initial_ignore_activity = true;
-    }
 
+  CoreFactory::get_configurator()->get_value_with_default(
+    "advanced/force_focus_on_break_start",
+    force_focus_on_break_start,
+    true
+    );
 #endif
 
   ICore *core = CoreFactory::get_core();
@@ -243,7 +243,12 @@ BreakWindow::init_gui()
               set_size_request(head.get_width(),
                                head.get_height());
               set_app_paintable(true);
-              set_desktop_background(GTK_WIDGET (gobj())->window);
+#ifdef HAVE_GTK3
+              Glib::RefPtr<Gdk::Window> window = get_window();
+              set_desktop_background(window->gobj());
+#else
+              set_desktop_background(GTK_WIDGET(gobj())->window);
+#endif
               Gtk::Alignment *align
                 = Gtk::manage(new Gtk::Alignment(0.5, 0.5, 0.0, 0.0));
               align->add(*window_frame);
@@ -259,9 +264,13 @@ BreakWindow::init_gui()
       // FIXME: check if it was intentionally not unset for RB
       if (break_id != BREAK_ID_REST_BREAK)
         {
+#ifdef HAVE_GTK3
+          set_can_focus(false);
+#else
           unset_flags(Gtk::CAN_FOCUS);
+#endif
         }
-      
+
       show_all_children();
       stick();
     }
@@ -305,7 +314,11 @@ BreakWindow::create_lock_button()
       ret = Gtk::manage(GtkUtil::create_image_button(_("_Lock"), "lock.png"));
       ret->signal_clicked()
         .connect(sigc::mem_fun(*this, &BreakWindow::on_lock_button_clicked));
+#ifdef HAVE_GTK3
+      ret->set_can_focus(false);
+#else
       GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
+#endif
     }
   else
     {
@@ -324,7 +337,11 @@ BreakWindow::create_shutdown_button()
       ret = Gtk::manage(GtkUtil::create_image_button(_("Shut _down"), "shutdown.png"));
       ret->signal_clicked()
         .connect(sigc::mem_fun(*this, &BreakWindow::on_shutdown_button_clicked));
+#ifdef HAVE_GTK3
+      ret->set_can_focus(false);
+#else
       GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
+#endif
     }
   else
     {
@@ -341,8 +358,11 @@ BreakWindow::create_skip_button()
   ret = Gtk::manage(GtkUtil::create_custom_stock_button(_("_Skip"), Gtk::Stock::CLOSE));
   ret->signal_clicked()
     .connect(sigc::mem_fun(*this, &BreakWindow::on_skip_button_clicked));
+#ifdef HAVE_GTK3
+  ret->set_can_focus(false);
+#else
   GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
-
+#endif
   return ret;
 }
 
@@ -355,7 +375,11 @@ BreakWindow::create_postpone_button()
   ret = Gtk::manage(GtkUtil::create_custom_stock_button(_("_Postpone"), Gtk::Stock::REDO));
   ret->signal_clicked()
     .connect(sigc::mem_fun(*this, &BreakWindow::on_postpone_button_clicked));
+#ifdef HAVE_GTK3
+  ret->set_can_focus(false);
+#else
   GTK_WIDGET_UNSET_FLAGS(ret->gobj(), GTK_CAN_FOCUS);
+#endif
   return ret;
 }
 
@@ -367,7 +391,7 @@ BreakWindow::on_lock_button_clicked()
 {
   if (System::is_lockable())
     {
-      GUI *gui = GUI::get_instance();
+      IGUI *gui = GUI::get_instance();
       assert(gui != NULL);
 
       gui->interrupt_grab();
@@ -379,11 +403,10 @@ BreakWindow::on_lock_button_clicked()
 void
 BreakWindow::on_shutdown_button_clicked()
 {
-#ifdef HAVE_GNOME
-  GUI *gui = GUI::get_instance();
+  IGUI *gui = GUI::get_instance();
   assert(gui != NULL);
   gui->interrupt_grab();
-#endif
+
   System::shutdown();
 }
 
@@ -439,7 +462,13 @@ void
 BreakWindow::resume_non_ignorable_break()
 {
   TRACE_ENTER("BreakWindow::resume_non_ignorable_break");
-  if (! (break_flags & BreakWindow::BREAK_FLAGS_USER_INITIATED))
+  ICore *core = CoreFactory::get_core();
+  OperationMode mode = core->get_operation_mode();
+
+  TRACE_MSG("break flags " << break_flags);
+  
+  if (! (break_flags & BreakWindow::BREAK_FLAGS_USER_INITIATED) &&
+      mode == OPERATION_MODE_NORMAL)
     {
       for (int id = break_id - 1; id >= 0; id--)
         {
@@ -449,10 +478,7 @@ BreakWindow::resume_non_ignorable_break()
           if (!ignorable)
             {
               TRACE_MSG("Break " << id << " not ignorable");
-          
-              ICore *core = CoreFactory::get_core();
-              assert(core != NULL);
-              
+
               IBreak *b = core->get_break(BreakId(id));
               assert(b != NULL);
 
@@ -477,12 +503,11 @@ BreakWindow::create_break_buttons(bool lockable,
 
   accel_group = Gtk::AccelGroup::create();
   add_accel_group(accel_group);
-  
+
   if ((break_flags != BREAK_FLAGS_NONE) || lockable || shutdownable)
     {
       box = new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6);
 
-      Gtk::Button *shutdown_button = NULL;
       if (shutdownable)
         {
           shutdown_button = create_shutdown_button();
@@ -506,7 +531,7 @@ BreakWindow::create_break_buttons(bool lockable,
           skip_button = create_skip_button();
           box->pack_end(*skip_button, Gtk::PACK_SHRINK, 0);
         }
-      
+
       if ((break_flags & BREAK_FLAGS_POSTPONABLE) != 0)
         {
           postpone_button = create_postpone_button();
@@ -548,6 +573,15 @@ BreakWindow::start()
   WindowHints::set_always_on_top(this, true);
   raise();
 
+#ifdef PLATFORM_OS_WIN32
+  if( force_focus_on_break_start && this->head.valid && ( this->head.count == 0 ) )
+    {
+      HWND hwnd = (HWND)GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
+      bool focused = W32ForceFocus::ForceWindowFocus( hwnd );
+      bool this_is_a_dummy_var_to_fool_visual_studio_debugger = focused;
+    }
+#endif
+
   // In case the show_all resized the window...
   center();
   TRACE_EXIT();
@@ -564,9 +598,13 @@ BreakWindow::stop()
       frame->set_frame_flashing(0);
     }
 
+#ifdef HAVE_GTK3
+  hide();
+#else
   hide_all();
+#endif
   visible = false;
-  
+
 #ifdef PLATFORM_OS_WIN32
   if (desktop_window)
     desktop_window->set_visible(false);
@@ -593,48 +631,11 @@ void
 BreakWindow::refresh()
 {
   TRACE_ENTER("BreakWindow::refresh");
-  
+
   update_break_window();
 
 #ifdef PLATFORM_OS_WIN32
-  ICore *core = CoreFactory::get_core();
-  bool user_active = core->is_user_active();
-
-  // GTK keyboard shortcuts can be accessed by using the ALT key. This appear
-  // to be non-standard behaviour on windows, so make shortcuts available
-  // without ALT after the user is idle for 5s
-  if (!user_active && !accel_added)
-    {
-      IBreak *b = core->get_break(BreakId(break_id));
-      assert(b != NULL);
-
-      TRACE_MSG(b->get_elapsed_idle_time());
-      if (b->get_elapsed_idle_time() > 5)
-        {
-          if (postpone_button != NULL)
-            {
-              GtkUtil::update_mnemonic(postpone_button, accel_group);
-            }
-          if (skip_button != NULL)
-            {
-              GtkUtil::update_mnemonic(skip_button, accel_group);
-            }
-          if (shutdown_button != NULL)
-            {
-              GtkUtil::update_mnemonic(shutdown_button, accel_group);
-            }
-          if (lock_button != NULL)
-            {
-              GtkUtil::update_mnemonic(lock_button, accel_group);
-            }
-          accel_added = true;
-        }
-    }
-
-  if (block_mode != GUIConfig::BLOCK_MODE_NONE)
-    {
-      WindowHints::set_always_on_top(this, true);
-    }
+  W32Compat::RefreshBreakWindow( *this );
 #endif
   TRACE_EXIT();
 }
