@@ -1,6 +1,6 @@
 // GUI.cc --- The WorkRave GUI
 //
-// Copyright (C) 2001 - 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -25,10 +25,10 @@
 #include "nls.h"
 #include "debug.hh"
 
-#include <gtkmm/main.h>
-#include <gtkmm/messagedialog.h>
-#include <glibmm/optiongroup.h>
-#include <glibmm/refptr.h>
+#include <iostream>
+#include <gtkmm.h>
+#include <glibmm.h>
+#include <gtk/gtk.h>
 
 #ifdef PLATFORM_OS_WIN32_NATIVE
 #undef HAVE_UNISTD_H
@@ -89,13 +89,12 @@
 #include <Carbon/Carbon.h>
 #endif
 
-#if defined(HAVE_GCONF)
-#include <gconf/gconf-client.h>
+#if defined(PLATFORM_OS_UNIX)
+#include <X11/Xlib.h>
 #endif
 
-#if defined(HAVE_KDE)
-#include <dcopclient.h>
-#include <kapp.h>
+#if defined(HAVE_GCONF)
+#include <gconf/gconf-client.h>
 #endif
 
 #if defined(HAVE_DBUS)
@@ -115,7 +114,6 @@ GUI *GUI::instance = NULL;
  *  \param argv all command line parameters.
  */
 GUI::GUI(int argc, char **argv) :
-  configurator(NULL),
   core(NULL),
   sound_player(NULL),
   break_windows(NULL),
@@ -126,7 +124,6 @@ GUI::GUI(int argc, char **argv) :
   active_break_id(BREAK_ID_NONE),
   main_window(NULL),
   menus(0),
-  tooltips(NULL),
   break_window_destroy(false),
   prelude_window_destroy(false),
   heads(NULL),
@@ -196,6 +193,10 @@ GUI::main()
 
   Glib::OptionContext option_ctx;
 
+#ifdef PLATFORM_OS_UNIX
+  XInitThreads();
+#endif
+  
   if (!Glib::thread_supported())
     Glib::thread_init();
 
@@ -212,7 +213,7 @@ GUI::main()
       std::cout << "Failed to initialize: " << e.what() << std::endl;
       exit(1);
     }
-  
+
   init_core();
   init_nls();
   init_platform();
@@ -223,17 +224,20 @@ GUI::main()
   init_session();
   init_gui();
   init_startup_warnings();
-  
+
   on_timer();
 
   TRACE_MSG("Initialized. Entering event loop.");
 
   // Enter the event loop
-  gdk_threads_enter();
   Gtk::Main::run();
-  gdk_threads_leave();
 
   cleanup_session();
+
+  for (list<sigc::connection>::iterator i = event_connections.begin(); i != event_connections.end(); i++)
+    {
+      i->disconnect();
+    }
     
   delete main_window;
   main_window = NULL;
@@ -243,7 +247,6 @@ GUI::main()
 
   delete kit;
 
-  
   TRACE_EXIT();
 }
 
@@ -256,11 +259,8 @@ GUI::terminate()
 
   // HACK: Without it status icon keeps on dangling in tray
   // Nicer solution: nicely cleanup complete gui ~GUI()
-  if (status_icon)
-    {
-      delete status_icon;
-      status_icon = 0;
-    }
+  delete status_icon;
+  status_icon = 0;
 
   CoreFactory::get_configurator()->save();
 
@@ -275,10 +275,7 @@ GUI::terminate()
 void
 GUI::open_main_window()
 {
-  if (main_window != NULL)
-    {
-      main_window->open_window();
-    }
+  main_window->open_window();
 }
 
 
@@ -286,30 +283,27 @@ GUI::open_main_window()
 void
 GUI::close_main_window()
 {
-  if (main_window != NULL)
-    {
-      main_window->close_window();
-    }
+  main_window->close_window();
 }
 
 
 //! The user close the main window.
 void
-GUI::main_window_closed()
+GUI::on_main_window_closed()
 {
-  TRACE_ENTER("GUI::main_window_closed");
-  if (status_icon)
+  TRACE_ENTER("GUI::on_main_window_closed");
+  bool closewarn = false;
+  CoreFactory::get_configurator()->get_value(GUIConfig::CFG_KEY_CLOSEWARN_ENABLED, closewarn);
+  TRACE_MSG(closewarn);
+  if (closewarn && !closewarn_shown)
     {
-      bool closewarn = false;
-      CoreFactory::get_configurator()->get_value(GUIConfig::CFG_KEY_CLOSEWARN_ENABLED, closewarn);
-      TRACE_MSG(closewarn);
-      if (closewarn && !closewarn_shown)
-        {
-          status_icon->show_balloon(_("Workrave is still running. "
-                                      "You can access Workrave by clicking on the white sheep icon."));
-          closewarn_shown =  true;
-        }
+      status_icon->show_balloon("closewarn",
+                                _("Workrave is still running. "
+                                  "You can access Workrave by clicking on the white sheep icon. "
+                                  "Click on this balloon to disable this message" ));
+      closewarn_shown =  true;
     }
+
   TRACE_EXIT();
 }
 
@@ -319,27 +313,13 @@ bool
 GUI::on_timer()
 {
   std::string tip = get_timers_tooltip();
-  
-  if (core != NULL)
-    {
-      core->heartbeat();
-    }
 
-  if (main_window != NULL)
-    {
-      main_window->update();
-    }
+  core->heartbeat();
+  main_window->update();
 
-  if (applet_control != NULL)
-    {
-      applet_control->heartbeat();
-      applet_control->set_timers_tooltip(tip);
-    }
-
-  if (status_icon)
-    {
-      status_icon->set_timers_tooltip(tip);
-    }
+  applet_control->heartbeat();
+  applet_control->set_tooltip(tip);
+  status_icon->set_tooltip(tip);
 
   heartbeat_signal();
 
@@ -355,7 +335,7 @@ GUI::on_timer()
           muted = false;
         }
     }
-  
+
   return true;
 }
 
@@ -370,13 +350,10 @@ void
 GUI::init_platform()
 {
   TRACE_ENTER("GUI::init_platform");
-#if defined(HAVE_KDE)
-  init_kde();
-#endif
 #if defined(PLATFORM_OS_OSX)
   [ [ AppController alloc ] init ];
 #endif
-  
+
 #if defined(PLATFORM_OS_UNIX)
   char *display = gdk_get_display();
   System::init(display);
@@ -384,7 +361,7 @@ GUI::init_platform()
 #else
   System::init();
 #endif
-  
+
   srand((unsigned int)time(NULL));
   TRACE_EXIT();
 }
@@ -395,7 +372,7 @@ GUI::session_quit_cb(EggSMClient *client, GUI *gui)
 {
   (void) client;
   (void) gui;
-  
+
   TRACE_ENTER("GUI::session_quit_cb");
 
   CoreFactory::get_configurator()->save();
@@ -411,7 +388,7 @@ GUI::session_save_state_cb(EggSMClient *client, GKeyFile *key_file, GUI *gui)
   (void) client;
   (void) key_file;
   (void) gui;
-  
+
   CoreFactory::get_configurator()->save();
 }
 
@@ -435,7 +412,7 @@ GUI::init_session()
 
   session = new Session();
   session->init();
-  
+
   TRACE_EXIT();
 }
 
@@ -459,19 +436,6 @@ GUI::cleanup_session()
 
 
 
-#if defined(HAVE_KDE)
-void
-GUI::init_kde()
-{
-  TRACE_ENTER("GUI::init_kde");
-  new KApplication(argc, argv, "Workrave");
-  bool rc = kapp->dcopClient()->attach();
-  TRACE_MSG(rc);
-
-  TRACE_EXIT();
-}
-#endif
-
 //! Initializes messages hooks.
 void
 GUI::init_debug()
@@ -501,13 +465,15 @@ GUI::init_nls()
       g_setenv("LANGUAGE", language.c_str(), 1);
     }
 
-#  if !defined(HAVE_GNOME)
+#  if !defined(HAVE_GNOME) && !defined(HAVE_GTK3)
   gtk_set_locale();
 #  endif
   const char *locale_dir;
 
 #if defined(PLATFORM_OS_WIN32)
-  string dir = Util::get_application_directory() + "\\lib\\locale";
+  string dir = Util::get_application_directory();
+  // Use the pre-install locale location if workrave is running from its MSVC build directory.
+  dir += Util::file_exists( dir + "\\..\\Workrave.sln" ) ? "\\..\\frontend" : "\\lib\\locale";
   locale_dir = dir.c_str();
 #elif defined(PLATFORM_OS_OSX)
   char locale_path[MAXPATHLEN * 4];
@@ -547,7 +513,7 @@ GUI::init_nls()
   bindtextdomain(PACKAGE, locale_dir);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
   textdomain(PACKAGE);
- 
+
 #endif
 }
 
@@ -557,7 +523,7 @@ void
 GUI::init_core()
 {
   string display_name;
-  
+
 #if defined(PLATFORM_OS_UNIX)
   char *display = gdk_get_display();
   if (display != NULL)
@@ -769,7 +735,7 @@ GUI::init_gtk_multihead()
                           irect.intersect(heads[k].geometry, overlap);
                         }
                     }
-                  
+
                   if (!overlap)
                     {
                       heads[count].screen = screen;
@@ -800,39 +766,33 @@ GUI::init_gtk_multihead()
 void
 GUI::init_gui()
 {
-  tooltips = Gtk::manage(new Gtk::Tooltips());
-  tooltips->enable();
-
   menus = new Menus();
 
   // The main status window.
   main_window = new MainWindow();
+  main_window->init();
 
   // The applet window.
   applet_control = new AppletControl();
   applet_control->init();
 
-  AppletWindow *applet_window = NULL;
-#if defined(HAVE_GNOME)
-  applet_window = applet_control->get_applet_window(AppletControl::APPLET_GNOME);
-#endif
-#if defined(PLATFORM_OS_WIN32)
-  applet_window = applet_control->get_applet_window(AppletControl::APPLET_W32);
-#endif
-
-  menus->init(main_window, applet_window);
+  // Menus
+  menus->init(applet_control);
   menus->resync();
 
-  // Status icon
-  bool tray_icon_enabled = GUIConfig::get_trayicon_enabled();
-  if (!tray_icon_enabled)
-    {
-      // Recover from bug in 1.9.3 where tray icon AND mainwindow could be disabled
-      TimerBoxControl::set_enabled("main_window", true);
-    }
-  status_icon = new StatusIcon(*main_window);
-  status_icon->set_visible(tray_icon_enabled);
-  CoreFactory::get_configurator()->add_listener(GUIConfig::CFG_KEY_TRAYICON_ENABLED, this);
+  // Status Icon
+  status_icon = new StatusIcon();
+  status_icon->init();
+
+  // Events
+  event_connections.push_back(main_window->signal_closed().connect(sigc::mem_fun(*this, &GUI::on_main_window_closed)));
+  event_connections.push_back(main_window->signal_visibility_changed().connect(sigc::mem_fun(*this, &GUI::on_visibility_changed)));
+  event_connections.push_back(applet_control->signal_visibility_changed().connect(sigc::mem_fun(*this, &GUI::on_visibility_changed)));
+  event_connections.push_back(status_icon->signal_balloon_activate().connect(sigc::mem_fun(*this, &GUI::on_status_icon_balloon_activate)));
+  event_connections.push_back(status_icon->signal_activate().connect(sigc::mem_fun(*this, &GUI::on_status_icon_activate)));
+  event_connections.push_back(status_icon->signal_visibility_changed().connect(sigc::mem_fun(*this, &GUI::on_visibility_changed)));
+
+  process_visibility();
   
 #ifdef HAVE_DBUS
   DBus *dbus = CoreFactory::get_dbus();
@@ -842,10 +802,10 @@ GUI::init_gui()
       dbus->connect("/org/workrave/Workrave/UI",
                     "org.workrave.ControlInterface",
                     menus);
-      
+
     }
 #endif
-  
+
 #if defined(PLATFORM_OS_WIN32)
   win32_init_filter();
 #endif
@@ -863,7 +823,12 @@ GUI::init_dbus()
 
   if (dbus != NULL && dbus->is_available())
     {
+#ifdef HAVE_DBUS_GIO
+      if (dbus->is_running("org.workrave.Workrave"))
+#else      
+      dbus->register_service("org.workrave.Workrave");
       if (!dbus->is_owner())
+#endif
         {
           Gtk::MessageDialog dialog(_("Workrave failed to start"),
                                     false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
@@ -872,19 +837,21 @@ GUI::init_dbus()
           dialog.run();
           exit(1);
         }
-      
+
       try
         {
-          dbus->register_service("org.workrave.Workrave.Activator");
           dbus->register_object_path("/org/workrave/Workrave/UI");
-
+#ifdef HAVE_DBUS_GIO
+          dbus->register_service("org.workrave.Workrave");
+#endif
+          
           extern void init_DBusGUI(DBus *dbus);
           init_DBusGUI(dbus);
         }
       catch (DBusException &)
         {
         }
-    }  
+    }
 #endif
 }
 
@@ -931,7 +898,7 @@ GUI::init_sound_player()
     {
       // Tell pulseaudio were are playing sound events
       g_setenv("PULSE_PROP_media.role", "event", TRUE);
-      
+
       sound_player = new SoundPlayer(); /* LEAK */
       sound_player->init();
     }
@@ -976,7 +943,7 @@ GUI::core_event_notify(const CoreEvent event)
       string msg = _("Workrave could not monitor your keyboard and mouse activity.\n");
 
 #ifdef PLATFORM_OS_UNIX
-      msg += _("Make sure that the RECORD extention is enabled in the X server.");
+      msg += _("Make sure that the RECORD extension is enabled in the X server.");
 #endif
       Gtk::MessageDialog dialog(_("Workrave failed to start"),
                                 false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
@@ -996,7 +963,14 @@ GUI::core_event_operation_mode_changed(const OperationMode m)
     {
       status_icon->set_operation_mode(m);
     }
-  
+
+  menus->resync();
+}
+
+void
+GUI::core_event_usage_mode_changed(const UsageMode m)
+{
+  (void) m;
   menus->resync();
 }
 
@@ -1004,7 +978,7 @@ void
 GUI::config_changed_notify(const std::string &key)
 {
   TRACE_ENTER_MSG("GUI::config_changed_notify", key);
-  
+
 #if defined(HAVE_LANGUAGE_SELECTION)
   if (key == GUIConfig::CFG_KEY_LOCALE)
     {
@@ -1014,29 +988,7 @@ GUI::config_changed_notify(const std::string &key)
       menus->locale_changed();
     }
 #endif
-  if (key == GUIConfig::CFG_KEY_TRAYICON_ENABLED)
-    {
-      if (status_icon != NULL)
-        {
-          bool tray = GUIConfig::get_trayicon_enabled();
-          status_icon->set_visible(tray);
 
-          if (!tray)
-            {
-              TimerBoxControl::set_enabled("main_window", true);
-              
-              AppletControl *applet_control = get_applet_control();
-              if (applet_control != NULL)
-                {
-                  if (!applet_control->is_visible())
-                    {
-                      open_main_window();
-                    }
-                }
-            }
-        }
-    }
-    
   TRACE_EXIT();
 }
 
@@ -1048,7 +1000,6 @@ GUI::set_break_response(IBreakResponse *rep)
 }
 
 
-
 void
 GUI::create_prelude_window(BreakId break_id)
 {
@@ -1060,7 +1011,6 @@ GUI::create_prelude_window(BreakId break_id)
   for (int i = 0; i < num_heads; i++)
     {
       prelude_windows[i] = new PreludeWindow(heads[i], break_id);
-      prelude_windows[i]->set_response(response);
     }
 
   active_prelude_count = num_heads;
@@ -1077,16 +1027,29 @@ GUI::create_break_window(BreakId break_id, BreakHint break_hint)
 
   BreakWindow::BreakFlags break_flags = BreakWindow::BREAK_FLAGS_NONE;
   bool ignorable = GUIConfig::get_ignorable(break_id);
+  bool skippable = GUIConfig::get_skippable(break_id);
 
-  if ( (break_hint & BREAK_HINT_USER_INITIATED) && !ignorable)
-    {
+  if (break_hint & BREAK_HINT_USER_INITIATED)
+  {
       break_flags = ( BreakWindow::BREAK_FLAGS_POSTPONABLE |
                       BreakWindow::BREAK_FLAGS_USER_INITIATED);
+
+      if (skippable)
+        {
+          break_flags |=  BreakWindow::BREAK_FLAGS_SKIPPABLE;
+        }
     }
-  else if (ignorable)
-    {
-      break_flags =  ( BreakWindow::BREAK_FLAGS_POSTPONABLE |
-                       BreakWindow::BREAK_FLAGS_SKIPPABLE);
+  else
+    { 
+      if (ignorable)
+        {
+          break_flags |= BreakWindow::BREAK_FLAGS_POSTPONABLE;
+        }
+
+      if(skippable)
+        {
+          break_flags |= BreakWindow::BREAK_FLAGS_SKIPPABLE;
+        }
     }
 
   if (break_hint & BREAK_HINT_NATURAL_BREAK)
@@ -1094,7 +1057,7 @@ GUI::create_break_window(BreakId break_id, BreakHint break_hint)
       break_flags |=  (BreakWindow::BREAK_FLAGS_NO_EXERCISES | BreakWindow::BREAK_FLAGS_NATURAL |
                        BreakWindow::BREAK_FLAGS_POSTPONABLE);
     }
-  
+
   active_break_id = break_id;
 
   for (int i = 0; i < num_heads; i++)
@@ -1173,7 +1136,7 @@ GUI::show_break_window()
     {
       grab();
     }
-  
+
   TRACE_EXIT();
 }
 
@@ -1390,12 +1353,14 @@ GUI::on_operational_mode_warning_timer()
   OperationMode mode = core->get_operation_mode();
   if (mode == OPERATION_MODE_SUSPENDED)
     {
-      status_icon->show_balloon(_("Workrave is in suspended mode. "
+      status_icon->show_balloon("operation_mode",
+                                _("Workrave is in suspended mode. "
                                   "Mouse and keyboard activity will not be monitored."));
     }
   else if (mode == OPERATION_MODE_QUIET)
     {
-      status_icon->show_balloon(_("Workrave is in quiet mode. "
+      status_icon->show_balloon("operation_mode",
+                                _("Workrave is in quiet mode. "
                                   "No break windows will appear."));
     }
   return false;
@@ -1478,7 +1443,7 @@ GUI::bound_head(int &x, int &y, int width, int height, int &head)
     {
       head = 0;
     }
-  
+
   HeadInfo &h = get_head(head);
   if (x < - h.get_width())
     {
@@ -1540,12 +1505,12 @@ GUI::get_timers_tooltip()
     case OPERATION_MODE_NORMAL:
     default:
 #if !defined(PLATFORM_OS_WIN32)
-          // Win32 tip is limited in length
+      // Win32 tip is limited in length
       tip = "Workrave";
-#endif      
+#endif
       break;
     }
-  
+
   for (int count = 0; count < BREAK_ID_SIZEOF; count++)
     {
       IBreak *b = core->get_break(BreakId(count));
@@ -1581,35 +1546,59 @@ GUI::get_timers_tooltip()
   return tip;
 }
 
-
-bool
-GUI::is_status_icon_visible() const
+void
+GUI::on_status_icon_balloon_activate(const std::string &id)
 {
-  bool ret = false;
-
-  if (status_icon != NULL)
+  if (id == "closewarn")
     {
-      ret = status_icon->is_embedded();
+      CoreFactory::get_configurator()->set_value(GUIConfig::CFG_KEY_CLOSEWARN_ENABLED, false);
     }
-
-  return ret;
 }
 
+void
+GUI::on_status_icon_activate()
+{
+  main_window->toggle_window();
+}
+
+void
+GUI::on_visibility_changed()
+{
+  TRACE_ENTER("GUI::on_visibility_changed");
+  process_visibility();
+  TRACE_EXIT();
+}
+
+void
+GUI::process_visibility()
+{
+  TRACE_ENTER("GUI::process_visibility");
+  TRACE_MSG(main_window->is_visible() << " " << applet_control->is_visible() << " " << status_icon->is_visible());
+#ifdef PLATFORM_OS_WIN32
+  if (!main_window->is_visible() && !applet_control->is_visible())
+    {
+      GUIConfig::set_trayicon_enabled(true);
+    }
+#else
+  bool can_close_main_window = applet_control->is_visible() || status_icon->is_visible();
+  main_window->set_can_close(can_close_main_window);
+#endif
+  TRACE_EXIT();
+}
 
 #if defined(PLATFORM_OS_WIN32)
 void
 GUI::win32_init_filter()
 {
   GtkWidget *window = (GtkWidget *)main_window->gobj();
-  GdkWindow *gdk_window = window->window;
+  GdkWindow *gdk_window = gtk_widget_get_window(window);
   gdk_window_add_filter(gdk_window, win32_filter_func, this);
 
   HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
-  
+
   WTSRegisterSessionNotification(hwnd, NOTIFY_FOR_THIS_SESSION);
 }
 
-
 GdkFilterReturn
 GUI::win32_filter_func (void     *xevent,
                         GdkEvent *event,
@@ -1635,7 +1624,7 @@ GUI::win32_filter_func (void     *xevent,
           }
       }
       break;
-      
+
     case WM_POWERBROADCAST:
       {
         TRACE_MSG("WM_POWERBROADCAST " << msg->wParam << " " << msg->lParam);
@@ -1656,10 +1645,7 @@ GUI::win32_filter_func (void     *xevent,
             {
               TRACE_MSG("Resume suspend");
               ICore *core = CoreFactory::get_core();
-              if (core != NULL)
-                {
-                  core->set_powersave(false);
-                }
+              core->set_powersave(false);
             }
             break;
 
@@ -1667,10 +1653,7 @@ GUI::win32_filter_func (void     *xevent,
             {
               TRACE_MSG("Suspend");
               ICore *core = CoreFactory::get_core();
-              if (core != NULL)
-                {
-                  core->set_powersave(true);
-                }
+              core->set_powersave(true);
             }
             break;
           }
@@ -1713,5 +1696,4 @@ GUI::win32_filter_func (void     *xevent,
   return ret;
 }
 
-
 #endif
