@@ -1,6 +1,6 @@
 // GUIConfig.cc --- The WorkRave GUI Configuration
 //
-// Copyright (C) 2007, 2008, 2010, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2007, 2008, 2010, 2011, 2012 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 using namespace std;
 
 const string GUIConfig::CFG_KEY_BREAK_IGNORABLE    = "gui/breaks/%b/ignorable_break";
+const string GUIConfig::CFG_KEY_BREAK_SKIPPABLE    = "gui/breaks/%b/skippable_break";
 const string GUIConfig::CFG_KEY_BREAK_EXERCISES    = "gui/breaks/%b/exercises";
 const string GUIConfig::CFG_KEY_BREAK_AUTO_NATURAL = "gui/breaks/%b/auto_natural";
 const string GUIConfig::CFG_KEY_BLOCK_MODE         = "gui/breaks/block_mode";
@@ -41,6 +42,14 @@ const string GUIConfig::CFG_KEY_LOCALE             = "gui/locale";
 const string GUIConfig::CFG_KEY_TRAYICON_ENABLED   = "gui/trayicon_enabled";
 const string GUIConfig::CFG_KEY_CLOSEWARN_ENABLED  = "gui/closewarn_enabled";
 
+const string GUIConfig::CFG_KEY_MAIN_WINDOW               = "gui/main_window";
+const string GUIConfig::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP = "gui/main_window/always_on_top";
+const string GUIConfig::CFG_KEY_MAIN_WINDOW_START_IN_TRAY = "gui/main_window/start_in_tray";
+const string GUIConfig::CFG_KEY_MAIN_WINDOW_X             = "gui/main_window/x";
+const string GUIConfig::CFG_KEY_MAIN_WINDOW_Y             = "gui/main_window/y";
+const string GUIConfig::CFG_KEY_MAIN_WINDOW_HEAD          = "gui/main_window/head";
+
+
 //!
 void
 GUIConfig::init()
@@ -49,17 +58,32 @@ GUIConfig::init()
 
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
-      config->set_value(CFG_KEY_BREAK_IGNORABLE % ((BreakId)i),
+      BreakId breakId = (BreakId)i;
+      
+      config->set_value(CFG_KEY_BREAK_IGNORABLE % breakId,
                         true,
                         CONFIG_FLAG_DEFAULT);
-      
-      config->set_value(CFG_KEY_BREAK_EXERCISES % ((BreakId)i),
+
+      config->set_value(CFG_KEY_BREAK_EXERCISES % breakId,
                         i == BREAK_ID_REST_BREAK ? 3 : 0,
                         CONFIG_FLAG_DEFAULT);
 
-      config->set_value(CFG_KEY_BREAK_AUTO_NATURAL % ((BreakId)i),
+      config->set_value(CFG_KEY_BREAK_AUTO_NATURAL % breakId,
                         false,
                         CONFIG_FLAG_DEFAULT);
+
+      // for backward compatibility with settings of older versions, we set the default
+      // default value of `skippable` to whatever `ignorable`. This works because the old
+      // meaning of `ignorable` was "show postpone and skip"; the new meaning is
+      // "show postpone".
+      bool ignorable;
+      config->get_value_with_default(CFG_KEY_BREAK_IGNORABLE % breakId, 
+                                     ignorable, 
+                                     true);
+
+      config->set_value(CFG_KEY_BREAK_SKIPPABLE % breakId, 
+                        ignorable, 
+                        CONFIG_FLAG_DEFAULT);
     }
 
   config->set_value(CFG_KEY_BLOCK_MODE, BLOCK_MODE_INPUT, CONFIG_FLAG_DEFAULT);
@@ -81,6 +105,18 @@ GUIConfig::get_ignorable(BreakId id)
 }
 
 
+//!
+bool
+GUIConfig::get_skippable(BreakId id)
+{
+  bool rc;
+  CoreFactory::get_configurator()
+    ->get_value_with_default(CFG_KEY_BREAK_SKIPPABLE % id,
+                             rc,
+                             true);
+  return rc;
+}
+
 //!
 void
 GUIConfig::set_ignorable(BreakId id, bool b)
@@ -181,3 +217,42 @@ GUIConfig::expand(const string &key, BreakId id)
 
   return str;
 }
+
+bool
+GUIConfig::get_always_on_top()
+{
+  bool rc;
+  CoreFactory::get_configurator()
+    ->get_value_with_default(GUIConfig::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP,
+                             rc,
+                             false);
+  return rc;
+}
+
+
+void
+GUIConfig::set_always_on_top(bool b)
+{
+  CoreFactory::get_configurator()
+    ->set_value(GUIConfig::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP, b);
+}
+
+
+bool
+GUIConfig::get_start_in_tray()
+{
+  bool rc;
+  CoreFactory::get_configurator()
+    ->get_value_with_default(CFG_KEY_MAIN_WINDOW_START_IN_TRAY, rc, false);
+  return rc;
+}
+
+
+void
+GUIConfig::set_start_in_tray(bool b)
+{
+  CoreFactory::get_configurator()
+    ->set_value(CFG_KEY_MAIN_WINDOW_START_IN_TRAY, b);
+}
+
+
