@@ -1,6 +1,6 @@
 // MainWindow.cc --- Main info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -35,49 +35,26 @@
 #include <unistd.h>
 #endif
 
-#include <iostream>
-
-#include "AppletControl.hh"
-
-#include "TimerBoxGtkView.hh"
-#include "TimerBoxControl.hh"
 #include "MainWindow.hh"
-#include "PreferencesDialog.hh"
-#include "StatisticsDialog.hh"
-#include "WindowHints.hh"
-#include "TimeBar.hh"
-#include "GUI.hh"
-#include "Util.hh"
-#include "Text.hh"
+
+#include <list>
+#include <gtkmm.h>
 
 #include "CoreFactory.hh"
 #include "IConfigurator.hh"
-#include "IStatistics.hh"
-
-#ifdef HAVE_DISTRIBUTION
-#include "NetworkJoinDialog.hh"
-#include "NetworkLogDialog.hh"
-#endif
 
+#include "TimerBoxGtkView.hh"
+#include "TimerBoxControl.hh"
+#include "GUI.hh"
+#include "Util.hh"
 #include "Menus.hh"
 
-#include <gtkmm/menu.h>
-#include <gtkmm/stock.h>
-
-#include "gtk/gtkmenu.h"
-
-const string MainWindow::CFG_KEY_MAIN_WINDOW               = "gui/main_window";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP = "gui/main_window/always_on_top";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_START_IN_TRAY = "gui/main_window/start_in_tray";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_X             = "gui/main_window/x";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_Y             = "gui/main_window/y";
-const string MainWindow::CFG_KEY_MAIN_WINDOW_HEAD          = "gui/main_window/head";
-
 #ifdef PLATFORM_OS_WIN32
 const char *WIN32_MAIN_CLASS_NAME = "Workrave";
 const UINT MYWM_TRAY_MESSAGE = WM_USER +0x100;
 #endif
 
+using namespace std;
 
 //! Constructor.
 /*!
@@ -86,11 +63,9 @@ const UINT MYWM_TRAY_MESSAGE = WM_USER +0x100;
  */
 MainWindow::MainWindow() :
   enabled(true),
+  can_close(true),
   timer_box_control(NULL),
   timer_box_view(NULL),
-  monitor_suspended(false),
-  visible(true),
-  applet_active(false),
   window_location(-1, -1),
   window_head_location(-1, -1),
   window_relocated_location(-1, -1)
@@ -101,7 +76,6 @@ MainWindow::MainWindow() :
 #ifdef PLATFORM_OS_WIN32
   show_retry_count = 0;
 #endif
-  init();
 }
 
 
@@ -109,6 +83,19 @@ MainWindow::MainWindow() :
 MainWindow::~MainWindow()
 {
   TRACE_ENTER("MainWindow::~MainWindow");
+
+  if (visible_connection.connected())
+    {
+      visible_connection.disconnect();
+    }
+
+#ifdef PLATFORM_OS_WIN32
+  if (timeout_connection.connected())
+    {
+      timeout_connection.disconnect();
+    }
+#endif
+
   delete timer_box_control;
 #ifdef PLATFORM_OS_WIN32
   win32_exit();
@@ -120,6 +107,142 @@ MainWindow::~MainWindow()
   TRACE_EXIT();
 }
 
+bool
+MainWindow::is_visible() const
+{
+#if defined(PLATFORM_OS_WIN32)
+  const GtkWidget *window = Gtk::Widget::gobj();
+  GdkWindow *gdk_window = gtk_widget_get_window(GTK_WIDGET(window));
+  HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
+  return IsWindowVisible(hwnd);
+#elif defined(HAVE_GTK3)
+  return get_visible();
+#else
+  return Gtk::Window::is_visible();
+#endif
+}
+
+
+void
+MainWindow::toggle_window()
+{
+  TRACE_ENTER("MainWindow::toggle_window");
+
+  bool visible = is_visible();
+  if (visible)
+    {
+      close_window();
+    }
+  else
+    {
+      open_window();
+    }
+  TRACE_EXIT();
+}
+
+
+//! Opens the main window.
+void
+MainWindow::open_window()
+{
+  TRACE_ENTER("MainWindow::open_window");
+  if (timer_box_view->get_visible_count() > 0)
+    {
+#ifdef PLATFORM_OS_WIN32
+      win32_show(true);
+      show_all();
+#else
+      show_all();
+      deiconify();
+#endif
+
+      int x, y, head;
+      set_position(Gtk::WIN_POS_NONE);
+      set_gravity(Gdk::GRAVITY_STATIC);
+      get_start_position(x, y, head);
+
+#ifdef HAVE_GTK3
+      GtkRequisition min_size;
+      GtkRequisition natural_size;
+      get_preferred_size(min_size, natural_size);
+#else
+      GtkRequisition min_size;
+      on_size_request(&min_size);
+#endif
+
+      GUI::get_instance()->bound_head(x, y, min_size.width, min_size.height, head);
+
+      GUI::get_instance()->map_from_head(x, y, head);
+      TRACE_MSG("moving to " << x << " " << y);
+      move(x, y);
+
+      bool always_on_top = GUIConfig::get_always_on_top();
+      WindowHints::set_always_on_top(this, always_on_top);
+      TimerBoxControl::set_enabled("main_window", true);
+    }
+  TRACE_EXIT();
+}
+
+
+
+//! Closes the main window.
+void
+MainWindow::close_window()
+{
+  TRACE_ENTER("MainWindow::close_window");
+#ifdef PLATFORM_OS_WIN32
+  win32_show(false);
+#elif defined(PLATFORM_OS_OSX)
+  hide_all();
+#else
+  if (can_close)
+    {
+      hide();
+    }
+  else
+    {
+      iconify();
+    }
+#endif
+
+  TimerBoxControl::set_enabled("main_window", false);
+  TRACE_EXIT();
+}
+
+
+void
+MainWindow::set_can_close(bool can_close)
+{
+  TRACE_ENTER_MSG("MainWindow::set_can_close", can_close);
+  this->can_close = can_close;
+
+  TRACE_MSG(enabled);
+  if (!enabled)
+    {
+      if (can_close)
+        { 
+          TRACE_MSG("hide");
+          hide();
+        }
+      else
+        {
+          TRACE_MSG("iconify");
+          iconify();
+          show_all();
+        }
+    }
+  TRACE_EXIT();
+}
+
+
+//! Updates the main window.
+void
+MainWindow::update()
+{
+  timer_box_control->update();
+}
+
+
 //! Initializes the main window.
 void
 MainWindow::init()
@@ -131,9 +254,16 @@ MainWindow::init()
 
   list<Glib::RefPtr<Gdk::Pixbuf> > icons;
 
-  const char *icon_files[] = { "workrave-icon-small.png",
-                               "workrave-icon-medium.png",
-                               "workrave-icon-large.png" };
+  const char *icon_files[] = {
+    "scalable/apps/workrave.svg",
+                               "16x16/apps/workrave.png",
+                               "24x24/apps/workrave.png",
+                               "32x32/apps/workrave.png",
+                               "48x48/apps/workrave.png",
+                               "64x64/apps/workrave.png",
+                               "96x96/apps/workrave.png",
+                               "128x128/apps/workrave.png",
+                               };
 
   for (unsigned int i = 0; i < sizeof(icon_files) / sizeof(char *); i++)
     {
@@ -142,6 +272,8 @@ MainWindow::init()
       try
         {
           Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create_from_file(file);
+
+          TRACE_MSG(file << " size = " << pixbuf->get_width() << " " << pixbuf->get_height());
           icons.push_back(pixbuf);
         }
       catch (...)
@@ -151,6 +283,7 @@ MainWindow::init()
 
   Glib::ListHandle<Glib::RefPtr<Gdk::Pixbuf> > icon_list(icons);
   Gtk::Window::set_default_icon_list(icon_list);
+  //Gtk::Window::set_default_icon_name("workrave");
 
   enabled = TimerBoxControl::is_enabled("main_window");
 
@@ -246,13 +379,11 @@ MainWindow::init()
   IConfigurator *config = CoreFactory::get_configurator();
   config->add_listener(TimerBoxControl::CFG_KEY_TIMERBOX + "main_window", this);
 
+  visible_connection = property_visible().signal_changed().connect(sigc::mem_fun(*this, &MainWindow::on_visibility_changed));
 
   TRACE_EXIT();
 }
 
-
-
-
 //! Setup configuration settings.
 void
 MainWindow::setup()
@@ -260,8 +391,9 @@ MainWindow::setup()
   TRACE_ENTER("MainWindow::setup");
 
   bool new_enabled = TimerBoxControl::is_enabled("main_window");
-  bool always_on_top = get_always_on_top();
+  bool always_on_top = GUIConfig::get_always_on_top();
 
+  TRACE_MSG("can_close " << new_enabled);
   TRACE_MSG("enabled " << new_enabled);
   TRACE_MSG("on top " << always_on_top);
 
@@ -278,116 +410,18 @@ MainWindow::setup()
         }
     }
 
-  if (is_visible())
-    {
-      WindowHints::set_always_on_top(this, always_on_top);
-    }
-  
-  if (is_visible() && always_on_top)
-    {
-      raise();
-    }
-  
-  TRACE_EXIT();
-}
-
-
-//! Updates the main window.
-void
-MainWindow::update()
-{
-  timer_box_control->update();
-}
-
-
-void
-MainWindow::on_activate()
-{
-  TRACE_ENTER("MainWindow::on_activate");
-
-#ifdef PLATFORM_OS_WIN32  
-  GtkWidget *window = Gtk::Widget::gobj();
-  GdkWindow *gdk_window = window->window;
-  HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
-  
-  bool visible = IsWindowVisible(hwnd);
-#else
-  bool visible = GTK_WIDGET_VISIBLE(GTK_WIDGET(gobj()));
-#endif
+  bool visible = is_visible();
 
   if (visible)
     {
-      close_window();
-    }
-  else
-    {
-      open_window();
-    }
-  TRACE_EXIT();
-}
-
-
-
-//! Opens the main window.
-void
-MainWindow::open_window()
-{
-  TRACE_ENTER("MainWindow::open_window");
-  if (timer_box_view->get_visible_count() > 0)
-    {
-#ifdef PLATFORM_OS_WIN32
-      win32_show(true);
-      show_all();
-#else
-      show_all();
-      deiconify();
-#endif
-
-      int x, y, head;
-      set_position(Gtk::WIN_POS_NONE);
-      set_gravity(Gdk::GRAVITY_STATIC);
-      get_start_position(x, y, head);
-
-      GtkRequisition req;
-      on_size_request(&req);
-      GUI::get_instance()->bound_head(x, y, req.width, req.height, head);
-
-      GUI::get_instance()->map_from_head(x, y, head);
-      TRACE_MSG("moving to " << x << " " << y);
-      move(x, y);
-
-      bool always_on_top = get_always_on_top();
       WindowHints::set_always_on_top(this, always_on_top);
-
-      TimerBoxControl::set_enabled("main_window", true);
     }
-  TRACE_EXIT();
-}
 
-
-
-//! Closes the main window.
-void
-MainWindow::close_window()
-{
-  TRACE_ENTER("MainWindow::close_window");
-#ifdef PLATFORM_OS_WIN32
-  win32_show(false);
-#elif defined(PLATFORM_OS_OSX)
-  hide_all();
-#else
-  GUI *gui = GUI::get_instance();
-
-  if (applet_active || gui->is_status_icon_visible())
+  if (visible && always_on_top)
     {
-      hide_all();
+      raise();
     }
-  else
-    {
-      iconify();
-    }
-#endif
-  GUIConfig::set_trayicon_enabled(true);
+
   TRACE_EXIT();
 }
 
@@ -397,45 +431,32 @@ bool
 MainWindow::on_delete_event(GdkEventAny *)
 {
   TRACE_ENTER("MainWindow::on_delete_event");
-  GUI *gui = GUI::get_instance();
+  IGUI *gui = GUI::get_instance();
   assert(gui != NULL);
 
-#ifdef PLATFORM_OS_WIN32
+#if defined(PLATFORM_OS_WIN32)
   win32_show(false);
-  gui->main_window_closed();
+  closed_signal.emit();
+  TimerBoxControl::set_enabled("main_window", false);
+#elif defined(PLATFORM_OS_OSX)
+  close_window();
+  TimerBoxControl::set_enabled("main_window", false);
 #else
-#if defined(HAVE_GNOME) || defined(HAVE_KDE)
-  bool terminate = true;
-  AppletControl *applet_control = gui->get_applet_control();
-  if (applet_control != NULL)
-    {
-      terminate = ( !applet_control->is_visible() &&
-                    !gui->is_status_icon_visible() );
-    }
-
-  if (terminate)
-    {
-      gui->terminate();
-    }
-  else
+  if (can_close)
     {
       close_window();
       TimerBoxControl::set_enabled("main_window", false);
     }
-#elif defined(PLATFORM_OS_OSX) 
-  close_window();
-  TimerBoxControl::set_enabled("main_window", false);
-#else
-  gui->terminate();
-#endif // HAVE_GNOME || HAVE_KDE
-#endif // PLATFORM_OS_WIN32
+  else
+    {
+      gui->terminate();
+    }
+#endif
 
   TRACE_EXIT();
   return true;
 }
 
-
-
 //! Users pressed some mouse button in the main window.
 bool
 MainWindow::on_button_press_event(GdkEventButton *event)
@@ -444,18 +465,19 @@ MainWindow::on_button_press_event(GdkEventButton *event)
   bool ret = false;
 
   (void) event;
-  
+
 #ifndef PLATFORM_OS_OSX
   // No popup menu on OS X
-  
+
   if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
     {
-      Menus::get_instance()->popup(Menus::MENU_MAINWINDOW,
-                                   event->button, event->time);
+      IGUI *gui = GUI::get_instance();
+      Menus *menus = gui->get_menus();
+      menus->popup(Menus::MENU_MAINWINDOW, event->button, event->time);
       ret = true;
     }
 #endif
-  
+
   TRACE_EXIT();
   return ret;
 }
@@ -464,9 +486,9 @@ void
 MainWindow::config_changed_notify(const string &key)
 {
   TRACE_ENTER_MSG("MainWindow::config_changed_notify", key);
-  if (key != CFG_KEY_MAIN_WINDOW_HEAD
-      && key != CFG_KEY_MAIN_WINDOW_X
-      && key != CFG_KEY_MAIN_WINDOW_Y)
+  if (key != GUIConfig::CFG_KEY_MAIN_WINDOW_HEAD
+      && key != GUIConfig::CFG_KEY_MAIN_WINDOW_X
+      && key != GUIConfig::CFG_KEY_MAIN_WINDOW_Y)
     {
       setup();
     }
@@ -474,57 +496,20 @@ MainWindow::config_changed_notify(const string &key)
 }
 
 
-bool
-MainWindow::get_always_on_top()
-{
-  bool rc;
-  CoreFactory::get_configurator()
-    ->get_value_with_default(MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP,
-                             rc,
-                             false);
-  return rc;
-}
-
-
-void
-MainWindow::set_always_on_top(bool b)
-{
-  CoreFactory::get_configurator()
-    ->set_value(MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP, b);
-}
-
-
-bool
-MainWindow::get_start_in_tray()
-{
-  bool rc;
-  CoreFactory::get_configurator()
-    ->get_value_with_default(CFG_KEY_MAIN_WINDOW_START_IN_TRAY, rc, false);
-  return rc;
-}
-
-
-void
-MainWindow::set_start_in_tray(bool b)
-{
-  CoreFactory::get_configurator()
-    ->set_value(CFG_KEY_MAIN_WINDOW_START_IN_TRAY, b);
-}
-
-
 #ifdef PLATFORM_OS_WIN32
 void
 MainWindow::win32_show(bool b)
 {
-  TRACE_ENTER("MainWindow::win32_show");
+  TRACE_ENTER_MSG("MainWindow::win32_show", b);
   bool retry = false;
 
   // Gtk's hide() seems to quit the program.
   GtkWidget *window = Gtk::Widget::gobj();
-  GdkWindow *gdk_window = window->window;
+  GdkWindow *gdk_window = gtk_widget_get_window(window);
   HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
   ShowWindow(hwnd, b ? SW_SHOWNORMAL : SW_HIDE);
-
+  visibility_changed_signal.emit();
+  
 	if (b)
 	  {
 		  present();
@@ -541,7 +526,7 @@ MainWindow::win32_show(bool b)
             }
 
           TRACE_MSG("2 " << show_retry_count);
-          retry = true;  
+          retry = true;
         }
     }
 
@@ -549,7 +534,7 @@ MainWindow::win32_show(bool b)
     {
       if (show_retry_count > 0)
         {
-          Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::win32_show_retry), 50);
+          timeout_connection = Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::win32_show_retry), 50);
         }
     }
   else
@@ -615,7 +600,7 @@ MainWindow::win32_init()
 
   // Reassign ownership
   GtkWidget *window = Gtk::Widget::gobj();
-  GdkWindow *gdk_window = window->window;
+  GdkWindow *gdk_window = gtk_widget_get_window(window);
   HWND hwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
   SetWindowLong(hwnd, GWL_HWNDPARENT, (LONG) win32_main_hwnd);
 
@@ -637,9 +622,9 @@ MainWindow::get_start_position(int &x, int &y, int &head)
   TRACE_ENTER("MainWindow::get_start_position");
   // FIXME: Default to right-bottom instead of 256x256
   IConfigurator *cfg = CoreFactory::get_configurator();
-  cfg->get_value_with_default(CFG_KEY_MAIN_WINDOW_X, x, 256);
-  cfg->get_value_with_default(CFG_KEY_MAIN_WINDOW_Y, y, 256);
-  cfg->get_value_with_default(CFG_KEY_MAIN_WINDOW_HEAD, head, 0);
+  cfg->get_value_with_default(GUIConfig::CFG_KEY_MAIN_WINDOW_X, x, 256);
+  cfg->get_value_with_default(GUIConfig::CFG_KEY_MAIN_WINDOW_Y, y, 256);
+  cfg->get_value_with_default(GUIConfig::CFG_KEY_MAIN_WINDOW_HEAD, head, 0);
   if (head < 0)
     {
       head = 0;
@@ -655,9 +640,9 @@ MainWindow::set_start_position(int x, int y, int head)
   TRACE_ENTER_MSG("MainWindow::set_start_position",
                   x << " " << y << " " << head);
   IConfigurator *cfg = CoreFactory::get_configurator();
-  cfg->set_value(CFG_KEY_MAIN_WINDOW_X, x);
-  cfg->set_value(CFG_KEY_MAIN_WINDOW_Y, y);
-  cfg->set_value(CFG_KEY_MAIN_WINDOW_HEAD, head);
+  cfg->set_value(GUIConfig::CFG_KEY_MAIN_WINDOW_X, x);
+  cfg->set_value(GUIConfig::CFG_KEY_MAIN_WINDOW_Y, y);
+  cfg->set_value(GUIConfig::CFG_KEY_MAIN_WINDOW_HEAD, head);
   TRACE_EXIT();
 }
 
@@ -671,17 +656,23 @@ MainWindow::move_to_start_position()
   int x, y, head;
   get_start_position(x, y, head);
 
-  GtkRequisition req;
-  on_size_request(&req);
+#ifdef HAVE_GTK3
+  GtkRequisition min_size;
+  GtkRequisition natural_size;
+  get_preferred_size(min_size, natural_size);
+#else
+  GtkRequisition min_size;
+  on_size_request(&min_size);
+#endif
 
-  GUI::get_instance()->bound_head(x, y, req.width, req.height, head);
+  GUI::get_instance()->bound_head(x, y, min_size.width, min_size.height, head);
 
   window_head_location.set_x(x);
   window_head_location.set_y(y);
 
   GUI::get_instance()->map_from_head(x, y, head);
 
-  TRACE_MSG("Main window size " << req.width << " " << req.height);
+  TRACE_MSG("Main window size " << min_size.width << " " << min_size.height);
 
   window_location.set_x(x);
   window_location.set_y(y);
@@ -692,51 +683,6 @@ MainWindow::move_to_start_position()
   move(x, y);
 }
 
-
-void
-MainWindow::set_applet_active(bool a)
-{
-  TRACE_ENTER_MSG("MainWindow::set_applet_active", a);
-  applet_active = a;
-
-  if (!enabled)
-    {
-      GUI *gui = GUI::get_instance();
-      if (applet_active || gui->is_status_icon_visible())
-        {
-          hide_all();
-        }
-      else
-        {
-          iconify();
-          show_all();
-        }
-    }
-
-  TRACE_EXIT();
-}
-
-
-void
-MainWindow::status_icon_changed()
-{
-  TRACE_ENTER("MainWindow::status_icon_changed");
-  if (!enabled)
-    {
-      GUI *gui = GUI::get_instance();
-      if (applet_active || gui->is_status_icon_visible())
-        {
-          hide_all();
-        }
-      else
-        {
-          iconify();
-          show_all();
-        }
-    }
-  TRACE_EXIT();
-}
-
 bool
 MainWindow::on_configure_event(GdkEventConfigure *event)
 {
@@ -763,7 +709,7 @@ MainWindow::locate_window(GdkEventConfigure *event)
       TRACE_EXIT();
       return;
     }
-  
+
 #ifndef PLATFORM_OS_WIN32
   // Returns bogus results on windows...sometime.
   if (event != NULL)
@@ -780,12 +726,18 @@ MainWindow::locate_window(GdkEventConfigure *event)
 
       get_position(x, y);
 
-      GtkRequisition req;
-      on_size_request(&req);
-      width = req.width;
-      height = req.height;
-    }
+#ifdef HAVE_GTK3
+      GtkRequisition min_size;
+      GtkRequisition natural_size;
+      get_preferred_size(min_size, natural_size);
+#else
+      GtkRequisition min_size;
+      on_size_request(&min_size);
+#endif
 
+      width = min_size.width;
+      height = min_size.height;
+    }
 
   TRACE_MSG("main window = " << x << " " << y);
 
@@ -847,16 +799,23 @@ MainWindow::relocate_window(int width, int height)
       x = window_head_location.get_x();
       y = window_head_location.get_y();
 
-      GUI *gui = GUI::get_instance();
+      IGUI *gui = GUI::get_instance();
       int num_heads = gui->get_number_of_heads();
       for (int i = 0; i < num_heads; i++)
         {
           HeadInfo &head = gui->get_head(i);
           if (head.valid)
             {
-              GtkRequisition req;
-              on_size_request(&req);
-              GUI::get_instance()->bound_head(x, y, req.width, req.height, i);
+#ifdef HAVE_GTK3
+              GtkRequisition min_size;
+              GtkRequisition natural_size;
+              get_preferred_size(min_size, natural_size);
+#else
+              GtkRequisition min_size;
+              on_size_request(&min_size);
+#endif
+
+              GUI::get_instance()->bound_head(x, y, min_size.width, min_size.height, i);
 
               gui->map_from_head(x, y, i);
               break;
@@ -894,3 +853,25 @@ MainWindow::win32_window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
   return DefWindowProc(hwnd, uMsg, wParam, lParam);
 }
 #endif
+
+
+void
+MainWindow::on_visibility_changed()
+{
+  TRACE_ENTER("MainWindow::on_visibility_changed");
+  TRACE_MSG(is_visible());
+  visibility_changed_signal.emit();
+  TRACE_EXIT();
+}
+
+sigc::signal<void> &
+MainWindow::signal_closed()
+{
+  return closed_signal;
+}
+
+sigc::signal<void> &
+MainWindow::signal_visibility_changed()
+{
+  return visibility_changed_signal;
+}
