@@ -1,6 +1,6 @@
 // Menus.cc --- Timer info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -44,8 +44,6 @@
 #include "ICore.hh"
 #include "IConfigurator.hh"
 
-#include "MainWindow.hh"
-
 #ifdef HAVE_DISTRIBUTION
 #include "IDistributionManager.hh"
 #include "NetworkJoinDialog.hh"
@@ -58,12 +56,21 @@
 #endif
 
 #include "MainGtkMenu.hh"
+#include "AppletControl.hh"
 
-#ifdef HAVE_GNOMEAPPLET
+#ifdef HAVE_PANELAPPLET2
 #include "GnomeAppletMenu.hh"
 #include "GnomeAppletWindow.hh"
 #endif
 
+#ifdef HAVE_INDICATOR
+#include "IndicatorAppletMenu.hh"
+#endif
+
+#ifdef HAVE_DBUS
+#include "GenericDBusApplet.hh"
+#endif
+
 #ifdef PLATFORM_OS_WIN32
 #include <shellapi.h>
 
@@ -76,8 +83,6 @@
 #include "OSXGtkMenu.hh"
 #endif
 
-Menus *Menus::instance = 0;
-
 //! Constructor.
 /*!
  *  \param gui the main GUI entry point.
@@ -93,12 +98,8 @@ Menus::Menus() :
 #ifdef HAVE_EXERCISES
   exercises_dialog(NULL),
 #endif
-  main_window(NULL),
-  applet_window(NULL),
   about(NULL)
 {
-  assert(instance == 0);
-  instance = this;
   gui = GUI::get_instance();
 
   for (int i = 0; i < MENU_SIZEOF; i++)
@@ -112,45 +113,49 @@ Menus::Menus() :
 Menus::~Menus()
 {
   TRACE_ENTER("Menus::~Menus");
-  instance = 0;
   TRACE_EXIT();
 }
 
 
-Menus *
-Menus::get_instance()
-{
-  assert(instance != 0);
-  return instance;
-}
-
-
 void
-Menus::init(MainWindow *main_window, AppletWindow *applet_window)
+Menus::init(AppletControl *applet_control)
 {
-  this->main_window = main_window;
-  this->applet_window = applet_window;
-  
-#ifdef PLATFORM_OS_WIN32
-  menus[MENU_APPLET] = new W32TrayMenu();
-#else
-  menus[MENU_APPLET] = new MainGtkMenu(true);
-#endif
+  IAppletWindow *applet_window = NULL;
 
-#if defined(PLATFORM_OS_WIN32)
-  W32AppletWindow *w32_applet_window = dynamic_cast<W32AppletWindow*>(applet_window);
-  menus[MENU_NATIVE] = new W32AppletMenu(main_window, w32_applet_window);
-#elif defined(HAVE_GNOMEAPPLET)
-  GnomeAppletWindow *gnome_applet_window = dynamic_cast<GnomeAppletWindow*>(applet_window);
-  menus[MENU_NATIVE] = new GnomeAppletMenu(gnome_applet_window);
-#endif
-  
 #if defined(PLATFORM_OS_OSX)
   menus[MENU_MAINWINDOW] = new OSXGtkMenu(true);
 #else
   menus[MENU_MAINWINDOW] = new MainGtkMenu(false);
-#endif      
+#endif
 
+#ifdef PLATFORM_OS_WIN32
+  menus[MENU_MAINAPPLET] = new W32TrayMenu();
+#else
+  menus[MENU_MAINAPPLET] = new MainGtkMenu(true);
+#endif
+
+#if defined(PLATFORM_OS_WIN32)
+  applet_window = applet_control->get_applet_window(AppletControl::APPLET_W32);
+  W32AppletWindow *w32_applet_window = dynamic_cast<W32AppletWindow*>(applet_window);
+  menus[MENU_APPLET_W32] = new W32AppletMenu(w32_applet_window);
+#endif
+
+#if defined(HAVE_PANELAPPLET2)
+  applet_window = applet_control->get_applet_window(AppletControl::APPLET_GNOME);
+  GnomeAppletWindow *gnome_applet_window = dynamic_cast<GnomeAppletWindow*>(applet_window);
+  menus[MENU_APPLET_GNOME] = new GnomeAppletMenu(gnome_applet_window);
+#endif
+
+#if defined(HAVE_DBUS_GIO)
+  applet_window = applet_control->get_applet_window(AppletControl::APPLET_GENERIC_DBUS);
+  GenericDBusApplet *indicator_applet = dynamic_cast<GenericDBusApplet*>(applet_window);
+  menus[MENU_APPLET_GENERICDBUS] = indicator_applet;
+
+#if defined(HAVE_INDICATOR)
+  menus[MENU_APPLET_INDICATOR] = new IndicatorAppletMenu();
+#endif
+#endif
+  
   for (int i = 0; i < MENU_SIZEOF; i++)
     {
       if (menus[i] != NULL)
@@ -159,11 +164,6 @@ Menus::init(MainWindow *main_window, AppletWindow *applet_window)
         }
     }
 
-  if (main_window != NULL)
-    {
-      menus[MENU_MAINWINDOW]->add_accel(*main_window);
-    }
-
   resync();
 }
 
@@ -173,13 +173,14 @@ Menus::popup(const MenuKind kind,
              const guint button,
              const guint activate_time)
 {
-  Menu *pop_menu = menus[kind];
+  IMenu *pop_menu = menus[kind];
   if (pop_menu)
     {
       pop_menu->popup(button, activate_time);
     }
 }
 
+
 void
 Menus::on_menu_open_main_window()
 {
@@ -211,8 +212,8 @@ void
 Menus::set_operation_mode(OperationMode m)
 {
   ICore *core = CoreFactory::get_core();
-  core->set_operation_mode(m, true);
-  resync();  
+  core->set_operation_mode(m);
+  resync();
 }
 
 
@@ -220,8 +221,8 @@ void
 Menus::set_usage_mode(UsageMode m)
 {
   ICore *core = CoreFactory::get_core();
-  core->set_usage_mode(m, true);
-  resync();  
+  core->set_usage_mode(m);
+  resync();
 }
 
 
@@ -234,7 +235,6 @@ Menus::on_menu_quiet()
 }
 
 
-
 void
 Menus::on_menu_suspend()
 {
@@ -306,7 +306,7 @@ Menus::on_exercises_response(int response)
   (void) response;
 
   assert(exercises_dialog != NULL);
-  exercises_dialog->hide_all();
+  exercises_dialog->hide();
 
   delete exercises_dialog;
   exercises_dialog = NULL;
@@ -359,14 +359,24 @@ Menus::on_menu_about()
       about = new Gtk::AboutDialog;
 
       about->set_name("Workrave");
+#ifdef HAVE_GTK3
+      std::vector<Glib::ustring> authors;
+      for (int index = 0; workrave_authors[index] != NULL; index++)
+        {
+          authors.push_back(workrave_authors[index]);
+        }
+      about->set_authors(authors);
+#else
       about->set_authors(workrave_authors);
+#endif
+
       about->set_copyright(workrave_copyright);
       about->set_comments(_("This program assists in the prevention and recovery"
                             " of Repetitive Strain Injury (RSI)."));
       about->set_logo(pixbuf);
       about->set_translator_credits(workrave_translators);
 
-#ifdef PLATFORM_OS_WIN32
+#if defined(PLATFORM_OS_WIN32) && !defined(HAVE_GTK3)
       about->set_url_hook(sigc::mem_fun(*this, &Menus::on_about_link_activate));
 #endif
 
@@ -422,7 +432,7 @@ Menus::on_network_join_response(int response)
   (void) response;
 
   assert(network_join_dialog != NULL);
-  network_join_dialog->hide_all();
+  network_join_dialog->hide();
 
   if (response == Gtk::RESPONSE_OK)
     {
@@ -486,10 +496,10 @@ Menus::on_menu_network_log(bool active)
     }
   else if (network_log_dialog != NULL)
     {
-      network_log_dialog->hide_all();
+      network_log_dialog->hide();
       delete network_log_dialog;
       network_log_dialog = NULL;
-      resync();  
+      resync();
     }
 
 
@@ -507,12 +517,12 @@ Menus::on_network_log_response(int response)
 
   assert(network_log_dialog != NULL);
 
-  network_log_dialog->hide_all();
+  network_log_dialog->hide();
 
   // done by gtkmm ??? delete network_log_dialog;
   network_log_dialog = NULL;
 
-  resync();  
+  resync();
 
   TRACE_EXIT();
 }
@@ -524,7 +534,7 @@ Menus::on_statistics_response(int response)
   (void) response;
 
   assert(statistics_dialog != NULL);
-  statistics_dialog->hide_all();
+  statistics_dialog->hide();
 
   delete statistics_dialog;
   statistics_dialog = NULL;
@@ -536,7 +546,7 @@ Menus::on_preferences_response(int response)
   (void) response;
 
   assert(preferences_dialog != NULL);
-  preferences_dialog->hide_all();
+  preferences_dialog->hide();
 
   CoreFactory::get_configurator()->save();
 
@@ -544,6 +554,7 @@ Menus::on_preferences_response(int response)
   preferences_dialog = NULL;
 }
 
+
 void
 Menus::applet_command(short cmd)
 {
@@ -598,6 +609,9 @@ Menus::applet_command(short cmd)
         on_menu_reading(core->get_usage_mode() == USAGE_MODE_NORMAL);
       }
       break;
+    case MENU_COMMAND_QUIT:
+      on_menu_quit();
+      break;
     }
 }
 
@@ -609,15 +623,21 @@ Menus::resync()
   if (syncing)
     return;
   syncing = true;
-  TRACE_ENTER("Menus::resync");
-  
+
   for (int i = 0; i < MENU_SIZEOF; i++)
     {
       if (menus[i] != NULL)
         {
           ICore *core = CoreFactory::get_core();
 
-          menus[i]->resync(core->get_operation_mode(),
+          /* Use operation_mode_regular here to show the mode that will be restored 
+          if an override is in place. That is also necessary because if get_operation_mode() 
+          were called instead then that would cause a mode that's possibly an override to 
+          be returned. If that then the override mode menu item would be set_active() during 
+          the resync, and cause a signal that then calls back into core with the override mode 
+          as a regular mode. That would erase whatever the user's regular mode was.
+          */
+          menus[i]->resync(core->get_operation_mode_regular(),
                            core->get_usage_mode(),
 #ifdef HAVE_DISTRIBUTION
                            network_log_dialog != NULL);
@@ -629,7 +649,6 @@ Menus::resync()
     }
 
   syncing = false;
-  TRACE_EXIT();
 }
 
 
@@ -640,8 +659,7 @@ Menus::locale_changed()
   if (syncing)
     return;
   syncing = true;
-  TRACE_ENTER("Menus::locale_changed");
-  
+
   for (int i = 0; i < MENU_SIZEOF; i++)
     {
       if (menus[i] != NULL)
@@ -651,7 +669,6 @@ Menus::locale_changed()
     }
 
   resync();
-  
+
   syncing = false;
-  TRACE_EXIT();
 }
