@@ -1,6 +1,6 @@
 // PreferencesDialog.cc --- Preferences dialog
 //
-// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -34,7 +34,7 @@
 #include <gtkmm/notebook.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/menu.h>
-#include <gtkmm/optionmenu.h>
+#include <gtkmm/combobox.h>
 #include <gtkmm/cellrenderer.h>
 #include <gtkmm/scale.h>
 #include <gtkmm/scrolledwindow.h>
@@ -45,7 +45,6 @@
 
 #include "GtkUtil.hh"
 #include "Hig.hh"
-#include "MainWindow.hh"
 #include "PreferencesDialog.hh"
 #include "TimeEntry.hh"
 #include "TimerBoxPreferencePage.hh"
@@ -83,7 +82,7 @@ PreferencesDialog::PreferencesDialog()
 
   connector = new DataConnector();
   inhibit_events = 0;
-  
+
   // Pages
   Gtk::Widget *timer_page = Gtk::manage(create_timer_page());
   Gtk::Notebook *gui_page = Gtk::manage(new Gtk::Notebook());
@@ -97,7 +96,7 @@ PreferencesDialog::PreferencesDialog()
   Gtk::Widget *gui_sounds_page = Gtk::manage(create_sounds_page());
   gui_page->append_page(*gui_sounds_page, _("Sounds"));
 #endif
-  
+
   Gtk::Widget *gui_mainwindow_page = Gtk::manage(create_mainwindow_page());
   gui_page->append_page(*gui_mainwindow_page, _("Status Window"));
 
@@ -105,7 +104,7 @@ PreferencesDialog::PreferencesDialog()
   Gtk::Widget *gui_applet_page = Gtk::manage(create_applet_page());
   gui_page->append_page(*gui_applet_page, _("Applet"));
 #endif
-  
+
 #ifdef HAVE_DISTRIBUTION
   Gtk::Widget *network_page = Gtk::manage(create_network_page());
 #endif
@@ -119,14 +118,11 @@ PreferencesDialog::PreferencesDialog()
 
   // Gtk::Widget *plugins_page = Gtk::manage( new PluginsPreferencePage() );
   // add_page( _("Plugins"), "workrave-icon-huge.png", *plugins_page );
-  
+
   // Dialog
   get_vbox()->pack_start(notebook, true, true, 0);
   add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
 
-  ICore *core = CoreFactory::get_core();
-  mode = core->get_operation_mode();
-
   show_all();
 
   TRACE_EXIT();
@@ -145,13 +141,14 @@ PreferencesDialog::~PreferencesDialog()
 
   GUIConfig::set_locale(code);
 #endif
-  
+
   ICore *core = CoreFactory::get_core();
-  core->set_operation_mode(mode, false);
+  core->remove_operation_mode_override( "preferences" );
 
   delete connector;
+#ifndef HAVE_GTK3
   delete filefilter;
-  
+#endif
   TRACE_EXIT();
 }
 
@@ -161,15 +158,16 @@ Gtk::Widget *
 PreferencesDialog::create_gui_page()
 {
   // Block types
-  block_button  = Gtk::manage(new Gtk::OptionMenu());
-  Gtk::Menu *block_menu = Gtk::manage(new Gtk::Menu());
-  Gtk::Menu::MenuList &block_list = block_menu->items();
-  block_button->set_menu(*block_menu);
-  block_list.push_back(Gtk::Menu_Helpers::MenuElem(_("No blocking")));
-  block_list.push_back(Gtk::Menu_Helpers::MenuElem
-                       (_("Block input")));
-  block_list.push_back(Gtk::Menu_Helpers::MenuElem
-                       (_("Block input and screen")));
+  block_button = Gtk::manage(new Gtk::ComboBoxText());
+#if GTKMM_CHECK_VERSION(2, 24, 0)
+  block_button->append(_("No blocking"));
+  block_button->append(_("Block input"));
+  block_button->append(_("Block input and screen"));
+#else
+  block_button->append_text(_("No blocking"));
+  block_button->append_text(_("Block input"));
+  block_button->append_text(_("Block input and screen"));
+#endif
 
   int block_idx;
   switch (GUIConfig::get_block_mode())
@@ -183,28 +181,28 @@ PreferencesDialog::create_gui_page()
     default:
       block_idx = 2;
     }
-  block_button->set_history(block_idx);
+  block_button->set_active(block_idx);
   block_button->signal_changed()
     .connect(sigc::mem_fun(*this, &PreferencesDialog::on_block_changed));
 
   // Options
   HigCategoryPanel *panel = Gtk::manage(new HigCategoryPanel(_("Options")));
 
-  panel->add(_("Block mode:"), *block_button);
-  
+  panel->add_label(_("Block mode:"), *block_button);
+
 #if defined(HAVE_LANGUAGE_SELECTION)
   string current_locale = GUIConfig::get_locale();
 
   languages_model = Gtk::ListStore::create(languages_columns);
   languages_combo.set_model(languages_model);
-  
+
   std::vector<std::string> all_linguas;
   StringUtil::split(string(ALL_LINGUAS), ' ', all_linguas);
   all_linguas.push_back("en");
-  
+
   Locale::LanguageMap languages_current_locale;
   Locale::LanguageMap languages_native_locale;
-  
+
   Locale::get_all_languages_in_current_locale(languages_current_locale);
   Locale::get_all_languages_in_native_locale(languages_native_locale);
 
@@ -215,7 +213,7 @@ PreferencesDialog::create_gui_page()
   row[languages_columns.code] = "";
   row[languages_columns.enabled] = true;
 
-  Gtk::TreeModel::iterator selected = iter;  
+  Gtk::TreeModel::iterator selected = iter;
 
   for (vector<std::string>::iterator i = all_linguas.begin(); i != all_linguas.end(); i++)
     {
@@ -230,9 +228,13 @@ PreferencesDialog::create_gui_page()
         {
           selected = iter;
         }
-      
+
       string txt = languages_current_locale[code].language_name;
-      if (languages_current_locale[code].country_name != "")
+      if( txt.empty() )
+      {
+          txt = "Unrecognized language: (" + code + ")";
+      }
+      else if (languages_current_locale[code].country_name != "")
         {
           txt += " (" + languages_current_locale[code].country_name + ")";
         }
@@ -253,7 +255,7 @@ PreferencesDialog::create_gui_page()
               txt = _("(font not available)");
               row[languages_columns.enabled] = false;
             }
-          
+
           row[languages_columns.native] = txt;
         }
     }
@@ -262,10 +264,10 @@ PreferencesDialog::create_gui_page()
  	languages_model->set_sort_func (languages_columns.current,
                                   sigc::mem_fun(*this,
                                                 &PreferencesDialog::on_cell_data_compare));
-    
-  languages_combo.pack_start(current_cellrenderer, true); 
+
+  languages_combo.pack_start(current_cellrenderer, true);
   languages_combo.pack_start(native_cellrenderer, false);
-  
+
   languages_combo.set_cell_data_func(native_cellrenderer,
                                      sigc::mem_fun(*this,
                                                    &PreferencesDialog::on_native_cell_data));
@@ -274,8 +276,8 @@ PreferencesDialog::create_gui_page()
                                                    &PreferencesDialog::on_current_cell_data));
 
   languages_combo.set_active(selected);
-  
-  panel->add(_("Language:"), languages_combo);
+
+  panel->add_label(_("Language:"), languages_combo);
 #endif
 
 #if defined(PLATFORM_OS_WIN32)
@@ -289,15 +291,15 @@ PreferencesDialog::create_gui_page()
   char value[MAX_PATH];
   bool rc = Util::registry_get_value(RUNKEY, "Workrave", value);
   autostart_cb->set_active(rc);
-#endif  
+#endif
 
   Gtk::Label *trayicon_lab = Gtk::manage(GtkUtil::create_label(_("Show system tray icon"), false));
   trayicon_cb = Gtk::manage(new Gtk::CheckButton());
   trayicon_cb->add(*trayicon_lab);
   connector->connect(GUIConfig::CFG_KEY_TRAYICON_ENABLED, dc::wrap(trayicon_cb));
 
-  panel->add(*trayicon_cb);
-
+  panel->add_widget(*trayicon_cb);
+  
   panel->set_border_width(12);
   return panel;
 }
@@ -311,17 +313,18 @@ PreferencesDialog::create_sounds_page()
   // Options
   HigCategoryPanel *hig = Gtk::manage(new HigCategoryPanel(_("Sound Options")));
   panel->pack_start(*hig, false, false, 0);
-  
+
   // Sound types
-  sound_button  = Gtk::manage(new Gtk::OptionMenu());
-  Gtk::Menu *sound_menu = Gtk::manage(new Gtk::Menu());
-  Gtk::Menu::MenuList &sound_list = sound_menu->items();
-  sound_button->set_menu(*sound_menu);
-  sound_list.push_back(Gtk::Menu_Helpers::MenuElem(_("No sounds")));
-  sound_list.push_back(Gtk::Menu_Helpers::MenuElem
-                       (_("Play sounds using sound card")));
-  sound_list.push_back(Gtk::Menu_Helpers::MenuElem
-                       (_("Play sounds using built-in speaker")));
+  sound_button  = Gtk::manage(new Gtk::ComboBoxText());
+#if GTKMM_CHECK_VERSION(2, 24, 0)
+  sound_button->append(_("No sounds"));
+  sound_button->append(_("Play sounds using sound card"));
+  sound_button->append(_("Play sounds using built-in speaker"));
+#else
+  sound_button->append_text(_("No sounds"));
+  sound_button->append_text(_("Play sounds using sound card"));
+  sound_button->append_text(_("Play sounds using built-in speaker"));
+#endif
   int idx;
   if (! SoundPlayer::is_enabled())
     idx = 0;
@@ -332,33 +335,33 @@ PreferencesDialog::create_sounds_page()
       else
         idx = 1;
     }
-  sound_button->set_history(idx);
+  sound_button->set_active(idx);
   sound_button->signal_changed().connect(sigc::mem_fun(*this, &PreferencesDialog::on_sound_changed));
 
-  GUI *gui = GUI::get_instance();
+  IGUI *gui = GUI::get_instance();
   SoundPlayer *snd = gui->get_sound_player();
-  
+
   if (snd->capability(SOUND_CAP_VOLUME))
     {
-      // Volume 
+      // Volume
       sound_volume_scale =  Gtk::manage(new Gtk:: HScale(0.0, 100.0, 0.0));
       sound_volume_scale->set_increments(1.0, 5.0);
       connector->connect(SoundPlayer::CFG_KEY_SOUND_VOLUME, dc::wrap(sound_volume_scale->get_adjustment()));
 
-      hig->add(_("Volume:"), *sound_volume_scale, true, true);
+      hig->add_label(_("Volume:"), *sound_volume_scale, true, true);
     }
 
-  hig->add(_("Sound:"), *sound_button);
+  hig->add_label(_("Sound:"), *sound_button);
 
   if (snd->capability(SOUND_CAP_MUTE))
     {
-      // Volume 
+      // Volume
       mute_cb = Gtk::manage(new Gtk::CheckButton
                             (_("Mute sounds during rest break and daily limit")));
-  
+
       connector->connect(SoundPlayer::CFG_KEY_SOUND_MUTE, dc::wrap(mute_cb));
 
-      hig->add(*mute_cb, true, true);
+      hig->add_widget(*mute_cb, true, true);
     }
 
   if (snd->capability(SOUND_CAP_EDIT))
@@ -366,16 +369,14 @@ PreferencesDialog::create_sounds_page()
       // Sound themes
       hig = Gtk::manage(new HigCategoryPanel(_("Sound Events"), true));
       panel->pack_start(*hig, true, true, 0);
-  
-      sound_theme_button  = Gtk::manage(new Gtk::OptionMenu());
-      Gtk::Menu *sound_theme_menu = Gtk::manage(new Gtk::Menu());
-      sound_theme_button->set_menu(*sound_theme_menu);
+
+      sound_theme_button  = Gtk::manage(new Gtk::ComboBoxText());
 
       update_theme_selection();
-  
+
       sound_theme_button->signal_changed().connect(sigc::mem_fun(*this, &PreferencesDialog::on_sound_theme_changed));
-      hig->add(_("Sound Theme:"), *sound_theme_button);
-  
+      hig->add_label(_("Sound Theme:"), *sound_theme_button);
+
       sound_store = Gtk::ListStore::create(sound_model);
       sound_treeview.set_model(sound_store);
 
@@ -396,44 +397,53 @@ PreferencesDialog::create_sounds_page()
 
       sound_treeview.set_rules_hint();
       sound_treeview.set_search_column(sound_model.description.index());
-  
+
       int cols_count = sound_treeview.append_column_editable(_("Play"), sound_model.enabled);
       Gtk::TreeViewColumn *column = sound_treeview.get_column(cols_count - 1);
 
       Gtk::CellRendererToggle *cell = dynamic_cast<Gtk::CellRendererToggle*>(sound_treeview.get_column_cell_renderer(cols_count - 1));
-    
+
       cols_count = sound_treeview.append_column(_("Event"), sound_model.description);
       column = sound_treeview.get_column(cols_count - 1);
       column->set_fixed_width(40);
-  
+
       Gtk::ScrolledWindow *sound_scroll = Gtk::manage(new Gtk::ScrolledWindow());
       sound_scroll->add(sound_treeview);
       sound_scroll->set_size_request(-1, 200);
       sound_treeview.set_size_request(-1, 200);
 
-      hig->add(*sound_scroll, true, true);
-                                                      
+      hig->add_widget(*sound_scroll, true, true);
+
       Gtk::HBox *hbox = Gtk::manage(new Gtk::HBox(false, 6));
-  
+
       sound_play_button = Gtk::manage(new Gtk::Button(_("Play")));
       hbox->pack_start(*sound_play_button, false, false, 0);
-  
+
       fsbutton = Gtk::manage(new Gtk::FileChooserButton(_("Choose a sound"),
                                                    Gtk::FILE_CHOOSER_ACTION_OPEN
                                                    ));
-  
+
       hbox->pack_start(*fsbutton, true, true, 0);
 
+#ifdef HAVE_GTK3
+      filefilter = Gtk::FileFilter::create();
+#else
       filefilter = new Gtk::FileFilter();
+#endif
+
       filefilter->set_name(_("Wavefiles"));
 #ifdef PLATFORM_OS_WIN32
       filefilter->add_pattern("*.wav");
 #else
       filefilter->add_mime_type("audio/x-wav");
 #endif
+#ifdef HAVE_GTK3
+      fsbutton->add_filter(filefilter);
+#else
       fsbutton->add_filter(*filefilter);
-  
-      hig->add(*hbox);
+#endif
+
+      hig->add_widget(*hbox);
 
       Gtk::HBox *selector_hbox = Gtk::manage(new Gtk::HBox(false, 0));
       Gtk::Button *selector_playbutton = Gtk::manage(new Gtk::Button(_("Play")));
@@ -447,10 +457,10 @@ PreferencesDialog::create_sounds_page()
 
       sound_play_button->signal_clicked().connect(sigc::mem_fun(*this,
                                                                 &PreferencesDialog::on_sound_play));
-  
+
       selector_playbutton->signal_clicked().connect(sigc::mem_fun(*this,
                                                                   &PreferencesDialog::on_sound_filechooser_play));
-  
+
       fsbutton->signal_selection_changed().connect(sigc::mem_fun(*this,
                                                                  &PreferencesDialog::on_sound_filechooser_select));
 
@@ -463,10 +473,10 @@ PreferencesDialog::create_sounds_page()
         {
           selection->select(iter);
         }
-      
+
       update_senstives();
     }
-    
+
   panel->set_border_width(12);
   return panel;
 }
@@ -489,11 +499,57 @@ PreferencesDialog::create_timer_page()
                                 ((BreakId) i));
       TimerPreferencesPanel *tp = Gtk::manage(new TimerPreferencesPanel(BreakId(i), hsize_group, vsize_group));
       box->show_all();
+#ifdef HAVE_GTK3
+      tnotebook->append_page(*tp, *box);
+#else
       tnotebook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*tp, *box));
+#endif
     }
+
+#if defined(PLATFORM_OS_WIN32)
+  Gtk::Widget *box = Gtk::manage(GtkUtil::create_label("Monitoring", false));
+  Gtk::Widget *monitoring_page = create_monitoring_page();
+  
+#ifdef HAVE_GTK3
+  tnotebook->append_page(*monitoring_page , *box);
+#else
+  tnotebook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*monitoring_page, *box));
+#endif
+#endif
+  
   return tnotebook;
 }
 
+#if defined(PLATFORM_OS_WIN32)
+Gtk::Widget *
+PreferencesDialog::create_monitoring_page()
+{
+  Gtk::VBox *panel = Gtk::manage(new Gtk::VBox(false, 6));
+  panel->set_border_width(12);
+
+  Gtk::Label *monitor_type_lab = Gtk::manage(GtkUtil::create_label(_("Use alternate monitor"), false));
+  monitor_type_cb = Gtk::manage(new Gtk::CheckButton());
+  monitor_type_cb->add(*monitor_type_lab);
+  monitor_type_cb->signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::on_monitor_type_toggled));
+  panel->pack_start(*monitor_type_cb, false, false, 0);
+  
+  Gtk::Label *monitor_type_help = Gtk::manage(GtkUtil::create_label(_("Enable this option if Workrave fails to detect when you are using your computer"), false));
+  panel->pack_start(*monitor_type_help, false, false, 0);
+
+  string monitor_type;
+  CoreFactory::get_configurator()->get_value_with_default("advanced/monitor",
+                                                          monitor_type,
+                                                          "default");
+
+
+  
+  monitor_type_cb->set_active(monitor_type != "default");
+
+  return panel;
+}
+
+#endif
+
 Gtk::Widget *
 PreferencesDialog::create_mainwindow_page()
 {
@@ -530,7 +586,7 @@ PreferencesDialog::add_page(const char *label, const char *image,
 void
 PreferencesDialog::on_sound_changed()
 {
-  int idx = sound_button->get_history();
+  int idx = sound_button->get_active_row_number();
   SoundPlayer::set_enabled(idx > 0);
   if (idx > 0)
     {
@@ -546,7 +602,7 @@ PreferencesDialog::on_sound_changed()
 void
 PreferencesDialog::update_senstives()
 {
-  int idx = sound_button->get_history();
+  int idx = sound_button->get_active_row_number();
   if (idx > 0)
     {
       SoundPlayer::Device dev = idx == 1
@@ -576,7 +632,7 @@ PreferencesDialog::update_senstives()
 void
 PreferencesDialog::on_block_changed()
 {
-  int idx = block_button->get_history();
+  int idx = block_button->get_active_row_number();
   GUIConfig::BlockMode m;
   switch (idx)
     {
@@ -611,10 +667,10 @@ PreferencesDialog::on_focus_in_event(GdkEventFocus *event)
     {
       ICore *core = CoreFactory::get_core();
 
-      mode = core->get_operation_mode();
+      OperationMode mode = core->get_operation_mode();
       if (mode == OPERATION_MODE_NORMAL)
         {
-          mode = core->set_operation_mode(OPERATION_MODE_QUIET, false);
+          core->set_operation_mode_override( OPERATION_MODE_QUIET, "preferences" );
         }
     }
   TRACE_EXIT();
@@ -628,7 +684,7 @@ PreferencesDialog::on_focus_out_event(GdkEventFocus *event)
   TRACE_ENTER("PreferencesDialog::focus_out");
   ICore *core = CoreFactory::get_core();
 
-  core->set_operation_mode(mode, false);
+  core->remove_operation_mode_override( "preferences" );
   TRACE_EXIT();
   return HigDialog::on_focus_out_event(event);
 }
@@ -702,11 +758,17 @@ PreferencesDialog::on_autostart_toggled()
 
       value = g_strdup_printf("%s" G_DIR_SEPARATOR_S "lib" G_DIR_SEPARATOR_S "workrave.exe", appdir.c_str());
     }
-  
+
   Util::registry_set_value(RUNKEY, "Workrave", value);
 }
-#endif
 
+void
+PreferencesDialog::on_monitor_type_toggled()
+{
+  bool on = monitor_type_cb->get_active();
+  CoreFactory::get_configurator()->set_value("advanced/monitor", on ? "lowlevel" : "default");
+}
+#endif
 
 void
 PreferencesDialog::on_sound_enabled(const Glib::ustring &path_string)
@@ -718,9 +780,9 @@ PreferencesDialog::on_sound_enabled(const Glib::ustring &path_string)
     {
       Gtk::TreeRow row = *iter;
 
-      GUI *gui = GUI::get_instance();
+      IGUI *gui = GUI::get_instance();
       SoundPlayer *snd = gui->get_sound_player();
-  
+
       snd->set_sound_enabled((SoundEvent)(int)row[sound_model.event],
                              row[sound_model.enabled]);
     }
@@ -742,9 +804,9 @@ PreferencesDialog::on_sound_play()
                                                               row[sound_model.label],
                                                               filename);
 
-      if (valid)
+      if (valid && filename != "")
         {
-          GUI *gui = GUI::get_instance();
+          IGUI *gui = GUI::get_instance();
           SoundPlayer *snd = gui->get_sound_player();
           snd->play_sound(filename);
         }
@@ -756,7 +818,7 @@ PreferencesDialog::on_sound_filechooser_play()
 {
   string filename = fsbutton->get_filename();
 
-  GUI *gui = GUI::get_instance();
+  IGUI *gui = GUI::get_instance();
   SoundPlayer *snd = gui->get_sound_player();
   snd->play_sound(filename);
 }
@@ -769,11 +831,11 @@ PreferencesDialog::on_sound_filechooser_select()
   string filename = fsbutton->get_filename();
 
   TRACE_MSG(filename << " " << fsbutton_filename << " " << inhibit_events);
-  
+
   if (inhibit_events == 0 && filename != "" && fsbutton_filename != filename)
     {
       TRACE_MSG("ok");
-  
+
       Glib::RefPtr<Gtk::TreeSelection> selection = sound_treeview.get_selection();
       Gtk::TreeModel::iterator iter = selection->get_selected();
 
@@ -782,7 +844,7 @@ PreferencesDialog::on_sound_filechooser_select()
 
         TRACE_MSG(row[sound_model.label]);
 
-        GUI *gui = GUI::get_instance();
+        IGUI *gui = GUI::get_instance();
         SoundPlayer *snd = gui->get_sound_player();
         snd->set_sound_wav_file( (SoundEvent)(int)row[sound_model.event], filename);
         TRACE_MSG(filename);
@@ -811,10 +873,10 @@ PreferencesDialog::on_sound_events_changed()
       bool valid = CoreFactory::get_configurator()->get_value(string(SoundPlayer::CFG_KEY_SOUND_EVENTS) +
                                                               row[sound_model.label],
                                                               filename);
-    
+
       TRACE_MSG(filename);
 
-      if (valid)
+      if (valid && filename != "")
         {
           inhibit_events++;
           fsbutton_filename = filename;
@@ -830,11 +892,11 @@ void
 PreferencesDialog::on_sound_theme_changed()
 {
   TRACE_ENTER("PreferencesDialog::on_sound_theme_changed");
-  int idx = sound_theme_button->get_history();
+  int idx = sound_theme_button->get_active_row_number();
 
   SoundPlayer::Theme &theme = sound_themes[idx];
 
-  GUI *gui = GUI::get_instance();
+  IGUI *gui = GUI::get_instance();
   SoundPlayer *snd = gui->get_sound_player();
   snd->activate_theme(theme);
 
@@ -846,12 +908,12 @@ PreferencesDialog::on_sound_theme_changed()
       Gtk::TreeModel::Row row = *iter;
       string event = (Glib::ustring) row[sound_model.label];
       string filename;
-      
+
       bool valid = CoreFactory::get_configurator()->get_value(string(SoundPlayer::CFG_KEY_SOUND_EVENTS) +
                                                               row[sound_model.label],
                                                               filename);
-      
-      if (valid)
+
+      if (valid && filename != "")
         {
           TRACE_MSG(filename << " " <<row[sound_model.label]);
           inhibit_events++;
@@ -869,24 +931,29 @@ PreferencesDialog::update_theme_selection()
 {
   TRACE_ENTER("PreferencesDialog::update_theme_selection");
   sound_themes.erase(sound_themes.begin(), sound_themes.end());
-  
-  GUI *gui = GUI::get_instance();
+
+  IGUI *gui = GUI::get_instance();
   SoundPlayer *snd = gui->get_sound_player();
   snd->get_sound_themes(sound_themes);
 
-  Gtk::Menu *sound_theme_menu = sound_theme_button->get_menu();
-  Gtk::Menu::MenuList &sound_theme_list = sound_theme_menu->items();
-
-  sound_theme_list.erase(sound_theme_list.begin(), sound_theme_list.end());
+#if GTKMM_CHECK_VERSION(2, 24, 0)
+  sound_theme_button->remove_all();
+#else
+  sound_theme_button->clear_items();
+#endif
 
   int idx = 0;
   for (vector<SoundPlayer::Theme>::iterator it = sound_themes.begin(); it != sound_themes.end(); it++)
     {
-      sound_theme_list.push_back(Gtk::Menu_Helpers::MenuElem(it->description));
-      
+#if GTKMM_CHECK_VERSION(2, 24, 0)
+      sound_theme_button->append(it->description);
+#else
+      sound_theme_button->append_text(it->description);
+#endif
+
       if (it->active)
        {
-          sound_theme_button->set_history(idx);
+          sound_theme_button->set_active(idx);
         }
       idx++;
     }
