@@ -1,6 +1,6 @@
 // PreludeWindow.cc
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2011, 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -23,9 +23,10 @@
 
 #include "preinclude.h"
 
-#include <gtkmm/label.h>
-#include <gtkmm/image.h>
-#include <gtkmm/box.h>
+#include <gtkmm.h>
+#ifdef HAVE_GTK3      
+#include <gdkmm/devicemanager.h>
+#endif
 
 #include "debug.hh"
 #include "nls.h"
@@ -54,7 +55,6 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
   : Gtk::Window(Gtk::WINDOW_POPUP),
     SCREEN_MARGIN(20),
     did_avoid(false),
-    break_id(break_id),
     time_bar(NULL),
     frame(NULL),
     window_frame(NULL),
@@ -62,8 +62,7 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
     image_icon(NULL),
     progress_value(0),
     progress_max_value(0),
-    flash_visible(false),
-    prelude_response(NULL)
+    flash_visible(false)
 {
   TRACE_ENTER("PreludeWindow::PreludeWindow");
   Gtk::Window::set_border_width(0);
@@ -120,7 +119,11 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
       break;
     }
 
+#ifdef HAVE_GTK3
+  set_can_focus(false);
+#else
   unset_flags(Gtk::CAN_FOCUS);
+#endif
 
   show_all_children();
   stick();
@@ -128,30 +131,30 @@ PreludeWindow::PreludeWindow(HeadInfo &head, BreakId break_id)
   // trace window handles:
   // FIXME: debug, remove later
 #ifdef PLATFORM_OS_WIN32
-  HWND _hwnd = (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
-  HWND _scope = (HWND) GDK_WINDOW_HWND( GTK_WIDGET( this->gobj() )->window );
+  HWND _hwnd = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(Gtk::Widget::gobj()));
+  HWND _scope = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET( this->gobj())));
   HWND _hRoot = GetAncestor( _hwnd, GA_ROOT );
   HWND _hParent = GetAncestor( _hwnd, GA_PARENT );
   HWND _hDesktop = GetDesktopWindow();
-  
+
   TRACE_MSG("PreludeWindow created" <<  hex << _hwnd  << dec);
-  
+
   if (_hwnd != _scope)
     {
       TRACE_MSG( "!!!!!!!!!!!!!!!" <<  "Scope issue: " << hex << _scope  << dec);
     }
-  
+
   if (_hwnd != _hRoot)
     {
       TRACE_MSG( "GetDesktopWindow()" <<  hex << _hDesktop  << dec);
       TRACE_MSG( "!!!!!!!!!!!!!!!" <<  "PreludeWindow GA_ROOT: " << hex << _hRoot  << dec);
     }
-  
+
   if( _hParent != _hDesktop )
     {
       TRACE_MSG( "GetDesktopWindow()" <<  hex << _hDesktop  << dec);
       TRACE_MSG( "!!!!!!!!!!!!!!!" << "PreludeWindow GA_PARENT: " << hex << _hParent  << dec);
-      
+
       HWND _hTemp;
       while( IsWindow( _hParent ) && _hParent != _hDesktop )
         {
@@ -210,9 +213,9 @@ PreludeWindow::start()
   show_all();
 
   WindowHints::set_always_on_top(this, true);
-  
+
   time_bar->set_bar_color(TimeBar::COLOR_ID_OVERDUE);
-  
+
   TRACE_EXIT();
 }
 
@@ -252,7 +255,11 @@ PreludeWindow::stop()
   TRACE_ENTER("PreludeWindow::stop");
 
   frame->set_frame_flashing(0);
+#ifdef HAVE_GTK3
+  hide();
+#else
   hide_all();
+#endif
 
   TRACE_EXIT();
 }
@@ -276,10 +283,10 @@ PreludeWindow::refresh()
     }
   time_bar->set_text(s);
   time_bar->update();
-  
+
 #if defined(PLATFORM_OS_WIN32)
 // Vista GTK phantom toplevel parent kludge:
-  HWND hwnd = (HWND) GDK_WINDOW_HWND( Gtk::Widget::gobj()->window );
+  HWND hwnd = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(Gtk::Widget::gobj()));
   if( hwnd )
     {
       HWND hAncestor = GetAncestor( hwnd, GA_ROOT );
@@ -287,7 +294,7 @@ PreludeWindow::refresh()
       if( hAncestor && hDesktop && hAncestor != hDesktop )
           hwnd = hAncestor;
       // Set toplevel window topmost!
-      SetWindowPos( hwnd, HWND_TOPMOST, 0, 0, 0, 0, 
+      SetWindowPos( hwnd, HWND_TOPMOST, 0, 0, 0, 0,
           SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
     }
 #endif
@@ -386,26 +393,40 @@ PreludeWindow::init_avoid_pointer()
     {
       POINT p;
       GetCursorPos(&p);
-      
+
+#ifdef HAVE_GTK3      
+      Glib::RefPtr<Gdk::Display> display = Gdk::Display::get_default();
+      Glib::RefPtr<Gdk::DeviceManager> device_manager = display->get_device_manager();
+      Glib::RefPtr<Gdk::Device> device = device_manager->get_client_pointer();
+      int x, y;
+      device->get_position(x, y);
+#else
       Glib::RefPtr<Gdk::Display> display = Gdk::Display::get_default();
       int x, y;
       Gdk::ModifierType mod;
       display->get_pointer(x, y, mod);
-
+#endif
+      
       TRACE_MSG("p " << p.x << " " << p.y);
       TRACE_MSG("d " << x << " " << y);
 
       gdk_offset_x = p.x - x;
       gdk_offset_y = p.y - y;
 
-      TRACE_MSG("offset " << gdk_offset_x << " " << gdk_offset_y); 
-     
+      TRACE_MSG("offset " << gdk_offset_x << " " << gdk_offset_y);
+
       avoid_signal = Glib::signal_timeout()
         .connect(sigc::mem_fun(*this, &PreludeWindow::on_avoid_pointer_timer),
                  150);
     }
 #else
-  if (! is_realized())
+  if (
+#ifdef HAVE_GTK3
+      ! get_realized()
+#else
+      ! is_realized()
+#endif
+     )
     {
       Gdk::EventMask events;
 
@@ -437,8 +458,13 @@ PreludeWindow::avoid_pointer(int px, int py)
   TRACE_ENTER_MSG("PreludeWindow::avoid_pointer", px << " " << py);
   Glib::RefPtr<Gdk::Window> window = get_window();
 
+#ifdef HAVE_GTK3
+  int winx, winy, width, height;
+  window->get_geometry(winx, winy, width, height);
+#else
   int winx, winy, width, height, wind;
   window->get_geometry(winx, winy, width, height, wind);
+#endif
 
   TRACE_MSG("geom" << winx << " " << winy << " " << width << " " << height << " ");
 
@@ -461,7 +487,7 @@ PreludeWindow::avoid_pointer(int px, int py)
   int top_y = head.get_y() + SCREEN_MARGIN;
   int bottom_y = head.get_y() + screen_height - height - SCREEN_MARGIN;
   TRACE_MSG("geom3" << screen_height << " " << top_y << " " << bottom_y);
-  
+
   if (winy < top_y + SCREEN_MARGIN)
     {
       winy = bottom_y;
