@@ -1,6 +1,6 @@
 // StatusIcon.cc --- Status icon
 //
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #include "config.h"
 #endif
 
+#include "debug.hh"
 #include <string>
 #include <assert.h>
 
@@ -34,15 +35,17 @@
 #include "W32StatusIcon.hh"
 #endif
 
-#include "MainWindow.hh"
-#include "CoreFactory.hh"
-#include "Menus.hh"
-#include "Util.hh"
 #include "StatusIcon.hh"
 
+#include "GUI.hh"
+#include "CoreFactory.hh"
+#include "IConfigurator.hh"
+#include "GUIConfig.hh"
+#include "Menus.hh"
+#include "Util.hh"
+#include "TimerBoxControl.hh"
 
-StatusIcon::StatusIcon(MainWindow& mw)
-  : main_window(mw)
+StatusIcon::StatusIcon()
 {
   // Preload icons
   const char *mode_files[] =
@@ -65,39 +68,55 @@ StatusIcon::StatusIcon(MainWindow& mw)
         }
     }
 
-#ifndef USE_W32STATUSICON
-#ifdef PLATFORM_OS_WIN32
+#if !defined(USE_W32STATUSICON) && defined(PLATFORM_OS_WIN32)
   wm_taskbarcreated = RegisterWindowMessage("TaskbarCreated");
 #endif
-#endif
-
-  insert_icon();
 }
 
 StatusIcon::~StatusIcon()
 {
 }
 
+void
+StatusIcon::init()
+{
+  insert_icon();
+
+  CoreFactory::get_configurator()->add_listener(GUIConfig::CFG_KEY_TRAYICON_ENABLED, this);
+  
+  bool tray_icon_enabled = GUIConfig::get_trayicon_enabled();
+  if (!tray_icon_enabled)
+    {
+      // Recover from bug in 1.9.3 where tray icon AND mainwindow could be disabled
+      TimerBoxControl::set_enabled("main_window", true);
+    }
+
+  status_icon->set_visible(tray_icon_enabled);
+}
+
 void
 StatusIcon::insert_icon()
 {
   // Create status icon
   ICore *core = CoreFactory::get_core();
-  OperationMode mode = core->get_operation_mode();
+  OperationMode mode = core->get_operation_mode_regular();
 
 #ifdef USE_W32STATUSICON
   status_icon = new W32StatusIcon();
   set_operation_mode(mode);
 #else
   status_icon = Gtk::StatusIcon::create(mode_icons[mode]);
+  status_icon->property_embedded().signal_changed().connect(sigc::mem_fun(*this, &StatusIcon::on_embedded_changed));
 #endif
 
-  status_icon->signal_size_changed().connect(sigc::mem_fun(*this, &StatusIcon::on_size_changed));
-  
-#ifdef HAVE_STATUSICON_SIGNAL 
+#ifdef HAVE_STATUSICON_SIGNAL
+#ifdef USE_W32STATUSICON
+  status_icon->signal_balloon_activate().connect(sigc::mem_fun(*this, &StatusIcon::on_balloon_activate));
+#endif
   status_icon->signal_activate().connect(sigc::mem_fun(*this, &StatusIcon::on_activate));
   status_icon->signal_popup_menu().connect(sigc::mem_fun(*this, &StatusIcon::on_popup_menu));
 #else
+
   // Hook up signals, missing from gtkmm
   GtkStatusIcon *gobj = status_icon->gobj();
 
@@ -108,57 +127,66 @@ StatusIcon::insert_icon()
 #endif
 }
 
-void StatusIcon::set_operation_mode(OperationMode m)
+void
+StatusIcon::set_operation_mode(OperationMode m)
 {
   status_icon->set(mode_icons[m]);
 }
 
-void StatusIcon::set_visible(bool b)
-{
-  status_icon->set_visible(b);
-}
-
-bool StatusIcon::is_embedded() const
+bool
+StatusIcon::is_visible() const
 {
   return status_icon->is_embedded() && status_icon->get_visible();
 }
 
-void StatusIcon::on_activate()
+void
+StatusIcon::set_tooltip(std::string& tip)
 {
-  main_window.on_activate();
-}
-
-void StatusIcon::show_balloon(const string &balloon)
-{
-#ifdef USE_W32STATUSICON
-  status_icon->show_balloon(balloon);
+#if defined(HAVE_GTK3) && !defined(USE_W32STATUSICON)
+  status_icon->set_tooltip_text(tip);
+#else
+  status_icon->set_tooltip(tip);
 #endif
 }
 
+void
+StatusIcon::show_balloon(string id, const string &balloon)
+{
+#ifdef USE_W32STATUSICON
+  status_icon->show_balloon(id, balloon);
+#else
+  (void) id;
+  (void) balloon;
+#endif
+}
 
-void StatusIcon::on_popup_menu(guint button, guint activate_time)
+void
+StatusIcon::on_popup_menu(guint button, guint activate_time)
 {
   (void) button;
 
   // Note the 1 is a hack. It used to be 'button'. See bugzilla 598
-  Menus::get_instance()->popup(Menus::MENU_APPLET, 1, activate_time);
+  IGUI *gui = GUI::get_instance();
+  Menus *menus = gui->get_menus();
+  menus->popup(Menus::MENU_MAINAPPLET, 1, activate_time);
 }
 
-bool StatusIcon::on_size_changed(guint size)
+void
+StatusIcon::on_embedded_changed()
 {
-  (void) size;
-  main_window.status_icon_changed();
-  return true;
+  visibility_changed_signal.emit();
 }
 
-#ifndef HAVE_STATUSICON_SIGNAL 
-void StatusIcon::activate_callback(GtkStatusIcon *,
+#ifndef HAVE_STATUSICON_SIGNAL
+void
+StatusIcon::activate_callback(GtkStatusIcon *,
                                    gpointer callback_data)
 {
   static_cast<StatusIcon*>(callback_data)->on_activate();
 }
 
-void StatusIcon::popup_menu_callback(GtkStatusIcon *,
+void
+StatusIcon::popup_menu_callback(GtkStatusIcon *,
                                      guint button,
                                      guint activate_time,
                                      gpointer callback_data)
@@ -167,13 +195,21 @@ void StatusIcon::popup_menu_callback(GtkStatusIcon *,
 }
 #endif
 
-void StatusIcon::set_timers_tooltip(std::string& tip)
+#if defined(PLATFORM_OS_WIN32) && defined(USE_W32STATUSICON)
+void
+StatusIcon::on_balloon_activate(string id)
 {
-  status_icon->set_tooltip(tip);
+  balloon_activate_signal.emit(id);
+}
+#endif
+
+void
+StatusIcon::on_activate()
+{
+  activate_signal.emit();
 }
 
-#ifndef USE_W32STATUSICON
-#ifdef PLATFORM_OS_WIN32
+#if !defined(USE_W32STATUSICON) && defined(PLATFORM_OS_WIN32)
 GdkFilterReturn
 StatusIcon::win32_filter_func (void     *xevent,
                                GdkEvent *event)
@@ -189,5 +225,39 @@ StatusIcon::win32_filter_func (void     *xevent,
   return ret;
 }
 #endif
-#endif
 
+void
+StatusIcon::config_changed_notify(const std::string &key)
+{
+  TRACE_ENTER_MSG("StatusIcon::config_changed_notify", key);
+
+  if (key == GUIConfig::CFG_KEY_TRAYICON_ENABLED)
+    {
+      bool visible = GUIConfig::get_trayicon_enabled();
+      if (status_icon->get_visible() != visible)
+        {
+          visibility_changed_signal.emit();
+          status_icon->set_visible(visible);
+        }
+    }
+
+  TRACE_EXIT();
+}
+
+sigc::signal<void> &
+StatusIcon::signal_visibility_changed()
+{
+  return visibility_changed_signal;
+}
+
+sigc::signal<void> &
+StatusIcon::signal_activate()
+{
+  return activate_signal;
+}
+
+sigc::signal<void, string> &
+StatusIcon::signal_balloon_activate()
+{
+  return balloon_activate_signal;
+}
