@@ -1,6 +1,6 @@
 // X11SystrayAppletWindow.cc --- Applet info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011, 2012 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -26,15 +26,12 @@
 #include "nls.h"
 #include "debug.hh"
 
-#include <gtkmm/alignment.h>
-
 #include "X11SystrayAppletWindow.hh"
 
+#include "GUI.hh"
 #include "TimerBoxGtkView.hh"
 #include "TimerBoxControl.hh"
-#include "AppletControl.hh"
 #include "Menus.hh"
-#include "System.hh"
 
 #include "gtktrayicon.h"
 
@@ -43,7 +40,7 @@
  *  \param gui the main GUI entry point.
  *  \param control Interface to the controller.
  */
-X11SystrayAppletWindow::X11SystrayAppletWindow(AppletControl *control) :
+X11SystrayAppletWindow::X11SystrayAppletWindow() :
   view(NULL),
   plug(NULL),
   container(NULL),
@@ -51,7 +48,6 @@ X11SystrayAppletWindow::X11SystrayAppletWindow(AppletControl *control) :
   applet_size(0),
   applet_active(false),
   embedded(false),
-  control(control),
   tray_icon(NULL)
 {
 }
@@ -90,17 +86,19 @@ X11SystrayAppletWindow::notify_callback()
       if (o != GTK_ORIENTATION_VERTICAL)
         {
           orientation = ORIENTATION_UP;
+          TRACE_MSG("up");
         }
       else
         {
           orientation = ORIENTATION_LEFT;
+          TRACE_MSG("left");
         }
 
       if (applet_orientation != orientation)
         {
+          TRACE_MSG("orientation " << orientation);
           applet_orientation = orientation;
           view->set_geometry(applet_orientation, applet_size);
-          TRACE_MSG("orientation " << applet_orientation);
         }
     }
   TRACE_EXIT();
@@ -136,18 +134,13 @@ X11SystrayAppletWindow::activate_applet()
                                                                   &X11SystrayAppletWindow::on_button_press_event));
       container = eventbox;
 
-      view = new TimerBoxGtkView(Menus::MENU_APPLET);
+      view = new TimerBoxGtkView(Menus::MENU_MAINAPPLET);
       timer_box_view = view;
       timer_box_control = new TimerBoxControl("applet", *timer_box_view);
 
       Gtk::VBox *box = manage(new Gtk::VBox());
       box->set_spacing(1);
       box->pack_start(*view, true, true, 0);
-      
-      if (System::is_kde())
-        {
-          timer_box_control->set_force_empty(true);
-        }
 
       container->add(*box);
 
@@ -160,9 +153,16 @@ X11SystrayAppletWindow::activate_applet()
 
       applet_orientation = ORIENTATION_UP;
 
+#ifdef HAVE_GTK3
+      GtkRequisition min_size;
+      GtkRequisition natural_size;
+      plug->get_preferred_size(min_size, natural_size);
+      applet_size = min_size.height;
+#else
       Gtk::Requisition req;
       plug->size_request(req);
       applet_size = req.height;
+#endif
 
       view->set_geometry(applet_orientation, applet_size);
 
@@ -202,8 +202,7 @@ X11SystrayAppletWindow::deactivate_applet()
       timer_box_view = NULL;
       view = NULL;
 
-      control->set_applet_state(AppletControl::APPLET_TRAY,
-                                AppletWindow::APPLET_STATE_DISABLED);
+      state_changed_signal.emit(AppletWindow::APPLET_STATE_DISABLED);
     }
 
   applet_active = false;
@@ -218,8 +217,7 @@ X11SystrayAppletWindow::on_delete_event(GdkEventAny *event)
 {
   (void) event;
   deactivate_applet();
-  control->set_applet_state(AppletControl::APPLET_TRAY,
-                            AppletWindow::APPLET_STATE_DISABLED);
+  state_changed_signal.emit(AppletWindow::APPLET_STATE_DISABLED);
   return true;
 }
 
@@ -235,7 +233,7 @@ X11SystrayAppletWindow::on_embedded()
       GtkOrientation o = wrgtk_tray_icon_get_orientation(tray_icon);
       Orientation orientation;
 
-      if (o == GTK_ORIENTATION_VERTICAL)
+      if (o != GTK_ORIENTATION_VERTICAL)
         {
           orientation = ORIENTATION_UP;
         }
@@ -246,13 +244,17 @@ X11SystrayAppletWindow::on_embedded()
 
       embedded = true;
       applet_size = 24;
-      
+      applet_orientation = orientation;
+
       view->set_geometry(applet_orientation, applet_size);
+
+#ifdef HAVE_GTK3
+      gint icon_size = wrgtk_tray_icon_get_icon_size(tray_icon);
+      TRACE_MSG("icon size " << icon_size);
+#endif
     }
 
-  control->set_applet_state(AppletControl::APPLET_TRAY,
-                            AppletWindow::APPLET_STATE_VISIBLE);
-
+  state_changed_signal.emit(AppletWindow::APPLET_STATE_VISIBLE);
   TRACE_EXIT();
 }
 
@@ -268,8 +270,9 @@ X11SystrayAppletWindow::on_button_press_event(GdkEventButton *event)
     {
       if (event->button == 3)
         {
-          Menus::get_instance()->popup(Menus::MENU_APPLET,
-                                       0 /*event->button */, event->time);
+          IGUI *gui = GUI::get_instance();
+          Menus *menus = gui->get_menus();
+          menus->popup(Menus::MENU_MAINAPPLET, 0 /*event->button */, event->time);
           ret = true;
         }
       if (event->button == 1)
@@ -302,9 +305,13 @@ X11SystrayAppletWindow::on_size_allocate(Gtk::Allocation& allocation)
                 allocation.get_y() << " " <<
                 allocation.get_width() << " " <<
                 allocation.get_height());
+#ifdef HAVE_GTK3
+      gint icon_size = wrgtk_tray_icon_get_icon_size(tray_icon);
+      TRACE_MSG("icon size " << icon_size);
+#endif
       GtkOrientation o = wrgtk_tray_icon_get_orientation(tray_icon);
       Orientation orientation;
-  
+
       if (o == GTK_ORIENTATION_VERTICAL)
         {
           orientation = ORIENTATION_UP;
@@ -313,14 +320,14 @@ X11SystrayAppletWindow::on_size_allocate(Gtk::Allocation& allocation)
         {
           orientation = ORIENTATION_LEFT;
         }
-  
+
       if (orientation == ORIENTATION_UP ||
           orientation == ORIENTATION_DOWN)
         {
           if (applet_size != allocation.get_width())
             {
               applet_size = allocation.get_width();
-              TRACE_MSG("New size = " << allocation.get_width() << " " << allocation.get_height() << " " << applet_orientation);
+              TRACE_MSG("New size = " << applet_size);
               view->set_geometry(applet_orientation, applet_size);
             }
         }
@@ -329,10 +336,24 @@ X11SystrayAppletWindow::on_size_allocate(Gtk::Allocation& allocation)
           if (applet_size != allocation.get_height())
             {
               applet_size = allocation.get_height();
-              TRACE_MSG("New size = " << allocation.get_width() << " " << allocation.get_height() << " " << applet_orientation);
+              TRACE_MSG("New size = " << applet_size);
               view->set_geometry(applet_orientation, applet_size);
             }
         }
+
+#ifdef HAVE_GTK3
+      Gtk::Requisition my_size;
+      GtkRequisition natural_size;
+      view->get_preferred_size(my_size, natural_size);
+      TRACE_MSG("my_size = " << my_size.width << " " << my_size.height);
+      TRACE_MSG("natural_size = " << natural_size.width << " " << natural_size.height);
+
+      // hack... 
+      if (!view->is_sheep_only())
+        {
+          view->set_sheep_only(allocation.get_width() < my_size.width || allocation.get_height() < my_size.height);
+        }
+#endif
     }
   TRACE_EXIT();
 }
