@@ -1,5 +1,5 @@
 /* gtktrayicon.h
- * Copyright (C) 2002 Anders Carlsson <andersca@gnu.org>
+ * Copyright (C) 2002, 2012 Anders Carlsson <andersca@gnu.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@ G_BEGIN_DECLS
 #define GTK_IS_TRAY_ICON(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_TRAY_ICON))
 #define GTK_IS_TRAY_ICON_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TRAY_ICON))
 #define GTK_TRAY_ICON_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_TRAY_ICON, WRGtkTrayIconClass))
-	
+
 typedef struct _WRGtkTrayIcon	   WRGtkTrayIcon;
 typedef struct _WRGtkTrayIconPrivate WRGtkTrayIconPrivate;
 typedef struct _WRGtkTrayIconClass   WRGtkTrayIconClass;
@@ -69,7 +69,11 @@ void           _wrgtk_tray_icon_cancel_message  (WRGtkTrayIcon *icon,
 					       guint        id);
 
 GtkOrientation wrgtk_tray_icon_get_orientation (WRGtkTrayIcon *icon);
-					    
+
+#ifdef HAVE_GTK3
+gint           wrgtk_tray_icon_get_icon_size (WRGtkTrayIcon *icon);
+#endif
+
 G_END_DECLS
 
 #endif /* __GTK_TRAY_ICON_H__ */
