@@ -1,6 +1,6 @@
 // W32AppletMenu.cc --- Menus using W32Applet+
 //
-// Copyright (C) 2001 - 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001 - 2013 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 
 #include <string>
 
+#include "GUI.hh"
 #include "Menus.hh"
 #include "Util.hh"
 #include "MainWindow.hh"
@@ -40,9 +41,8 @@ using namespace workrave;
 
 
 //! Constructor.
-W32AppletMenu::W32AppletMenu(MainWindow *main_window, W32AppletWindow *applet_window)
-  : main_window(main_window),
-    applet_window(applet_window)
+W32AppletMenu::W32AppletMenu(W32AppletWindow *applet_window)
+  : applet_window(applet_window)
 {
 }
 
@@ -52,37 +52,19 @@ W32AppletMenu::~W32AppletMenu()
 {
 }
 
-void
-W32AppletMenu::popup(const guint button, const guint activate_time)
-{
-  (void) button;
-  (void) activate_time;
-}
-
-
-void
-W32AppletMenu::init()
-{
-}
-
-
-void
-W32AppletMenu::add_accel(Gtk::Window &window)
-{
-  (void) window;
-}
-
 void
 W32AppletMenu::resync(OperationMode mode, UsageMode usage, bool show_log)
 {
   TRACE_ENTER_MSG("W32AppletMenu::resync", mode << " " << show_log);
+
+  IGUI *gui = GUI::get_instance();
+  MainWindow *main_window = gui->get_main_window();
   
-  if (applet_window != NULL && main_window != NULL)
+  if (applet_window != NULL)
     {
       TRACE_MSG("ok");
 #ifndef PLATFORM_OS_WIN32_NATIVE
-      HWND cmd_win = (HWND) GDK_WINDOW_HWND(main_window
-                                             ->Gtk::Widget::gobj()->window);
+      HWND cmd_win = (HWND) GDK_WINDOW_HWND(gtk_widget_get_window(main_window->Gtk::Widget::gobj()));
 #else
       HWND cmd_win = (HWND) GDK_WINDOW_HWND(((GtkWidget*)main_window->gobj())->window);
 #endif
@@ -101,18 +83,18 @@ W32AppletMenu::resync(OperationMode mode, UsageMode usage, bool show_log)
                       |(mode == OPERATION_MODE_NORMAL
                         ? W32AppletWindow::MENU_FLAG_SELECTED
                         : 0));
-      w32aw->add_menu(_("_Suspended"), Menus::MENU_COMMAND_MODE_SUSPENDED,
-                      W32AppletWindow::MENU_FLAG_TOGGLE
-                      |W32AppletWindow::MENU_FLAG_POPUP
-                      |(mode == OPERATION_MODE_SUSPENDED
-                        ? W32AppletWindow::MENU_FLAG_SELECTED
-                        : 0));
       w32aw->add_menu(_("Q_uiet"), Menus::MENU_COMMAND_MODE_QUIET,
                       W32AppletWindow::MENU_FLAG_TOGGLE
                       |W32AppletWindow::MENU_FLAG_POPUP
                       |(mode == OPERATION_MODE_QUIET
                         ? W32AppletWindow::MENU_FLAG_SELECTED
                         : 0));
+      w32aw->add_menu(_("_Suspended"), Menus::MENU_COMMAND_MODE_SUSPENDED,
+                      W32AppletWindow::MENU_FLAG_TOGGLE
+                      |W32AppletWindow::MENU_FLAG_POPUP
+                      |(mode == OPERATION_MODE_SUSPENDED
+                        ? W32AppletWindow::MENU_FLAG_SELECTED
+                        : 0));
 
       w32aw->add_menu(_("_Mode"), 0, 0);
 
@@ -140,7 +122,7 @@ W32AppletMenu::resync(OperationMode mode, UsageMode usage, bool show_log)
                       |(usage == USAGE_MODE_READING
                         ? W32AppletWindow::MENU_FLAG_SELECTED
                         : 0));
-      
+
       w32aw->add_menu(_("Statistics"), Menus::MENU_COMMAND_STATISTICS, 0);
       w32aw->add_menu(_("About..."), Menus::MENU_COMMAND_ABOUT, 0);
     }
