@@ -1,6 +1,6 @@
 // AppletWindow.cc --- Applet info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -27,6 +27,11 @@
 
 #include "W32AppletWindow.hh"
 #include "TimerBoxControl.hh"
+
+#if defined(interface)
+#undef interface
+#endif
+
 #include "Applet.hh"
 #include "GUI.hh"
 #include "Menus.hh"
@@ -39,7 +44,7 @@ W32AppletWindow::W32AppletWindow()
   memset(&local_menu_data, 0, sizeof(AppletMenuData));
   memset(&heartbeat_data, 0, sizeof(AppletHeartbeatData));
   memset(&menu_data, 0, sizeof(AppletMenuData));
-  
+
   thread_id = 0;
   thread_handle = NULL;
   timer_box_view = this;
@@ -49,6 +54,7 @@ W32AppletWindow::W32AppletWindow()
   init_menu(NULL);
 
   ::InitializeCriticalSection(&heartbeat_data_lock);
+  thread_abort_event = ::CreateEvent(NULL, FALSE, FALSE, NULL);
   heartbeat_data_event = ::CreateEvent(NULL, FALSE, FALSE, NULL);
 
   // Intentionally last line, as this one calls W32AW::set_enabled(), e.g.
@@ -59,13 +65,31 @@ W32AppletWindow::W32AppletWindow()
 
 W32AppletWindow::~W32AppletWindow()
 {
-  TRACE_ENTER("W32AppletWindow::~W32AppletWindow");
-  delete timer_box_control;
+	TRACE_ENTER("W32AppletWindow::~W32AppletWindow");
 
-  ::DeleteCriticalSection(&heartbeat_data_lock);
-  CloseHandle(heartbeat_data_event);
+	/* before this instance is destroyed we signal and wait for its worker thread to terminate. this 
+	isn't ideal because the gui will be blocked while we wait for termination if this destructor is 
+	called from the main thread. current conditions are acceptable, however. 2/12/2012
+	*/
+	heartbeat_data.enabled = false;
+	SetEvent( thread_abort_event );
+	if( thread_handle )
+	{
+		WaitForSingleObject( thread_handle, INFINITE );
+		CloseHandle( thread_handle );
+	}
 
-  TRACE_EXIT();
+	if( thread_abort_event )
+		CloseHandle( thread_abort_event );
+
+	if( heartbeat_data_event )
+		CloseHandle( heartbeat_data_event );
+
+	DeleteCriticalSection(&heartbeat_data_lock);
+
+	delete timer_box_control;
+
+	TRACE_EXIT();
 }
 
 
@@ -127,19 +151,6 @@ W32AppletWindow::set_time_bar(BreakId id,
   TRACE_EXIT();
 }
 
-void
-W32AppletWindow::set_tip(std::string tip)
-{
-  (void) tip;
-}
-
-void
-W32AppletWindow::set_icon(IconType type)
-{
-  (void) type;
-}
-
-
 void
 W32AppletWindow::update_view()
 {
@@ -150,15 +161,15 @@ W32AppletWindow::update_view()
   {
     memcpy(&local_heartbeat_data, &heartbeat_data, sizeof(AppletHeartbeatData));
 
-    HWND window = get_applet_window();
+    update_applet_window();
 
     if (!menu_sent)
       {
         memcpy(&local_menu_data, &menu_data, sizeof(AppletMenuData));
-        local_applet_window = window;
+        local_applet_window = applet_window;
         menu_sent = true;
       }
-          
+
     SetEvent(heartbeat_data_event);
     ::LeaveCriticalSection(&heartbeat_data_lock);
   }
@@ -203,51 +214,77 @@ W32AppletWindow::update_time_bars()
   TRACE_EXIT();
 }
 
-HWND
-W32AppletWindow::get_applet_window()
+void
+W32AppletWindow::update_applet_window()
 {
   TRACE_ENTER("W32AppletWindow::get_applet_window");
+  HWND previous_applet_window = applet_window;
   if (applet_window == NULL || !IsWindow(applet_window))
     {
       HWND taskbar = FindWindow("Shell_TrayWnd",NULL);
       applet_window = RecursiveFindWindow(taskbar, APPLET_WINDOW_CLASS_NAME);
       menu_sent = false;
     }
-  TRACE_RETURN((applet_window ? "Applet found" : "Applet not found"));
-  return applet_window;
-}
 
-
-void
-W32AppletWindow::set_enabled(bool enabled)
-{
-  TRACE_ENTER_MSG("W32AppletWindow::set_enabled", enabled);
-  heartbeat_data.enabled = enabled;
-
-  if (enabled)
+  if (previous_applet_window == NULL && applet_window != NULL)
     {
-      thread_id = 0;
-      thread_handle = CreateThread(NULL, 0, run_event_pipe_static, this, 0, (DWORD *)&thread_id);
-  
-      if (thread_handle == NULL || thread_id == 0)
-        {
-          TRACE_MSG( "Thread could not be created. GetLastError : " << GetLastError());
-          TRACE_EXIT();
-        }
-
-      CloseHandle(thread_handle);
-      thread_handle = NULL;
+      state_changed_signal.emit(AppletWindow::APPLET_STATE_VISIBLE);
     }
-
+  else if (previous_applet_window != NULL && applet_window == NULL)
+    {
+      state_changed_signal.emit(AppletWindow::APPLET_STATE_DISABLED);
+    }
+  
   TRACE_EXIT();
 }
 
 
-DWORD WINAPI
-W32AppletWindow::run_event_pipe_static(LPVOID lpParam)
+void
+W32AppletWindow::set_enabled( bool enabled )
 {
-  W32AppletWindow *pThis = (W32AppletWindow *) lpParam;
+	TRACE_ENTER_MSG( "W32AppletWindow::set_enabled", enabled );
+	DWORD thread_exit_code = 0;
+
+	heartbeat_data.enabled = enabled;
+
+	if( !enabled )
+		return;
+
+	if( thread_id 
+		&& thread_handle 
+		&& GetExitCodeThread( thread_handle, &thread_exit_code ) 
+		&& ( thread_exit_code == STILL_ACTIVE ) 
+	)
+		return;
+
+	if( !thread_id )
+	{
+		// if there is no id but a handle then this instance's worker thread has exited or is exiting.
+		if( thread_handle )
+			CloseHandle( thread_handle );
+
+		thread_id = 0;
+		SetLastError( 0 );
+		thread_handle = 
+			(HANDLE)_beginthreadex( NULL, 0, run_event_pipe_static, this, 0, (unsigned int *)&thread_id );
+
+		if( !thread_handle || !thread_id )
+		{
+			TRACE_MSG( "Thread could not be created. GetLastError : " << GetLastError() );
+		}
+	}
+
+	TRACE_EXIT();
+}
+
+
+unsigned __stdcall
+W32AppletWindow::run_event_pipe_static( void *param )
+{
+  W32AppletWindow *pThis = (W32AppletWindow *) param;
   pThis->run_event_pipe();
+  // invalidate the id to signal the thread is exiting
+  pThis->thread_id = 0;
   return (DWORD) 0;
 }
 
@@ -255,37 +292,53 @@ W32AppletWindow::run_event_pipe_static(LPVOID lpParam)
 void
 W32AppletWindow::run_event_pipe()
 {
-  while (heartbeat_data.enabled)
-    {
-      DWORD wait_result = ::WaitForSingleObjectEx(heartbeat_data_event,
-                                                  INFINITE,
-                                                  FALSE);
-      if (wait_result != WAIT_FAILED)
-        {
-          ::EnterCriticalSection(&heartbeat_data_lock);
+	const DWORD current_thread_id = GetCurrentThreadId();
 
-          update_time_bars();
-          update_menu();
+	TRACE_ENTER_MSG( "W32AppletWindow::run_event_pipe [ id: ", current_thread_id << " ]" );
 
-          ::LeaveCriticalSection(&heartbeat_data_lock);
-        }
-    }
+	while( thread_id == current_thread_id )
+	{
+		/* JS: thread_abort_event must be first in the array of events. 
+		the index returned by WaitForMultipleObjectsEx() corresponds to the first 
+		signaled event in the array if more than one is signaled
+		*/
+		HANDLE events[ 2 ] = { thread_abort_event, heartbeat_data_event };
+		int const events_count = ( sizeof( events ) / sizeof( events[ 0 ] ) );
+
+		DWORD wait_result = WaitForMultipleObjectsEx( events_count, events, FALSE, INFINITE, FALSE );
+
+		if( ( wait_result == WAIT_FAILED ) || ( wait_result == ( WAIT_OBJECT_0 + 0 ) ) )
+			break;
+		
+		if( heartbeat_data.enabled && ( wait_result == ( WAIT_OBJECT_0 + 1 ) ) )
+		{
+			EnterCriticalSection( &heartbeat_data_lock );
+
+			update_time_bars();
+			update_menu();
+
+			LeaveCriticalSection( &heartbeat_data_lock );
+		}
+	}
+
+	TRACE_EXIT();
 }
 
-    
+
 void
 W32AppletWindow::init_menu(HWND hwnd)
 {
   menu_data.num_items = 0;
   menu_sent = false;
-  
+
   /*
-    As noted in frontend/win32/applet/include/applet.hh: 
+    As noted in frontend/win32/applet/include/applet.hh:
     We pass the command_window HWND as a LONG for compatibility.
   */
   menu_data.command_window = HandleToLong( hwnd );
 }
 
+
 void
 W32AppletWindow::add_menu(const char *text, short cmd, int flags)
 {
@@ -295,10 +348,12 @@ W32AppletWindow::add_menu(const char *text, short cmd, int flags)
   d->flags = flags;
 }
 
+
 AppletWindow::AppletState
 W32AppletWindow::activate_applet()
 {
-  return APPLET_STATE_VISIBLE;
+  update_applet_window();
+  return applet_window != NULL ? APPLET_STATE_VISIBLE : APPLET_STATE_DISABLED;
 }
 
 
@@ -320,7 +375,8 @@ bool
 W32AppletWindow::on_applet_command(int command)
 {
   TRACE_ENTER_MSG("W32AppletWindow::on_applet_command", command);
-  Menus *menus = Menus::get_instance();
+  IGUI *gui = GUI::get_instance();
+  Menus *menus = gui->get_menus();
   menus->applet_command(command);
   TRACE_EXIT();
   return false;
