@@ -67,7 +67,7 @@ W32StatusIcon::set(const Glib::RefPtr<Gdk::Pixbuf> &pixbuf)
 
   HICON old_hicon = nid.hIcon;
   int size = 16;
-  
+
   if (width > size || height > size)
     {
       scaled = pixbuf->scale_simple(MIN (size, width),
@@ -79,7 +79,7 @@ W32StatusIcon::set(const Glib::RefPtr<Gdk::Pixbuf> &pixbuf)
     {
       nid.hIcon = pixbuf_to_hicon(pixbuf->gobj());
     }
-  
+
   nid.uFlags |= NIF_ICON;
   if (nid.hWnd != NULL && visible)
     {
@@ -109,7 +109,7 @@ W32StatusIcon::set_tooltip(const Glib::ustring &text)
     {
       nid.szTip[0] = 0;
     }
-  
+
   if (nid.hWnd != NULL && visible)
     {
       Shell_NotifyIconW(NIM_MODIFY, &nid);
@@ -118,12 +118,14 @@ W32StatusIcon::set_tooltip(const Glib::ustring &text)
 
 
 void
-W32StatusIcon::show_balloon(const Glib::ustring &balloon)
+W32StatusIcon::show_balloon(string id, const Glib::ustring &balloon)
 {
   TRACE_ENTER("W32StatusIcon::show_balloon");
   gunichar2 *winfo = g_utf8_to_utf16(balloon.c_str(), -1, NULL, NULL, NULL);
   gunichar2 *wtitle = g_utf8_to_utf16("Workrave", -1, NULL, NULL, NULL);
 
+  current_id = id;
+  
   if (winfo != NULL && wtitle != NULL)
     {
       nid.uFlags |= NIF_INFO;
@@ -184,12 +186,6 @@ W32StatusIcon::is_embedded() const
   return true;
 }
 
-sigc::signal<bool, int> &
-W32StatusIcon::signal_size_changed()
-{
-    return size_changed_signal;
-}
-
 sigc::signal<void>
 W32StatusIcon::signal_activate()
 {
@@ -202,6 +198,12 @@ W32StatusIcon::signal_popup_menu()
   return popup_menu_signal;
 }
 
+sigc::signal<void, string>
+W32StatusIcon::signal_balloon_activate()
+{
+  return balloon_activate_signal;
+}
+
 void
 W32StatusIcon::init()
 {
@@ -214,7 +216,7 @@ W32StatusIcon::init()
       wclass.lpszClassName = "WorkraveTrayObserver";
       wclass.lpfnWndProc = window_proc;
       wclass.hInstance = hinstance;
-      
+
       ATOM atom = RegisterClass(&wclass);
       if (atom != 0)
         {
@@ -242,7 +244,7 @@ W32StatusIcon::init()
   nid.hWnd = tray_hwnd;
 
   set_tooltip("Workrave");
-  
+
   if (tray_hwnd != NULL)
     {
       SetWindowLong(tray_hwnd, GWL_USERDATA, (LONG) this);
@@ -267,7 +269,7 @@ void
 W32StatusIcon::add_tray_icon()
 {
   memset(&nid, 0, sizeof(NOTIFYICONDATA));
-    
+
   nid.cbSize = sizeof(NOTIFYICONDATA);
   nid.hWnd = tray_hwnd;
   nid.uID = 1;
@@ -301,17 +303,19 @@ W32StatusIcon::window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
             case WM_LBUTTONDOWN:
               status_icon->activate_signal.emit();
               break;
+            case NIN_BALLOONUSERCLICK:
+              status_icon->balloon_activate_signal.emit(status_icon->current_id);
             }
         }
     }
-  
+
   TRACE_EXIT();
   return DefWindowProc(hwnd, uMsg, wParam, lParam);
 }
 
 
 
-/* This is ganked from GTK+. 
+/* This is ganked from GTK+.
 
  * GDK - The GIMP Drawing Kit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
@@ -322,7 +326,7 @@ W32StatusIcon::window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  */
- 
+
 static gboolean
 _gdk_win32_pixbuf_to_hicon_supports_alpha (void)
 {
