@@ -1,6 +1,6 @@
 // StatisticsDialog.cc --- Statistics dialog
 //
-// Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Rob Caelers <robc@krandor.nl>
+// Copyright (C) 2002 - 2013 Rob Caelers <robc@krandor.nl>
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -41,12 +41,7 @@
 #include <sstream>
 #include <stdio.h>
 
-#include <gtkmm/notebook.h>
-#include <gtkmm/calendar.h>
-#include <gtkmm/box.h>
-#include <gtkmm/stock.h>
-#include <gtkmm/label.h>
-#include <gtkmm/separator.h>
+#include <gtkmm.h>
 
 #include "debug.hh"
 #include "nls.h"
@@ -72,7 +67,7 @@ StatisticsDialog::StatisticsDialog()
     {
       activity_labels[i] = NULL;
     }
-  
+
   init_gui();
   display_calendar_date();
 }
@@ -104,14 +99,20 @@ StatisticsDialog::init_gui()
 #else
   Gtk::HBox *tnotebook= Gtk::manage(new Gtk::HBox(false, 6));
 #endif
-  
+
   // Calendar
   calendar = Gtk::manage(new Gtk::Calendar());
   calendar->signal_month_changed().connect(sigc::mem_fun(*this, &StatisticsDialog::on_calendar_month_changed));
   calendar->signal_day_selected().connect(sigc::mem_fun(*this, &StatisticsDialog::on_calendar_day_selected));
+#ifdef HAVE_GTK3
+  calendar->set_display_options(Gtk::CALENDAR_SHOW_WEEK_NUMBERS
+                                |Gtk::CALENDAR_SHOW_DAY_NAMES
+                                |Gtk::CALENDAR_SHOW_HEADING);
+#else
   calendar->display_options(Gtk::CALENDAR_SHOW_WEEK_NUMBERS
                             |Gtk::CALENDAR_SHOW_DAY_NAMES
                             |Gtk::CALENDAR_SHOW_HEADING);
+#endif
 
   // Button box.
   Gtk::HBox *btnbox= Gtk::manage(new Gtk::HBox(false, 6));
@@ -143,14 +144,19 @@ StatisticsDialog::init_gui()
   // Navigation box
   HigCategoryPanel *browsebox
     = Gtk::manage(new HigCategoryPanel(_("Browse history")));
-  browsebox->add(*btnbox);
-  browsebox->add(*calendar);
+  browsebox->add_widget(*btnbox);
+  browsebox->add_widget(*calendar);
+
+  // Delete button
+  delete_btn = Gtk::manage( GtkUtil::create_custom_stock_button( _("Delete all statistics history"), Gtk::Stock::DELETE ) );
+  delete_btn->signal_clicked().connect( sigc::mem_fun( *this, &StatisticsDialog::on_history_delete_all ) );
+  browsebox->add_widget( *delete_btn );
 
   // Stats box
   HigCategoriesPanel *navbox = Gtk::manage(new HigCategoriesPanel());
   HigCategoryPanel *statbox = Gtk::manage(new HigCategoryPanel(_("Statistics")));
-  statbox->add(_("Date:"), *date_label);
-  statbox->add(*tnotebook);
+  statbox->add_label(_("Date:"), *date_label);
+  statbox->add_widget(*tnotebook);
   navbox->add(*statbox);
 
   Gtk::HBox *hbox = Gtk::manage(new Gtk::HBox(false, 12));
@@ -168,7 +174,7 @@ StatisticsDialog::init_gui()
 #if !defined(PLATFORM_OS_OSX)
   tnotebook->set_current_page(0);
 #endif
-  
+
   get_vbox()->pack_start(*hbox, true, true, 0);
 
   // Dialog
@@ -287,9 +293,13 @@ StatisticsDialog::create_break_page(Gtk::Widget *tnotebook)
     }
 
   box->show_all();
-  
+
 #if !defined(PLATFORM_OS_OSX)
+  #ifdef HAVE_GTK3
+  ((Gtk::Notebook *)tnotebook)->append_page(*table, *box);
+  #else
   ((Gtk::Notebook *)tnotebook)->pages().push_back(Gtk::Notebook_Helpers::TabElem(*table, *box));
+  #endif
 #else
   ((Gtk::HBox *)tnotebook)->pack_start(*table, true, true, 0);
 #endif
@@ -345,7 +355,12 @@ StatisticsDialog::create_activity_page(Gtk::Widget *tnotebook)
     }
 
   box->show_all();
+#ifdef HAVE_GTK3
+  ((Gtk::Notebook *)tnotebook)->append_page(*table, *box);
+#else
   ((Gtk::Notebook *)tnotebook)->pages().push_back(Gtk::Notebook_Helpers::TabElem(*table, *box));
+#endif
+
 }
 
 
@@ -432,7 +447,7 @@ StatisticsDialog::display_statistics(IStatistics::DailyStats *stats)
   if (activity_labels[0] != NULL)
     {
       // Label not available is OS X
-      
+
       value = stats->misc_stats[IStatistics::STATS_VALUE_TOTAL_MOVEMENT_TIME];
       if (value > 24 * 60 * 60) {
         value = 0;
@@ -508,11 +523,9 @@ StatisticsDialog::get_calendar_day_index(int &idx, int &next, int &prev)
 void
 StatisticsDialog::set_calendar_day_index(int idx)
 {
-  calendar->freeze();
   IStatistics::DailyStats *stats = statistics->get_day(idx);
   calendar->select_month(stats->start.tm_mon, stats->start.tm_year+1900);
   calendar->select_day(stats->start.tm_mday);
-  calendar->thaw();
   display_calendar_date();
 }
 
@@ -572,6 +585,62 @@ StatisticsDialog::on_history_goto_first()
 }
 
 
+void 
+StatisticsDialog::on_history_delete_all()
+{
+    /* Modal dialogs interrupt GUI input. That can be a problem if for example a break is 
+    triggered while the message boxes are shown. The user would have no way to interact 
+    with the break window without closing out the dialog which may be hidden behind it.
+    Temporarily override operation mode to avoid catastrophe, and remove the 
+    override before any return.
+    */
+    const char funcname[] = "StatisticsDialog::on_history_delete_all";
+    CoreFactory::get_core()->set_operation_mode_override( OPERATION_MODE_SUSPENDED, funcname );
+
+    // Confirm the user's intention
+    string msg = HigUtil::create_alert_text( 
+        _("Warning"), 
+        _("You have chosen to delete your statistics history. Continue?")
+        );
+    Gtk::MessageDialog mb_ask( *this, msg, true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO, false );
+    mb_ask.set_title( _("Warning") );
+    mb_ask.get_widget_for_response( Gtk::RESPONSE_NO )->grab_default();
+    if( mb_ask.run() == Gtk::RESPONSE_YES )
+    {
+        mb_ask.hide();
+
+        // Try to delete statistics history files
+        for( ;; )
+        {
+            if( statistics->delete_all_history() )
+            {
+                msg = HigUtil::create_alert_text( 
+                    _("Files deleted!"), 
+                    _("The files containing your statistics history have been deleted.")
+                    );
+                Gtk::MessageDialog mb_info( *this, msg, true, Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK, false );
+                mb_info.set_title( _("Info") );
+                mb_info.run();
+                break;
+            }
+
+            msg = HigUtil::create_alert_text( 
+                _("File deletion failed!"), 
+                _("The files containing your statistics history could not be deleted. Try again?")
+                );
+            Gtk::MessageDialog mb_error( *this, msg, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_YES_NO, false );
+            mb_error.set_title( _("Error") );
+            mb_error.get_widget_for_response( Gtk::RESPONSE_NO )->grab_default();
+            if( mb_error.run() != Gtk::RESPONSE_YES )
+                break;
+        }
+    }
+
+    // Remove this function's operation mode override
+    CoreFactory::get_core()->remove_operation_mode_override( funcname );
+}
+
+
 //! Periodic heartbeat.
 bool
 StatisticsDialog::on_timer()
